@@ -21,10 +21,10 @@ import os
 from pathlib import Path
 
 # NOTE: All required Python package versions are in setup.cfg
-LATEST_PROJECT_VERSION = 8
+LATEST_PROJECT_VERSION = 10
 
 # For the Add/Update SpineOpt wizard
-REQUIRED_SPINE_OPT_VERSION = "0.5.3"
+REQUIRED_SPINE_OPT_VERSION = "0.6.9"
 
 # Invalid characters for directory names
 # NOTE: "." is actually valid in a directory name but this is
@@ -47,7 +47,7 @@ ONLINE_DOCUMENTATION_URL = "https://spine-toolbox.readthedocs.io/en/master/"
 
 PLUGINS_PATH = os.path.abspath(os.path.join(str(Path.home()), ".spinetoolbox", "plugins"))
 
-PLUGIN_REGISTRY_URL = "https://spine-project.github.io/PluginRegistry/registry.json"
+PLUGIN_REGISTRY_URL = "https://spine-tools.github.io/PluginRegistry/registry.json"
 # Jupyter kernel constants
 JUPYTER_KERNEL_TIME_TO_DEAD = 8.0
 
@@ -92,9 +92,8 @@ SETTINGS_SS = (
     "QSlider::sub-page:horizontal{background: transparent;}"
 )
 
-ICON_BACKGROUND = "qlineargradient(x1: 1, y1: 1, x2: 0, y2: 0, stop: 0 #cce0ff, stop: 1 #66a1ff);"
-
-ICON_TOOLBAR_SS = f"QToolBar{{spacing: 6px; background: {ICON_BACKGROUND}; padding: 3px; border-style: solid;}}"
+# ICON_BACKGROUND = "qlineargradient(x1: 1, y1: 1, x2: 0, y2: 0, stop: 0 #cce0ff, stop: 1 #66a1ff);"
+# ICON_TOOLBAR_SS = f"QToolBar{{spacing: 6px; background: {ICON_BACKGROUND}; padding: 3px; border-style: solid;}}"
 
 TEXTBROWSER_SS = (
     f"QTextBrowser {{background-color: {BG_COLOR}; border: 1px solid #32414B; color: {FG_COLOR}; border-radius: 2px;}}"
@@ -116,7 +115,7 @@ MAINWINDOW_SS = (
     "QToolButton:pressed{background-color: #f2f2f2;}"
     "QToolButton::menu-indicator{width: 0px;}"
     "QCheckBox{padding: 2px; spacing: 10px; outline-style: dashed; outline-width: 1px; outline-color: black;}"
-    "QComboBox:focus{border-color: black; border-width: 1px; border-style: ridge;}"
+    "QComboBox QAbstractItemView{border: 1px solid darkgray; selection-background-color: lightgray;}"
     "QLineEdit:focus{border-color: black; border-width: 1px; border-style: ridge;}"
     "QTextEdit:focus{border-color: black; border-width: 1px; border-style: ridge;}"
     "QTreeView:focus{border-color: darkslategray; border-width: 2px; border-style: ridge;}"
