@@ -15,8 +15,8 @@ General helper functions and classes.
 :authors: P. Savolainen (VTT)
 :date:   10.1.2018
 """
-
-from enum import Enum, unique, auto
+import time
+from enum import Enum, unique
 import itertools
 import os
 import glob
@@ -29,12 +29,13 @@ import re
 import pathlib
 import bisect
 from contextlib import contextmanager
+
 import matplotlib
-from PySide2.QtCore import Qt, Slot, QFile, QIODevice, QSize, QRect, QPoint, QUrl, QObject, QEvent
-from PySide2.QtCore import __version__ as qt_version
-from PySide2.QtCore import __version_info__ as qt_version_info
-from PySide2.QtWidgets import QApplication, QMessageBox, QFileIconProvider, QStyle, QFileDialog, QInputDialog
-from PySide2.QtGui import (
+from PySide6.QtCore import Qt, Slot, QFile, QIODevice, QSize, QRect, QPoint, QUrl, QObject, QEvent
+from PySide6.QtCore import __version__ as qt_version
+from PySide6.QtCore import __version_info__ as qt_version_info
+from PySide6.QtWidgets import QApplication, QMessageBox, QFileIconProvider, QStyle, QFileDialog, QInputDialog
+from PySide6.QtGui import (
     QGuiApplication,
     QCursor,
     QImageReader,
@@ -161,11 +162,15 @@ def rename_dir(old_dir, new_dir, toolbox, box_title):
     if os.path.exists(new_dir):
         msg = "Directory <b>{0}</b> already exists.<br/><br/>Would you like to overwrite its contents?".format(new_dir)
         box = QMessageBox(
-            QMessageBox.Question, box_title, msg, buttons=QMessageBox.Ok | QMessageBox.Cancel, parent=toolbox
+            QMessageBox.Icon.Question,
+            box_title,
+            msg,
+            buttons=QMessageBox.StandardButton.Ok | QMessageBox.StandardButton.Cancel,
+            parent=toolbox,
         )
-        box.button(QMessageBox.Ok).setText("Overwrite")
-        answer = box.exec_()
-        if answer != QMessageBox.Ok:
+        box.button(QMessageBox.StandardButton.Ok).setText("Overwrite")
+        answer = box.exec()
+        if answer != QMessageBox.StandardButton.Ok:
             return False
         shutil.rmtree(new_dir)
     try:
@@ -238,23 +243,20 @@ def supported_img_formats():
     logging.debug("Supported Image formats:\n%s", img_formats_str)
 
 
-def pyside2_version_check():
-    """Check that PySide2 version is 5.14 or 5.15.
-    Version 5.15 is allowed but it is not promoted yet
-    because user's may need to update their VC++ runtime
-    libraries on Windows.
+def pyside6_version_check():
+    """Check that PySide6 version is at least 6.4.
 
-    qt_version is the Qt version used to compile PySide2 as string. E.g. "5.14.2"
-    qt_version_info is a tuple with each version component of Qt used to compile PySide2. E.g. (5, 14, 2)
+    qt_version (str) is the Qt version used to compile PySide6. E.g. "6.4.1"
+    qt_version_info (tuple) contains each version component separately e.g. (6, 4, 1)
     """
-    if not (qt_version_info[0] == 5 and qt_version_info[1] in (14, 15)):
+    if not (qt_version_info[0] == 6 and qt_version_info[1] >= 4):
         print(
             f"""Sorry for the inconvenience but,
 
-            Spine Toolbox does not support PySide2 version {qt_version}.
-            At the moment, supported PySide2 versions are 5.14 & 5.15.
+            Spine Toolbox does not support PySide6 version {qt_version}.
+            At the moment, PySide6 version must be 6.4 or greater.
 
-            To upgrade PySide2 to latest supported version, run
+            To upgrade PySide6 to latest supported version, run
 
                 pip install -r requirements.txt --upgrade
 
@@ -451,8 +453,8 @@ class IconListManager:
         for codepoint, searchterms in self.searchterms.items():
             item = QStandardItem()
             display_icon = int(codepoint, 16)
-            item.setData(display_icon, Qt.UserRole)
-            item.setData(searchterms, Qt.UserRole + 1)
+            item.setData(display_icon, Qt.ItemDataRole.UserRole)
+            item.setData(searchterms, Qt.ItemDataRole.UserRole + 1)
             items.append(item)
         self.model.invisibleRootItem().appendRows(items)
 
@@ -466,11 +468,11 @@ class IconListManager:
         Returns:
             Any: role-dependent model data
         """
-        if role == Qt.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole:
             return None
-        if role != Qt.DecorationRole:
+        if role != Qt.ItemDataRole.DecorationRole:
             return QStandardItemModel.data(self.model, index, role)
-        display_icon = index.data(Qt.UserRole)
+        display_icon = index.data(Qt.ItemDataRole.UserRole)
         return object_icon(display_icon)
 
 
@@ -1032,7 +1034,7 @@ def unique_name(prefix, existing):
     reserved = set()
     for name in existing:
         if pattern.fullmatch(name) is not None:
-            _, _, number = name.partition(" ")
+            _, _, number = name.rpartition(" ")
             reserved.add(int(number))
     free = len(reserved) + 1
     for i in range(len(reserved)):
@@ -1249,25 +1251,37 @@ def parameter_identifier(database, parameter, names, alternative):
 class SignalWaiter(QObject):
     """A 'traffic light' that allows waiting for a signal to be emitted in another thread."""
 
-    def __init__(self):
+    def __init__(self, condition=None, timeout=None):
+        """
+        Args:
+            condition (function, optional): receiving the self.args and returning whether to stop waiting.
+            timeout (float, optional): timeout in seconds; wait will raise after timeout
+        """
         super().__init__()
         self._triggered = False
         self.args = ()
+        self._condition = condition
+        self._timeout = timeout
+        self._start = time.monotonic() if self._timeout is not None else None
 
     def trigger(self, *args):
         """Signal receiving slot."""
-        self._triggered = True
+        if self._triggered:
+            return
+        self._triggered = True if self._condition is None else self._condition(*args)
         self.args = args
 
     def wait(self):
         """Wait for signal to be received."""
         while not self._triggered:
             QApplication.processEvents()
+            if self._timeout is not None and time.monotonic() - self._start > self._timeout:
+                raise RuntimeError("timeout exceeded")
 
 
 @contextmanager
-def signal_waiter(signal):
-    waiter = SignalWaiter()
+def signal_waiter(signal, condition=None, timeout=None):
+    waiter = SignalWaiter(condition=condition, timeout=timeout)
     signal.connect(waiter.trigger)
     try:
         yield waiter
@@ -1328,12 +1342,12 @@ def inquire_index_name(model, column, title, parent_widget):
         title (str): input dialog's title
         parent_widget (QWidget): dialog's parent widget
     """
-    index_name = model.headerData(column, Qt.Horizontal)
+    index_name = model.headerData(column, Qt.Orientation.Horizontal)
     dialog_flags = Qt.Dialog | Qt.CustomizeWindowHint | Qt.WindowTitleHint | Qt.WindowCloseButtonHint
     new_name, ok = QInputDialog.getText(parent_widget, title, "Index name:", text=index_name, flags=dialog_flags)
     if not ok:
         return
-    model.setHeaderData(column, Qt.Horizontal, new_name)
+    model.setHeaderData(column, Qt.Orientation.Horizontal, new_name)
 
 
 def preferred_row_height(widget, factor=1.5):
@@ -1389,6 +1403,16 @@ def save_ui(window, app_settings, settings_group):
 
 
 def bisect_chunks(current_data, new_data, key=None):
+    """Finds insertion points for chunks of data using binary search.
+
+    Args:
+        current_data (list): sorted list where to insert new data
+        new_data (list): data to insert
+        key (Callable, optional): sort key
+
+    Returns:
+        tuple: sorted chunk of new data, insertion position
+    """
     if key is not None:
         current_data = [key(x) for x in current_data]
     else:
@@ -1411,97 +1435,6 @@ def bisect_chunks(current_data, new_data, key=None):
     yield chunk, lo
 
 
-class FetchParent:
-    @unique
-    class Init(Enum):
-        UNINITIALIZED = auto()
-        IN_PROGRESS = auto()
-        FINISHED = auto()
-        FAILED = auto()
-
-    fetch_token = None
-    _is_fetched = False
-    _is_busy_fetching = False
-    query = None
-    query_key = None
-    query_iterator = None
-    query_initialized = Init.UNINITIALIZED
-
-    @property
-    def fetch_item_type(self):
-        """Returns the type of item to fetch, e.g., "object_class".
-        Used to create an initial query for this item.
-
-        Returns:
-            str
-        """
-        raise NotImplementedError()
-
-    # pylint: disable=no-self-use
-    def filter_query(self, query, subquery, db_map):
-        """Filters the initial query created using the ``fetch_item_type`` property.
-
-        Args:
-            query (Query): The query
-            subquery (Alias): The source of the query
-            db_map (DiffDatabaseMapping)
-
-        Returns:
-            Query
-        """
-        return query
-
-    def fetch_status_change(self):
-        """Called when fetch status changes."""
-
-    @property
-    def is_fetched(self):
-        return self._is_fetched
-
-    def set_fetched(self, fetched):
-        """Sets the fetched status.
-
-        Args:
-            fetched (bool): whether parent has been fetched completely
-        """
-        self._is_fetched = fetched
-
-    @property
-    def is_busy_fetching(self):
-        return self._is_busy_fetching
-
-    def set_busy_fetching(self, busy):
-        """Sets the busy status.
-
-        Args:
-            busy (bool): whether the parent is busy
-        """
-        self._is_busy_fetching = busy
-
-    def reset_fetching(self, fetch_token):
-        """Resets fetch parent as if nothing was ever fetched.
-
-        Args:
-            fetch_token (object): current fetch token
-        """
-        self.fetch_token = fetch_token
-        self._is_fetched = False
-        self.query = None
-        self.query_iterator = None
-        self.query_initialized = FetchParent.Init.UNINITIALIZED
-        self.fetch_status_change()
-
-
-class ItemTypeFetchParent(FetchParent):
-    def __init__(self, fetch_item_type):
-        super().__init__()
-        self._fetch_item_type = fetch_item_type
-
-    @property
-    def fetch_item_type(self):
-        return self._fetch_item_type
-
-
 def load_project_dict(project_config_dir, logger):
     """Loads project dictionary from project directory.
 
