@@ -18,8 +18,8 @@ Classes for drawing graphics items on QGraphicsScene.
 
 import functools
 from math import sin, cos, pi, radians
-from PySide2.QtCore import Qt, Slot, QPointF, QLineF, QRectF, QVariantAnimation
-from PySide2.QtWidgets import (
+from PySide6.QtCore import Qt, Slot, QPointF, QLineF, QRectF, QVariantAnimation
+from PySide6.QtWidgets import (
     QGraphicsItem,
     QGraphicsPathItem,
     QGraphicsTextItem,
@@ -28,8 +28,9 @@ from PySide2.QtWidgets import (
     QToolTip,
     QGraphicsColorizeEffect,
 )
-from PySide2.QtGui import QColor, QPen, QBrush, QPainterPath, QLinearGradient, QFont, QCursor, QPainterPathStroker
-from PySide2.QtSvg import QGraphicsSvgItem, QSvgRenderer
+from PySide6.QtGui import QColor, QPen, QBrush, QPainterPath, QLinearGradient, QFont, QCursor, QPainterPathStroker
+from PySide6.QtSvgWidgets import QGraphicsSvgItem
+from PySide6.QtSvg import QSvgRenderer
 from spinetoolbox.helpers import color_from_index
 from .project_item_icon import ConnectorButton
 
@@ -316,8 +317,6 @@ class _SvgIcon(_IconBase):
 class _TextIcon(_IconBase):
     """A font awesome icon to show over a Link."""
 
-    FONT_SIZE_PIXELS = 16  # Using pixel size to prevent font scaling by system.
-
     def __init__(self, parent, extent, char, tooltip=None, active=False):
         super().__init__(0, 0, extent, extent, parent, tooltip=tooltip, active=active)
         self._text_item = QGraphicsTextItem(self)
@@ -334,6 +333,15 @@ class _TextIcon(_IconBase):
         self.scene().removeItem(self)
 
 
+class _WarningTextIcon(_TextIcon):
+    """A font awesome icon to show over a Link."""
+
+    def __init__(self, parent, extent, char, tooltip):
+        super().__init__(parent, extent, char, tooltip, active=True)
+        self._fg_color = QColor("red")
+        self._text_item.setDefaultTextColor(self._fg_color)
+
+
 class JumpOrLink(LinkBase):
     """Base class for Jump and Link."""
 
@@ -394,8 +402,8 @@ class JumpOrLink(LinkBase):
 
     def paint(self, painter, option, widget=None):
         """Sets a dashed pen if selected."""
-        if option.state & QStyle.State_Selected:
-            option.state &= ~QStyle.State_Selected
+        if option.state & QStyle.StateFlag.State_Selected:
+            option.state &= ~QStyle.StateFlag.State_Selected
             self._outline.setPen(self.selected_pen)
             for icon in self._icons:
                 icon.setPen(self.selected_pen)
@@ -460,6 +468,7 @@ class Link(JumpOrLink):
     _MEMORY = "\uf538"
     _FILTERS = "\uf0b0"
     _PURGE = "\uf0e7"
+    _WARNING = "\uf06a"
     _DATAPACKAGE = ":/icons/datapkg.svg"
 
     def __init__(self, toolbox, src_connector, dst_connector, connection):
@@ -493,6 +502,10 @@ class Link(JumpOrLink):
             sibling_conns = self._toolbox.project().incoming_connections(self.connection.destination)
             active = any(l.write_index > 1 for l in sibling_conns)
             self._icons.append(_TextIcon(self, self._icon_extent, str(self._connection.write_index), active=active))
+        notifications = self._connection.notifications()
+        if notifications:
+            tooltip = "Check Link properties. " + " ".join(notifications)
+            self._icons.append(_WarningTextIcon(self, self._icon_extent, self._WARNING, tooltip))
         self._place_icons()
 
     @property
