@@ -17,9 +17,9 @@ Provides the main() function.
 """
 import os
 import multiprocessing
-import PySide2
+import PySide6
 
-dirname = os.path.dirname(PySide2.__file__)
+dirname = os.path.dirname(PySide6.__file__)
 plugin_path = os.path.join(dirname, "plugins", "platforms")
 os.environ["QT_QPA_PLATFORM_PLUGIN_PATH"] = plugin_path
 
@@ -27,8 +27,8 @@ os.environ["QT_QPA_PLATFORM_PLUGIN_PATH"] = plugin_path
 from argparse import ArgumentParser
 import sys
 import logging
-from PySide2.QtGui import QFontDatabase
-from PySide2.QtWidgets import QApplication
+from PySide6.QtGui import QFontDatabase
+from PySide6.QtWidgets import QApplication
 
 # Importing resources_icons_rc initializes resources and Font Awesome gets added to the application
 from . import resources_icons_rc  # pylint: disable=unused-import
@@ -37,7 +37,7 @@ from spine_items import resources_icons_rc  # pylint: disable=unused-import
 from .ui_main import ToolboxUI
 from .version import __version__
 from .headless import headless_main, Status
-from .helpers import pyside2_version_check
+from .helpers import pyside6_version_check
 
 
 def main():
@@ -49,7 +49,7 @@ def main():
         format='%(asctime)s %(levelname)s: %(message)s',
         datefmt='%Y-%m-%d %H:%M:%S',
     )
-    if not pyside2_version_check():
+    if not pyside6_version_check():
         return 1
     _add_pywin32_system32_to_path()
     parser = _make_argument_parser()
@@ -68,7 +68,7 @@ def main():
     window.show()
     window.init_project(args.project)
     # Enter main event loop and wait until exit() is called
-    return_code = app.exec_()
+    return_code = app.exec()
     return return_code
 
 
@@ -105,7 +105,7 @@ def _make_argument_parser():
 
 def _add_pywin32_system32_to_path():
     """Adds a directory to PATH on Windows that is required to make pywin32 work
-    on (Conda) Python 3.8. See https://github.com/Spine-project/Spine-Toolbox/issues/1230."""
+    on (Conda) Python 3.8. See https://github.com/spine-tools/Spine-Toolbox/issues/1230."""
     if not sys.platform == "win32":
         return
     if sys.version_info[0:2] == (3, 8):
