@@ -17,7 +17,7 @@ Models that vertically concatenate two or more table models.
 """
 
 import bisect
-from PySide2.QtCore import Qt, Signal, Slot, QModelIndex, QTimer
+from PySide6.QtCore import Qt, Signal, Slot, QModelIndex, QTimer
 from ..mvcmodels.minimal_table_model import MinimalTableModel
 
 
@@ -114,20 +114,16 @@ class CompoundTableModel(MinimalTableModel):
         self.layoutAboutToBeChanged.emit()
         self._do_refresh()
         self.layoutChanged.emit()
+        if self.canFetchMore(QModelIndex()):
+            self.fetchMore(QModelIndex())
 
     def _do_refresh(self):
         """Recomputes the row and inverse row maps."""
         self._row_map.clear()
         self._inv_row_map.clear()
-        useful_sub_models = []
         for model in self.sub_models:
-            if model.rowCount() == 0:
-                model.deleteLater()
-                continue
-            useful_sub_models.append(model)
             row_map = self._row_map_for_model(model)
             self._append_row_map(row_map)
-        self.sub_models = useful_sub_models
         self.refreshed.emit()
 
     def _append_row_map(self, row_map):
@@ -182,7 +178,7 @@ class CompoundTableModel(MinimalTableModel):
     def flags(self, index):
         return self.map_to_sub(index).flags()
 
-    def data(self, index, role=Qt.DisplayRole):
+    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
         return self.map_to_sub(index).data(role)
 
     def rowCount(self, parent=QModelIndex()):
@@ -339,12 +335,17 @@ class CompoundWithEmptyTableModel(CompoundTableModel):
         row_map = self._row_map_for_model(model)
         if not row_map:
             return
-        first = self._inv_row_map[row_map[0]]
+        try:
+            first = self._inv_row_map[row_map[0]]
+        except KeyError:
+            # Sometimes the submodel may get reset before it has been added to the inverted row map.
+            # In this case there are no rows to remove, so we can bail out here.
+            return
         last = first + len(row_map) - 1
         tail_row_map = self._row_map[last + 1 :]
         self.beginRemoveRows(QModelIndex(), first, last)
         for key in self._row_map[first:]:
-            self._inv_row_map.pop(key)
+            del self._inv_row_map[key]
         self._row_map[first:] = []
         self._append_row_map(tail_row_map)
         self.endRemoveRows()
@@ -374,7 +375,12 @@ class CompoundWithEmptyTableModel(CompoundTableModel):
         for model in self.sub_models[pos:]:
             first_row_map_item = next(self._row_map_iterator_for_model(model), None)
             if first_row_map_item is not None:
-                return self._inv_row_map[first_row_map_item]
+                try:
+                    return self._inv_row_map[first_row_map_item]
+                except KeyError:
+                    # Sometimes the submodel is not yet in the inverted row map.
+                    # In this case we just skip it and try another insertion point.
+                    pass
         return self.rowCount()
 
     def _insert_row_map(self, pos, single_row_map):
