@@ -15,7 +15,7 @@ Models to represent items in a tree.
 :authors: P. Vennström (VTT), M. Marin (KTH)
 :date:   11.3.2019
 """
-from PySide2.QtCore import Qt, QAbstractItemModel, QModelIndex
+from PySide6.QtCore import Qt, QAbstractItemModel, QModelIndex
 
 
 class TreeItem:
@@ -31,10 +31,10 @@ class TreeItem:
         self._model = model
         self._parent_item = None
         self._fetched = False
-        self._finalized = False
+        self._set_up_once = False
 
     def has_children(self):
-        """Returns whether or not this item has or could have children."""
+        """Returns whether this item has or could have children."""
         if self.can_fetch_more():
             return True
         return bool(self.child_count())
@@ -72,6 +72,7 @@ class TreeItem:
         self._parent_item = parent_item
         if parent_item is not None:
             self._model = parent_item.model
+            self._model.destroyed.connect(lambda obj=None: self.tear_down())
 
     def child(self, row):
         """Returns the child at given row or None if out of bounds."""
@@ -117,13 +118,13 @@ class TreeItem:
     def index(self):
         return self.model.index_from_item(self)
 
-    def finalize(self):
-        if not self._finalized:
-            self._do_finalize()
-            self._finalized = True
+    def set_up(self):
+        if not self._set_up_once:
+            self._set_up_once = True
+            self._do_set_up()
 
-    def _do_finalize(self):
-        """Do some final initialization after setting the parent."""
+    def _do_set_up(self):
+        """Do stuff after the item has been inserted."""
 
     def insert_children(self, position, children):
         """Insert new children at given position. Returns a boolean depending on how it went.
@@ -131,25 +132,37 @@ class TreeItem:
         Args:
             position (int): insert new items here
             children (list of TreeItem): insert items from this iterable
+
+        Returns:
+            bool: True if the children were inserted successfully, False otherwise
         """
         bad_types = [type(child) for child in children if not isinstance(child, TreeItem)]
         if bad_types:
             raise TypeError(f"Can't insert children of type {bad_types} to an item of type {type(self)}")
         if position < 0 or position > self.child_count():
             return False
+        parent_index = self.index()
+        self.model.beginInsertRows(parent_index, position, position + len(children) - 1)
         for child in children:
             child.parent_item = self
-        self.model.beginInsertRows(self.index(), position, position + len(children) - 1)
         self.children[position:position] = children
         self.model.endInsertRows()
         for child in children:
-            child.finalize()
+            child.set_up()
         return True
 
     def append_children(self, children):
         """Append children at the end."""
         return self.insert_children(self.child_count(), children)
 
+    def tear_down(self):
+        """Do stuff after the item has been removed."""
+
+    def tear_down_recursively(self):
+        for child in self.children:
+            child.tear_down_recursively()
+        self.tear_down()
+
     def remove_children(self, position, count):
         """Removes count children starting from the given position.
 
@@ -166,11 +179,14 @@ class TreeItem:
             return False
         if last >= self.child_count():
             last = self.child_count() - 1
+        children = self.children[first : last + 1]
         self.model.beginRemoveRows(self.index(), first, last)
-        for child in self.children[first : last + 1]:
+        for child in children:
             child.parent_item = None
         del self.children[first : last + 1]
         self.model.endRemoveRows()
+        for child in children:
+            child.tear_down_recursively()
         return True
 
     def clear_children(self):
@@ -183,12 +199,12 @@ class TreeItem:
         return Qt.ItemIsSelectable | Qt.ItemIsEnabled
 
     # pylint: disable=no-self-use
-    def data(self, column, role=Qt.DisplayRole):
+    def data(self, column, role=Qt.ItemDataRole.DisplayRole):
         """Returns data for given column and role."""
         return None
 
     def can_fetch_more(self):
-        """Returns whether or not this item can fetch more."""
+        """Returns whether this item can fetch more."""
         return not self._fetched
 
     def fetch_more(self):
@@ -280,8 +296,18 @@ class MinimalTreeModel(QAbstractItemModel):
         return self._invisible_root_item
 
     def index_from_item(self, item):
-        """Return a model index corresponding to the given item."""
-        return self.createIndex(item.child_number(), 0, item)
+        """Return a model index corresponding to the given item.
+
+        Args:
+            item (StandardTreeItem): item
+
+        Returns:
+            QModelIndex: item's index
+        """
+        row = item.child_number()
+        if row < 0:
+            return QModelIndex()
+        return self.createIndex(row, 0, item)
 
     def index(self, row, column, parent=QModelIndex()):
         """Returns the index of the item in the model specified by the given row, column and parent index."""
@@ -297,7 +323,7 @@ class MinimalTreeModel(QAbstractItemModel):
             return QModelIndex()
         item = self.item_from_index(index)
         parent_item = item.parent_item
-        if parent_item is None or parent_item == self._invisible_root_item:
+        if parent_item is None or parent_item is self._invisible_root_item:
             return QModelIndex()
         return self.createIndex(parent_item.child_number(), 0, parent_item)
 
@@ -310,14 +336,14 @@ class MinimalTreeModel(QAbstractItemModel):
         parent_item = self.item_from_index(parent)
         return parent_item.child_count()
 
-    def data(self, index, role=Qt.DisplayRole):
+    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
         """Returns the data stored under the given role for the index."""
         if not index.isValid():
             return None
         item = self.item_from_index(index)
         return item.data(index.column(), role)
 
-    def setData(self, index, value, role=Qt.EditRole):
+    def setData(self, index, value, role=Qt.ItemDataRole.EditRole):
         """Sets data for given index and role.
         Returns True if successful; otherwise returns False.
         """
