@@ -15,30 +15,30 @@ Contains ResourceFilterModel.
 :author: M. Marin (KTH)
 :date:   26.11.2020
 """
-from PySide2.QtCore import Qt, Signal
-from PySide2.QtGui import QStandardItemModel, QStandardItem
-
+from PySide6.QtCore import Qt, Signal
+from PySide6.QtGui import QStandardItemModel, QStandardItem
 from spinedb_api.filters.scenario_filter import SCENARIO_FILTER_TYPE
 from spinedb_api.filters.tool_filter import TOOL_FILTER_TYPE
 from ..project_commands import SetFiltersOnlineCommand
 
 
 class ResourceFilterModel(QStandardItemModel):
-
     tree_built = Signal()
     _SELECT_ALL = "Select all"
     _FILTER_TYPES = {"Scenario filter": SCENARIO_FILTER_TYPE, "Tool filter": TOOL_FILTER_TYPE}
     _FILTER_TYPE_TO_TEXT = dict(zip(_FILTER_TYPES.values(), _FILTER_TYPES.keys()))
 
-    def __init__(self, connection, undo_stack, logger):
+    def __init__(self, connection, project, undo_stack, logger):
         """
         Args:
             connection (LoggingConnection): connection whose resources to model
+            project (SpineToolboxProject): project
             undo_stack (QUndoStack): an undo stack
             logger (LoggerInterface): a logger
         """
         super().__init__()
         self._connection = connection
+        self._project = project
         self._undo_stack = undo_stack
         self._logger = logger
 
@@ -49,10 +49,12 @@ class ResourceFilterModel(QStandardItemModel):
     def build_tree(self):
         """Rebuilds model's contents."""
 
-        def append_filter_items(parent_item, filters, filter_type, disabled):
-            for name in filters[filter_type]:
+        def append_filter_items(parent_item, filter_names, filter_type, online, online_default):
+            for name in filter_names[filter_type]:
                 filter_item = QStandardItem(name)
-                filter_item.setData(Qt.Checked if name not in disabled else Qt.Unchecked, Qt.CheckStateRole)
+                filter_item.setCheckState(
+                    Qt.CheckState.Checked if online.get(name, online_default) else Qt.CheckState.Unchecked
+                )
                 filter_item.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsUserCheckable)
                 parent_item.appendRow(filter_item)
 
@@ -73,12 +75,14 @@ class ResourceFilterModel(QStandardItemModel):
                     continue
                 filter_parent.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable)
                 select_all_item = QStandardItem(self._SELECT_ALL)
-                select_all_item.setData(False, Qt.CheckStateRole)
                 select_all_item.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsUserCheckable)
+                select_all_item.setCheckState(Qt.CheckState.Unchecked)
                 filter_parent.appendRow(select_all_item)
                 root_item.appendRow(filter_parent)
-                disabled_names = self._connection.disabled_filter_names(resource_label, type_)
-                append_filter_items(filter_parent, filters_by_type, type_, disabled_names)
+                online_filters = self._connection.online_filters(resource_label, type_)
+                append_filter_items(
+                    filter_parent, filters_by_type, type_, online_filters, self._connection.is_filter_online_by_default
+                )
                 self._set_all_selected_item(resource_label, filter_parent)
         self.tree_built.emit()
 
@@ -96,10 +100,10 @@ class ResourceFilterModel(QStandardItemModel):
                 filters.setdefault(resource.label, {})[TOOL_FILTER_TYPE] = tool_names
         return filters
 
-    def setData(self, index, value, role=Qt.EditRole):
-        if role != Qt.CheckStateRole:
+    def setData(self, index, value, role=Qt.ItemDataRole.EditRole):
+        if role != Qt.ItemDataRole.CheckStateRole:
             return super().setData(index, value, role)
-        self._change_filter_checked_state(index, value == Qt.Checked)
+        self._change_filter_checked_state(index, value == Qt.CheckState.Checked.value)
         return True
 
     def _change_filter_checked_state(self, index, is_on):
@@ -118,9 +122,11 @@ class ResourceFilterModel(QStandardItemModel):
         resource_label = root_item.text()
         if item.text() == self._SELECT_ALL:
             activated = {resource_type_item.child(row).text(): is_on for row in range(1, resource_type_item.rowCount())}
-            cmd = SetFiltersOnlineCommand(self, resource_label, filter_type, activated)
+            cmd = SetFiltersOnlineCommand(self._project, self.connection, resource_label, filter_type, activated)
         else:
-            cmd = SetFiltersOnlineCommand(self, resource_label, filter_type, {item.text(): is_on})
+            cmd = SetFiltersOnlineCommand(
+                self._project, self.connection, resource_label, filter_type, {item.text(): is_on}
+            )
         self._undo_stack.push(cmd)
 
     def set_online(self, resource, filter_type, online):
@@ -138,10 +144,10 @@ class ResourceFilterModel(QStandardItemModel):
             filter_item = filter_type_item.child(row)
             is_on = online.get(filter_item.text(), None)
             if is_on is not None:
-                checked = Qt.Checked if is_on else Qt.Unchecked
-                if filter_item.data(Qt.CheckStateRole) != checked:
-                    filter_item.setData(checked, Qt.CheckStateRole)
-                    self.dataChanged.emit(filter_item.index(), filter_item.index(), [Qt.CheckStateRole])
+                checked = Qt.CheckState.Checked if is_on else Qt.CheckState.Unchecked
+                if filter_item.data(Qt.ItemDataRole.CheckStateRole) != checked.value:
+                    filter_item.setCheckState(checked)
+                    self.dataChanged.emit(filter_item.index(), filter_item.index(), [Qt.ItemDataRole.CheckStateRole])
         self._set_all_selected_item(resource, filter_type_item, True)
 
     def _find_filter_type_item(self, resource, filter_type):
@@ -171,17 +177,14 @@ class ResourceFilterModel(QStandardItemModel):
             filter_type_item (QStandardItem): filter type item
             emit_data_changed (bool): if True, emit dataChanged signal if the state was updated
         """
-        disabled_filters = self._connection.disabled_filter_names(resource, self._FILTER_TYPES[filter_type_item.text()])
-        all_online = True
-        if disabled_filters:
-            for row in range(1, filter_type_item.rowCount()):
-                if filter_type_item.child(row).text() in disabled_filters:
-                    all_online = False
-                    break
+        online_filters = self._connection.online_filters(resource, self._FILTER_TYPES[filter_type_item.text()])
+        all_online = all(online_filters.values())
         all_selected_item = filter_type_item.child(0)
-        all_selected = all_selected_item.data(Qt.CheckStateRole) == Qt.Checked
+        all_selected = all_selected_item.data(Qt.ItemDataRole.CheckStateRole) == Qt.CheckState.Checked.value
         if all_selected != all_online:
-            checked = Qt.Checked if all_online else Qt.Unchecked
-            all_selected_item.setData(checked, Qt.CheckStateRole)
+            checked = Qt.CheckState.Checked if all_online else Qt.CheckState.Unchecked
+            all_selected_item.setCheckState(checked)
             if emit_data_changed:
-                self.dataChanged.emit(all_selected_item.index(), all_selected_item.index(), [Qt.CheckStateRole])
+                self.dataChanged.emit(
+                    all_selected_item.index(), all_selected_item.index(), [Qt.ItemDataRole.CheckStateRole]
+                )
