@@ -15,55 +15,51 @@ Functions for plotting on PlotWidget.
 :author: A. Soininen (VTT)
 :date:   9.7.2019
 """
+import datetime
+from enum import auto, Enum, unique
 import math
 from contextlib import contextmanager
 from dataclasses import dataclass, field, replace
 import functools
-from numbers import Number
 from operator import methodcaller, itemgetter
 from typing import Dict, List, Optional, Union
 
+from matplotlib.container import BarContainer
+from matplotlib.patches import Patch
 from matplotlib.ticker import MaxNLocator
 import numpy as np
-from PySide2.QtCore import Qt, QModelIndex
+from PySide6.QtCore import Qt, QModelIndex
 
 from spinedb_api.parameter_value import NUMPY_DATETIME64_UNIT, from_database
-from spinedb_api import (
-    Array,
-    convert_leaf_maps_to_specialized_containers,
-    IndexedValue,
-    Map,
-    ParameterValueFormatError,
-    TimeSeries,
-    DateTime,
-)
-from .helpers import first_non_null
+from spinedb_api import Array, IndexedValue, TimeSeries, DateTime
 from .mvcmodels.shared import PARSED_ROLE
+from .widgets.plot_canvas import LegendPosition
 from .widgets.plot_widget import PlotWidget
 
 
+LEGEND_PLACEMENT_THRESHOLD = 8
+
+
+@unique
+class PlotType(Enum):
+    SCATTER = auto()
+    SCATTER_LINE = auto()
+    LINE = auto()
+    STACKED_LINE = auto()
+    BAR = auto()
+    STACKED_BAR = auto()
+
+
 _BASE_SETTINGS = {"alpha": 0.7}
 _SCATTER_PLOT_SETTINGS = {"linestyle": "", "marker": "o"}
-_LINE_PLOT_SETTINGS = {"linestyle": "solid", "marker": "o"}
-_TIME_SERIES_PLOT_SETTINGS = {"where": "post"}
+_LINE_PLOT_SETTINGS = {"linestyle": "solid"}
+_SCATTER_LINE_PLOT_SETTINGS = dict(_SCATTER_PLOT_SETTINGS, **_LINE_PLOT_SETTINGS)
+_TIME_SERIES_PLOT_SETTINGS = dict(_LINE_PLOT_SETTINGS, where="post")
 
 
 class PlottingError(Exception):
     """An exception signalling failure in plotting."""
 
-    def __init__(self, message):
-        """
-        Args:
-            message (str): an error message
-        """
-        super().__init__()
-        self._message = message
-
-    @property
-    def message(self):
-        """str: the error message."""
-        return self._message
-
 
 @dataclass(frozen=True)
 class XYData:
@@ -94,7 +90,7 @@ class ParameterTableHeaderSection:
 
 
 def convert_indexed_value_to_tree(value):
-    """Converts Maps to tree nodes recursively.
+    """Converts indexed values to tree nodes recursively.
 
     Args:
         value (IndexedValue): value to convert
@@ -118,11 +114,12 @@ def convert_indexed_value_to_tree(value):
     return d
 
 
-def turn_nodes_to_xy_data(root_node, index_names=None, indexes=None):
+def turn_node_to_xy_data(root_node, y_label_position, index_names=None, indexes=None):
     """Constructs plottable data and indexes recursively.
 
     Args:
         root_node (TreeNode): root node
+        y_label_position (int, optional): position of y label in indexes
         index_names (list of str, optional): list of current index names
         indexes (list): list of current indexes
 
@@ -139,13 +136,13 @@ def turn_nodes_to_xy_data(root_node, index_names=None, indexes=None):
     for index, sub_node in root_node.content.items():
         if isinstance(sub_node, TreeNode):
             current_indexes = indexes + [index]
-            yield from turn_nodes_to_xy_data(sub_node, current_index_names, current_indexes)
+            yield from turn_node_to_xy_data(sub_node, y_label_position, current_index_names, current_indexes)
         else:
             x.append(index)
             y.append(sub_node)
     if x:
         x_label = current_index_names[-1]
-        y_label = ""
+        y_label = indexes[y_label_position] if y_label_position is not None else ""
         yield XYData(x, y, x_label, y_label, indexes, current_index_names[:-1])
 
 
@@ -237,18 +234,36 @@ def combine_data_with_same_indexes(data_list):
     return combined_data
 
 
-def plot_data(data_list, plot_widget=None):
+def _always_single_y_axis(plot_type):
+    """Returns True if a single y-axis should be used.
+
+    Args:
+        plot_type (PlotType): plot type
+
+    Returns:
+        bool: True if single y-axis is required, False otherwise
+    """
+    return plot_type in (PlotType.STACKED_LINE,)
+
+
+def plot_data(data_list, plot_widget=None, plot_type=None):
     """
     Returns a plot widget with plots of the given data.
 
     Args:
         data_list (list of XYData): data to plot
         plot_widget (PlotWidget, optional): an existing plot widget to draw into or None to create a new widget
+        plot_type (PlotType, optional): plot type
+
     Returns:
         a PlotWidget object
     """
     if plot_widget is None:
-        plot_widget = PlotWidget()
+        plot_widget = PlotWidget(
+            legend_axes_position=LegendPosition.BOTTOM
+            if len(data_list) < LEGEND_PLACEMENT_THRESHOLD
+            else LegendPosition.RIGHT
+        )
         needs_redraw = False
     else:
         needs_redraw = True
@@ -265,24 +280,105 @@ def plot_data(data_list, plot_widget=None):
     raise_if_incompatible_x(squeezed_data)
     if needs_redraw:
         _clear_plot(plot_widget)
+    if plot_type is None:
+        plot_type = PlotType.SCATTER_LINE if not isinstance(squeezed_data[0].x[0], np.datetime64) else PlotType.LINE
     _limit_string_x_tick_labels(squeezed_data, plot_widget)
-    plot = _make_plot_function(squeezed_data, plot_widget)
-    for data in squeezed_data:
-        plot_label = " | ".join(map(str, data.data_index))
-        x = _make_x_plottable(data.x)
-        plot(x, data.y, label=plot_label)
+    y_labels = sorted({xy_data.y_label for xy_data in data_list})
+    if len(y_labels) == 1 or _always_single_y_axis(plot_type):
+        legend_handles = _plot_single_y_axis(squeezed_data, y_labels[0], plot_widget, plot_type)
+    elif len(y_labels) == 2:
+        legend_handles = _plot_double_y_axis(squeezed_data, y_labels, plot_widget, plot_type)
+    else:
+        legend_handles = _plot_single_y_axis(squeezed_data, "", plot_widget, plot_type)
     plot_widget.canvas.axes.set_xlabel(squeezed_data[0].x_label)
-    plot_widget.canvas.axes.set_ylabel(squeezed_data[0].y_label)
     plot_title = " | ".join(map(str, common_indexes))
     plot_widget.canvas.axes.set_title(plot_title)
     if len(squeezed_data) > 1:
-        plot_widget.add_legend()
+        plot_widget.add_legend(legend_handles)
     if needs_redraw:
         plot_widget.canvas.draw()
     plot_widget.original_xy_data = all_data
     return plot_widget
 
 
+def _plot_single_y_axis(data_list, y_label, plot_widget, plot_type):
+    """Plots all data on single y-axis.
+
+    Args:
+        data_list (list of XYData): data to plot
+        y_label (str): y-axis label
+        plot_widget (PlotWidget): plot widget
+        plot_type (PlotType): plot type
+
+    Returns:
+        list: legend handles
+    """
+    if plot_type == PlotType.STACKED_LINE:
+        return _plot_stacked_line(data_list, y_label, plot_widget)
+    legend_handles = []
+    plot = _make_plot_function(plot_type, type(data_list[0].x[0]), plot_widget.canvas.axes)
+    for data in data_list:
+        plot_label = " | ".join(map(str, data.data_index))
+        x = _make_x_plottable(data.x)
+        handles = plot(x, data.y, label=plot_label)
+        if isinstance(handles, BarContainer):
+            handles = [Patch(color=handles.patches[0].get_facecolor(), label=plot_label)]
+        legend_handles += handles
+    plot_widget.canvas.axes.set_ylabel(y_label)
+    return legend_handles
+
+
+def _plot_stacked_line(data_list, y_label, plot_widget):
+    """Plots all data as stacked lines.
+
+    Args:
+        data_list (list of XYData): data to plot
+        y_label (str): y-axis label
+        plot_widget (PlotWidget): plot widget
+
+    Returns:
+        list: legend handles
+    """
+    if any(data.x != data_list[0].x for data in data_list[1:]):
+        raise PlottingError("Cannot stack plots when x-axes don't match.")
+    x = _make_x_plottable(data_list[0].x)
+    y = [data.y for data in data_list]
+    labels = [" | ".join(map(str, data.data_index)) for data in data_list]
+    handles = plot_widget.canvas.axes.stackplot(x, y, labels=labels, **_LINE_PLOT_SETTINGS, **_BASE_SETTINGS)
+    plot_widget.canvas.axes.set_ylabel(y_label)
+    return handles
+
+
+def _plot_double_y_axis(data_list, y_labels, plot_widget, plot_type):
+    """Plots all data on two y-axes.
+
+    Args:
+        data_list (list of XYData): data to plot
+        y_labels (list of str): y-axis labels
+        plot_widget (PlotWidget): plot widget
+        plot_type (PlotType): plot type
+
+    Returns:
+        list: legend handles
+    """
+    legend_handles = []
+    left_label = y_labels[0]
+    right_label = y_labels[1]
+    x_data_type = type(data_list[0].x[0])
+    plot_left = _make_plot_function(plot_type, x_data_type, plot_widget.canvas.axes)
+    right_axes = plot_widget.canvas.axes.twinx()
+    plot_right = _make_plot_function(plot_type, x_data_type, right_axes)
+    for data in data_list:
+        plot_label = " | ".join(map(str, data.data_index))
+        x = _make_x_plottable(data.x)
+        plot = plot_left if data.y_label == left_label else plot_right
+        handles = plot(x, data.y, label=plot_label)
+        legend_handles += handles
+    plot_widget.canvas.axes.set_ylabel(left_label)
+    right_axes.set_ylabel(right_label)
+    return legend_handles
+
+
 def _make_x_plottable(xs):
     """Converts x-axis values to something matplotlib can handle.
 
@@ -297,20 +393,53 @@ def _make_x_plottable(xs):
     return xs
 
 
-def _make_plot_function(data_list, plot_widget):
+class _PlotStackedBars:
+    def __init__(self, axes):
+        self._axes = axes
+        self._cumulative_height = {}
+
+    def __call__(self, x, height, **kwargs):
+        bottom = [self._cumulative_height.get(key, 0.0) for key in x]
+        for key, h in zip(x, height):
+            cumulative = self._cumulative_height.get(key, 0.0)
+            self._cumulative_height[key] = cumulative + h
+        return self._axes.bar(x, height, bottom=bottom, **_BASE_SETTINGS, **kwargs)
+
+
+def _make_plot_function(plot_type, x_data_type, axes):
     """Decides plot method and default keyword arguments based on XYData.
 
     Args:
-        data_list (list of XYData): data to plot
+        plot_type (PlotType): plot type
+        x_data_type (Type): data type of x-axis
+        axes (Axes): plot axes
 
     Returns:
         Callable: plot method
     """
-    if data_list:
-        x = data_list[0].x
-        if x and isinstance(x[0], np.datetime64):
-            return functools.partial(plot_widget.canvas.axes.step, **_TIME_SERIES_PLOT_SETTINGS, **_BASE_SETTINGS)
-    return functools.partial(plot_widget.canvas.axes.plot, **_SCATTER_PLOT_SETTINGS, **_BASE_SETTINGS)
+    if plot_type == PlotType.SCATTER:
+        return functools.partial(_plot_or_step(x_data_type, axes), **_SCATTER_PLOT_SETTINGS, **_BASE_SETTINGS)
+    if plot_type == PlotType.SCATTER_LINE:
+        return functools.partial(_plot_or_step(x_data_type, axes), **_SCATTER_LINE_PLOT_SETTINGS, **_BASE_SETTINGS)
+    if plot_type == PlotType.LINE:
+        return functools.partial(_plot_or_step(x_data_type, axes), **_LINE_PLOT_SETTINGS, **_BASE_SETTINGS)
+    if plot_type == PlotType.BAR:
+        return functools.partial(axes.bar, **_BASE_SETTINGS)
+    if plot_type == PlotType.STACKED_BAR:
+        return _PlotStackedBars(axes)
+    raise RuntimeError(f"Unknown plot type '{plot_type}'")
+
+
+def _plot_or_step(x_data_type, axes):
+    """Makes choice between Axes.plot() and Axes.step().
+
+    Args:
+        x_data_type (Type): data type of x-axis
+        axes (Axes): plot axes
+    """
+    if x_data_type in (np.datetime64, datetime.datetime, datetime.date, datetime.time):
+        return axes.step
+    return axes.plot
 
 
 def _clear_plot(plot_widget):
@@ -387,7 +516,8 @@ def plot_parameter_table_selection(model, model_indexes, table_header_sections,
             index = model.index(row, index_column).data()
             node = _set_default_node(node, index, header_data(index_columns[i + 1]))
         node.content[model.index(row, index_columns[-1]).data()] = leaf_content
-    data_list = list(turn_nodes_to_xy_data(root_node))
+    y_label_position = index_columns.index(header_columns["parameter_name"])
+    data_list = list(turn_node_to_xy_data(root_node, y_label_position))
     return plot_data(data_list, plot_widget)
 
 
@@ -406,7 +536,7 @@ def plot_value_editor_table_selection(model, model_indexes, plot_widget=None):
     model_indexes = [i for i in model_indexes if model.is_leaf_value(i)]
     if not model_indexes:
         raise PlottingError("Nothing to plot.")
-    header_columns = [model.headerData(column, Qt.Horizontal) for column in range(model.columnCount())]
+    header_columns = [model.headerData(column, Qt.Orientation.Horizontal) for column in range(model.columnCount())]
     root_node = TreeNode(header_columns[0])
     for model_index in sorted(model_indexes, key=methodcaller("row")):
         value = _get_parsed_value(model_index, _table_display_row)
@@ -420,7 +550,7 @@ def plot_value_editor_table_selection(model, model_indexes, plot_widget=None):
         for i, index in enumerate(indexes[:-1]):
             node = _set_default_node(node, index, header_columns[i + 1])
         node.content[indexes[-1]] = leaf_content
-    data_list = list(turn_nodes_to_xy_data(root_node))
+    data_list = list(turn_node_to_xy_data(root_node, None))
     return plot_data(data_list, plot_widget)
 
 
@@ -464,7 +594,7 @@ def plot_pivot_table_selection(model, model_indexes, plot_widget=None):
         for i, index in enumerate(indexes[:-1]):
             node = _set_default_node(node, index, index_names[i])
         node.content[indexes[-1]] = leaf_content
-    data_list = list(turn_nodes_to_xy_data(root_node))
+    data_list = list(turn_node_to_xy_data(root_node, 1))
     return plot_data(data_list, plot_widget)
 
 
@@ -503,7 +633,7 @@ def plot_db_mngr_items(items, db_maps, plot_widget=None):
         for i, index in enumerate(indexes[:-1]):
             node = _set_default_node(node, index, index_names[i])
         node.content[indexes[-1]] = leaf_content
-    data_list = list(turn_nodes_to_xy_data(root_node))
+    data_list = list(turn_node_to_xy_data(root_node, 1))
     return plot_data(data_list, plot_widget)
 
 
