@@ -16,22 +16,20 @@ Contains SpecificationEditorWindowBase and ChangeSpecPropertyCommand
 :date:   12.4.2018
 """
 
-from PySide2.QtGui import QKeySequence, QIcon
-from PySide2.QtCore import Slot, Qt
-from PySide2.QtWidgets import (
+from PySide6.QtGui import QKeySequence, QIcon, QUndoStack, QAction, QUndoCommand
+from PySide6.QtCore import Signal, Slot, Qt
+from PySide6.QtWidgets import (
     QMainWindow,
-    QUndoStack,
     QWidget,
     QToolBar,
     QLabel,
     QHBoxLayout,
     QMessageBox,
-    QAction,
     QMenu,
     QLineEdit,
     QCheckBox,
-    QUndoCommand,
     QErrorMessage,
+    QToolButton,
 )
 from spinetoolbox.widgets.notification import ChangeNotifier, Notification
 from spinetoolbox.helpers import CharIconEngine, restore_ui, save_ui
@@ -83,7 +81,7 @@ class SpecificationEditorWindowBase(QMainWindow):
         self._ui = self._make_ui()
         self._ui.setupUi(self)
         self._ui_error = QErrorMessage(self)
-        self._ui_error.setWindowFlag(Qt.WindowContextHelpButtonHint, False)
+        self._ui_error.setWindowFlag(Qt.WindowType.WindowContextHelpButtonHint, False)
         self._ui_error.setWindowTitle("Error")
         self.setWindowTitle(specification.name if specification else "")
         # Restore ui
@@ -95,11 +93,11 @@ class SpecificationEditorWindowBase(QMainWindow):
         # Setup toolbar
         self._spec_toolbar = _SpecNameDescriptionToolbar(self, specification, self._undo_stack)
         self._spec_toolbar.show_toolbox_action.triggered.connect(self._toolbox.restore_and_activate)
+        self._spec_toolbar.name_changed.connect(self._set_window_title)
         self.addToolBar(Qt.TopToolBarArea, self._spec_toolbar)
         self._populate_main_menu()
         self._spec_toolbar.save_action.triggered.connect(self._save)
         self._spec_toolbar.duplicate_action.triggered.connect(self._duplicate)
-        self._spec_toolbar.close_action.triggered.connect(self.close)
         self._undo_stack.cleanChanged.connect(self._update_window_modified)
 
     @property
@@ -134,6 +132,10 @@ class SpecificationEditorWindowBase(QMainWindow):
         """
         raise NotImplementedError()
 
+    def spec_toolbar(self):
+        """Returns spec editor window's toolbar, which contains e.g. the hamburger menu."""
+        return self._spec_toolbar
+
     @Slot(str)
     def show_error(self, message):
         self._ui_error.showMessage(message)
@@ -148,7 +150,9 @@ class SpecificationEditorWindowBase(QMainWindow):
         undo_action = self._undo_stack.createUndoAction(self)
         redo_action = self._undo_stack.createRedoAction(self)
         undo_action.setShortcuts(QKeySequence.Undo)
+        undo_action.setIcon(QIcon(":/icons/menu_icons/undo.svg"))
         redo_action.setShortcuts(QKeySequence.Redo)
+        redo_action.setIcon(QIcon(":/icons/menu_icons/redo.svg"))
         self._spec_toolbar.menu.insertActions(self._spec_toolbar.save_action, [redo_action, undo_action])
         self._spec_toolbar.menu.insertSeparator(self._spec_toolbar.save_action)
 
@@ -159,6 +163,16 @@ class SpecificationEditorWindowBase(QMainWindow):
         self.setWindowTitle(self._spec_toolbar.name())
         self.windowTitleChanged.emit(self.windowTitle())
 
+    @Slot(str)
+    def _set_window_title(self, title):
+        """Sets window title.
+
+        Args:
+            title (str): new window title
+        """
+        self.setWindowTitle(title)
+        self.windowTitleChanged.emit(self.windowTitle())
+
     def _save(self):
         """Saves spec.
 
@@ -228,11 +242,12 @@ class SpecificationEditorWindowBase(QMainWindow):
 
 
 class _SpecNameDescriptionToolbar(QToolBar):
-    """A QToolBar to let users set name and description for an Spec."""
+    """QToolBar for line edits and a hamburger menu."""
+
+    name_changed = Signal(str)
 
     def __init__(self, parent, spec, undo_stack):
         """
-
         Args:
             parent (QMainWindow): QMainWindow instance
             spec (ProjectItemSpecification): specification that is being edited
@@ -252,9 +267,9 @@ class _SpecNameDescriptionToolbar(QToolBar):
         self.setMovable(False)
         widget = QWidget()
         layout = QHBoxLayout(widget)
-        layout.addWidget(QLabel("Name:"))
+        layout.addWidget(QLabel("Name"))
         layout.addWidget(self._line_edit_name)
-        layout.addWidget(QLabel("Description:"))
+        layout.addWidget(QLabel("Description"))
         layout.addWidget(self._line_edit_description)
         layout.setContentsMargins(3, 3, 3, 3)
         layout.setStretchFactor(self._line_edit_name, 1)
@@ -262,7 +277,7 @@ class _SpecNameDescriptionToolbar(QToolBar):
         self.addWidget(widget)
         toolbox_icon = QIcon(":/symbols/Spine_symbol.png")
         self.show_toolbox_action = self.addAction(toolbox_icon, "Show Spine Toolbox (Ctrl+ESC)")
-        self.show_toolbox_action.setShortcut(Qt.CTRL + Qt.Key_Escape)
+        self.show_toolbox_action.setShortcut(Qt.CTRL | Qt.Key_Escape)
         self.menu = self._make_main_menu()
         self.save_action = self.menu.addAction("Save")
         self.duplicate_action = self.menu.addAction("Duplicate")
@@ -270,8 +285,11 @@ class _SpecNameDescriptionToolbar(QToolBar):
         self.save_action.setEnabled(False)
         self.duplicate_action.setEnabled(self._parent.specification is not None)
         self.save_action.setShortcut(QKeySequence.Save)
-        self.duplicate_action.setShortcut(QKeySequence(Qt.CTRL + Qt.Key_D))
+        self.save_action.setIcon(QIcon(":/icons/menu_icons/save_solid.svg"))
+        self.duplicate_action.setShortcut(QKeySequence(Qt.CTRL | Qt.Key_D))
+        self.duplicate_action.setIcon(QIcon(":/icons/menu_icons/copy.svg"))
         self.close_action.setShortcut(QKeySequence.Close)
+        self.close_action.setIcon(QIcon(":/icons/menu_icons/window-close.svg"))
         self.setObjectName("_SpecNameDescriptionToolbar")
         if spec:
             self.do_set_name(spec.name)
@@ -284,10 +302,10 @@ class _SpecNameDescriptionToolbar(QToolBar):
         menu_action = self.addAction(QIcon(CharIconEngine("\uf0c9")), "")
         menu_action.setMenu(menu)
         menu_button = self.widgetForAction(menu_action)
-        menu_button.setPopupMode(menu_button.InstantPopup)
+        menu_button.setPopupMode(QToolButton.ToolButtonPopupMode.InstantPopup)
         action = QAction(self)
         action.triggered.connect(menu_button.showMenu)
-        keys = [QKeySequence(Qt.ALT + Qt.Key_F), QKeySequence(Qt.ALT + Qt.Key_E)]
+        keys = [QKeySequence(Qt.ALT | Qt.Key_F), QKeySequence(Qt.ALT | Qt.Key_E)]
         action.setShortcuts(keys)
         self._parent.addAction(action)
         keys_str = ", ".join([key.toString() for key in keys])
@@ -318,6 +336,7 @@ class _SpecNameDescriptionToolbar(QToolBar):
     def do_set_name(self, name):
         self._current_name = name
         self._line_edit_name.setText(name)
+        self.name_changed.emit(name)
 
     def do_set_description(self, description):
         self._current_description = description
@@ -348,20 +367,22 @@ def prompt_to_save_changes(parent, settings, save_callback):
     if save_spec == 2:
         return save_callback()
     msg = QMessageBox(parent)
-    msg.setIcon(QMessageBox.Question)
+    msg.setIcon(QMessageBox.Icon.Question)
     msg.setWindowTitle(parent.windowTitle())
     msg.setText("Do you want to save your changes to the specification?")
-    msg.setStandardButtons(QMessageBox.Yes | QMessageBox.No | QMessageBox.Cancel)
+    msg.setStandardButtons(
+        QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No | QMessageBox.StandardButton.Cancel
+    )
     chkbox = QCheckBox()
     chkbox.setText("Do not ask me again")
     msg.setCheckBox(chkbox)
-    answer = msg.exec_()
-    if answer == QMessageBox.Cancel:
+    answer = msg.exec()
+    if answer == QMessageBox.StandardButton.Cancel:
         return False
     if chkbox.checkState() == 2:
         # Save preference
-        preference = "2" if answer == QMessageBox.Yes else "0"
+        preference = "2" if answer == QMessageBox.StandardButton.Yes else "0"
         settings.setValue("appSettings/saveSpecBeforeClosing", preference)
-    if answer == QMessageBox.Yes:
+    if answer == QMessageBox.StandardButton.Yes:
         return save_callback()
     return True
