@@ -21,7 +21,7 @@ import shutil
 import os
 import json
 import copy
-from PySide2.QtWidgets import QFileDialog, QMessageBox
+from PySide6.QtWidgets import QFileDialog, QMessageBox
 from spine_engine.utils.serialization import serialize_path, deserialize_path
 from .config import LATEST_PROJECT_VERSION, PROJECT_FILENAME
 from .helpers import home_dir
@@ -49,11 +49,10 @@ class ProjectUpgrader:
             dict: Latest version of the project info dictionary
         """
         v = project_dict["project"]["version"]
-        n = project_dict["project"]["name"]
         if v > LATEST_PROJECT_VERSION:
             # User is trying to load a more recent project than this version of Toolbox can handle
             self._toolbox.msg_warning.emit(
-                f"Opening project <b>{n}</b> failed. The project's version is {v}, while "
+                f"Opening project <b>{project_dir}</b> failed. The project's version is {v}, while "
                 f"this version of Spine Toolbox supports project versions up to and "
                 f"including {LATEST_PROJECT_VERSION}. To open this project, you should "
                 f"upgrade Spine Toolbox"
@@ -100,6 +99,10 @@ class ProjectUpgrader:
                 project_dict = self.upgrade_v6_to_v7(project_dict)
             elif v == 7:
                 project_dict = self.upgrade_v7_to_v8(project_dict)
+            elif v == 8:
+                project_dict = self.upgrade_v8_to_v9(project_dict)
+            elif v == 9:
+                project_dict = self.upgrade_v9_to_v10(project_dict)
             v += 1
             self._toolbox.msg_success.emit(f"Project upgraded to version {v}")
         return project_dict
@@ -192,12 +195,10 @@ class ProjectUpgrader:
                     factories[item_type].item_class().upgrade_v2_to_v3(item_name, old_item_dict, self)
                 )
             except KeyError:
-                # This happens when a Combiner is encountered.
+                # This happens when an unknown item type is encountered.
                 # Factories do not contain 'Combiner' anymore
                 if item_type == "Combiner":
                     new["items"][item_name] = old_item_dict
-                else:
-                    print(f"Some unknown item_type encountered: {item_type}")
             if item_type == "Importer":
                 mappings = old_item_dict.get("mappings")
                 # Sanitize old mappings, as we use to do in Importer.from_dict
@@ -449,6 +450,61 @@ class ProjectUpgrader:
                 conn.setdefault("options", {}).update(purge_options)
         return new
 
+    @staticmethod
+    def upgrade_v8_to_v9(old):
+        """Upgrades version 8 project dictionary to version 9.
+
+        Changes:
+            1. Remove ["project"]["name"] key
+
+        Args:
+            old (dict): Version 8 project dictionary
+
+        Returns:
+            dict: Version 9 project dictionary
+        """
+        new = copy.deepcopy(old)
+        new["project"]["version"] = 9
+        try:
+            new["project"].pop("name")
+        except KeyError:
+            pass
+        return new
+
+    @staticmethod
+    def upgrade_v9_to_v10(old):
+        """Upgrades version 9 project dictionary to version 10.
+
+        Changes:
+            1. Remove connections from Gimlets and GDXExporters
+            2. Remove Gimlet items
+
+        Args:
+            old (dict): Version 9 project dictionary
+
+        Returns:
+            dict: Version 10 project dictionary
+        """
+        new = copy.deepcopy(old)
+        new["project"]["version"] = 10
+        names_to_remove = list()  # Gimlet and GdxExporter item names
+        # Get Gimlet and GdxExporter names and remove connections
+        for name, item_dict in new["items"].items():
+            if item_dict["type"] in ["Gimlet", "GdxExporter"]:
+                names_to_remove.append(name)
+        # Get list of connections to remove
+        connections_to_remove = list()
+        for conn in new["project"]["connections"]:
+            for name_to_remove in names_to_remove:
+                if name_to_remove in conn["from"] or name_to_remove in conn["to"]:
+                    connections_to_remove.append(conn)
+        for conn_to_remove in connections_to_remove:
+            new["project"]["connections"].remove(conn_to_remove)
+        # Remove Gimlet and GdxExporter item dictionaries
+        for name in names_to_remove:
+            new["items"].pop(name)
+        return new
+
     @staticmethod
     def make_unique_importer_specification_name(importer_name, label, k):
         return f"{importer_name} - {os.path.basename(label['path'])} - {k}"
@@ -477,15 +533,15 @@ class ProjectUpgrader:
                 "\n\nWould you like to overwrite it?".format(answer)
             )
             message_box = QMessageBox(
-                QMessageBox.Question,
+                QMessageBox.Icon.Question,
                 "Overwrite?",
                 msg,
-                buttons=QMessageBox.Ok | QMessageBox.Cancel,
+                buttons=QMessageBox.StandardButton.Ok | QMessageBox.StandardButton.Cancel,
                 parent=self._toolbox,
             )
-            message_box.button(QMessageBox.Ok).setText("Overwrite")
-            msgbox_answer = message_box.exec_()
-            if msgbox_answer != QMessageBox.Ok:
+            message_box.button(QMessageBox.StandardButton.Ok).setText("Overwrite")
+            msgbox_answer = message_box.exec()
+            if msgbox_answer != QMessageBox.StandardButton.Ok:
                 return ""
         return answer  # New project directory
 
@@ -494,7 +550,9 @@ class ProjectUpgrader:
         if v == 1:
             return self.is_valid_v1(p)
         if 2 <= v <= 8:
-            return self.is_valid_v2_to_8(p, v)
+            return self.is_valid_v2_to_v8(p, v)
+        if 9 <= v <= 10:
+            return self.is_valid_v9_to_v10(p)
         raise NotImplementedError(f"No validity check available for version {v}")
 
     def is_valid_v1(self, p):
@@ -543,9 +601,9 @@ class ProjectUpgrader:
             return False
         return True
 
-    def is_valid_v2_to_8(self, p, v):
+    def is_valid_v2_to_v8(self, p, v):
         """Checks that the given project JSON dictionary contains
-        a valid version 2 to 6 Spine Toolbox project. Valid meaning, that
+        a valid version 2 to 8 Spine Toolbox project. Valid meaning, that
         it contains all required keys and values are of the correct
         type.
 
@@ -554,7 +612,7 @@ class ProjectUpgrader:
             v (int): Version
 
         Returns:
-            bool: True if project is a valid version 2 project, False if it is not
+            bool: True if project is a valid version 2 to version 8 project, False if it is not
         """
         if "project" not in p.keys():
             self._toolbox.msg_error.emit("Invalid project.json file. Key 'project' not found.")
@@ -590,6 +648,39 @@ class ProjectUpgrader:
             return False
         return True
 
+    def is_valid_v9_to_v10(self, p):
+        """Checks that the given project JSON dictionary contains
+        a valid version 9 or 10 Spine Toolbox project. Valid meaning, that
+        it contains all required keys and values are of the correct
+        type.
+
+        Args:
+            p (dict): Project information JSON
+
+        Returns:
+            bool: True if project is a valid version 9 and 10 project, False otherwise
+        """
+        if "project" not in p.keys():
+            self._toolbox.msg_error.emit("Invalid project.json file. Key 'project' not found.")
+            return False
+        if "items" not in p.keys():
+            self._toolbox.msg_error.emit("Invalid project.json file. Key 'items' not found.")
+            return False
+        required_project_keys = ["version", "description", "specifications", "connections"]
+        project = p["project"]
+        items = p["items"]
+        if not isinstance(project, dict):
+            self._toolbox.msg_error.emit("Invalid project.json file. 'project' must be a dict.")
+            return False
+        if not isinstance(items, dict):
+            self._toolbox.msg_error.emit("Invalid project.json file. 'items' must be a dict.")
+            return False
+        for req_key in required_project_keys:
+            if req_key not in project:
+                self._toolbox.msg_error.emit("Invalid project.json file. Key {0} not found.".format(req_key))
+                return False
+        return True
+
     def backup_project_file(self, project_dir, v):
         """Makes a backup copy of project.json file."""
         src = os.path.join(project_dir, ".spinetoolbox", PROJECT_FILENAME)
