@@ -15,10 +15,9 @@ Classes for drawing graphics items on graph view's QGraphicsScene.
 :authors: M. Marin (KTH), P. Savolainen (VTT)
 :date:   4.4.2018
 """
-from PySide2.QtCore import Qt, Signal, Slot, QLineF
-from PySide2.QtSvg import QGraphicsSvgItem
-from PySide2.QtWidgets import (
-    QAction,
+from PySide6.QtCore import Qt, Signal, Slot, QLineF
+from PySide6.QtSvgWidgets import QGraphicsSvgItem
+from PySide6.QtWidgets import (
     QGraphicsItem,
     QGraphicsTextItem,
     QGraphicsRectItem,
@@ -28,7 +27,7 @@ from PySide2.QtWidgets import (
     QApplication,
     QMenu,
 )
-from PySide2.QtGui import QPen, QBrush, QPainterPath, QPalette, QGuiApplication
+from PySide6.QtGui import QPen, QBrush, QPainterPath, QPalette, QGuiApplication, QAction
 from matplotlib.figure import Figure
 from matplotlib.backends.backend_qt5agg import FigureCanvas  # pylint: disable=no-name-in-module
 
@@ -215,9 +214,9 @@ class EntityItem(QGraphicsRectItem):
 
     def paint(self, painter, option, widget=None):
         """Shows or hides the selection halo."""
-        if option.state & (QStyle.State_Selected):
+        if option.state & (QStyle.StateFlag.State_Selected):
             self._paint_as_selected()
-            option.state &= ~QStyle.State_Selected
+            option.state &= ~QStyle.StateFlag.State_Selected
         else:
             self._paint_as_deselected()
         super().paint(painter, option, widget)
@@ -354,9 +353,11 @@ class RelationshipItem(EntityItem):
 
     def default_parameter_data(self):
         """Return data to put as default in a parameter table when this item is selected."""
+        if not self.db_map_ids:
+            return {}
         return dict(
             relationship_class_name=self.entity_class_name,
-            object_name_list=self.object_name_list,
+            object_name_list=DB_ITEM_SEPARATOR.join(self.object_name_list),
             database=self.first_db_map.codename,
         )
 
@@ -391,7 +392,7 @@ class RelationshipItem(EntityItem):
             return None
         return (
             f"""<html><p style="text-align:center;">{self.entity_class_name}<br>"""
-            f"""{self.object_name_list.replace(",", DB_ITEM_SEPARATOR)}<br>"""
+            f"""{DB_ITEM_SEPARATOR.join(self.object_name_list)}<br>"""
             f"""@{self.display_database}</p></html>"""
         )
 
@@ -448,6 +449,8 @@ class ObjectItem(EntityItem):
 
     def default_parameter_data(self):
         """Return data to put as default in a parameter table when this item is selected."""
+        if not self.db_map_ids:
+            return {}
         return dict(
             object_class_name=self.entity_class_name, object_name=self.entity_name, database=self.first_db_map.codename
         )
@@ -475,7 +478,7 @@ class ObjectItem(EntityItem):
             self.label_item.setPlainText(name)
             return True
         current_name = self.label_item.toPlainText()
-        self.db_map_ids = tuple(db_map_ids_by_name.get(current_name, ()))
+        self._db_map_ids = tuple(db_map_ids_by_name.get(current_name, ()))
         return False
 
     def _make_tool_tip(self):
@@ -540,7 +543,7 @@ class ObjectItem(EntityItem):
         for db_map, rel_clss in self.db_mngr.find_cascading_relationship_classes(db_map_object_class_ids).items():
             for rel_cls in rel_clss:
                 rel_cls = rel_cls.copy()
-                rel_cls["object_class_id_list"] = [int(id_) for id_ in rel_cls["object_class_id_list"].split(",")]
+                rel_cls["object_class_id_list"] = list(rel_cls["object_class_id_list"])
                 rel_cls["relationship_ids"] = relationship_ids_per_class.get((db_map, rel_cls["id"]), set())
                 self._db_map_relationship_class_lists.setdefault(rel_cls["name"], []).append((db_map, rel_cls))
 
@@ -766,10 +769,9 @@ class CrossHairsRelationshipItem(RelationshipItem):
     def refresh_icon(self):
         """Refreshes the icon."""
         obj_items = [arc_item.obj_item for arc_item in self.arc_items]
-        object_class_name_list = [
+        object_class_name_list = tuple(
             obj_item.entity_class_name for obj_item in obj_items if not isinstance(obj_item, CrossHairsItem)
-        ]
-        object_class_name_list = ",".join(object_class_name_list)
+        )
         renderer = self.db_mngr.get_icon_mngr(self.first_db_map).relationship_class_renderer(
             None, object_class_name_list
         )
