@@ -16,10 +16,11 @@ These models concatenate several 'single' models and one 'empty' model.
 :authors: M. Marin (KTH)
 :date:   28.6.2019
 """
-from PySide2.QtCore import Qt, Slot, QTimer, QModelIndex
-from PySide2.QtGui import QFont
+from PySide6.QtCore import Qt, Slot, QTimer, QModelIndex
+from PySide6.QtGui import QFont
 from spinedb_api.parameter_value import join_value_and_type
-from ...helpers import rows_to_row_count_tuples, parameter_identifier, FetchParent
+from ...helpers import rows_to_row_count_tuples, parameter_identifier
+from ...fetch_parent import FlexibleFetchParent
 from ..widgets.custom_menus import ParameterViewFilterMenu
 from ...mvcmodels.compound_table_model import CompoundWithEmptyTableModel
 from .empty_parameter_models import (
@@ -36,14 +37,13 @@ from .single_parameter_models import (
 )
 
 
-class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
+class CompoundParameterModel(CompoundWithEmptyTableModel):
     """A model that concatenates several single parameter models
     and one empty parameter model.
     """
 
     def __init__(self, parent, db_mngr, *db_maps):
-        """Initializes model.
-
+        """
         Args:
             parent (SpineDBEditor): the parent object
             db_mngr (SpineDBManager): the database manager
@@ -55,27 +55,36 @@ class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
         self.db_maps = db_maps
         self._filter_class_ids = {}
         self._auto_filter_menus = {}
-        self._auto_filter = {}  # Maps field to db map, to entity id, to *accepted* item ids
+        self._auto_filter = {}
         self._filter_timer = QTimer(self)
         self._filter_timer.setSingleShot(True)
         self._filter_timer.setInterval(100)
         self._filter_timer.timeout.connect(self.refresh)
-
-    @property
-    def fetch_item_type(self):
-        return self.item_type
+        self._fetch_parent = FlexibleFetchParent(
+            self.item_type,
+            accepts_item=self.accepts_item,
+            shows_item=self.shows_item,
+            handle_items_added=self.handle_items_added,
+            handle_items_removed=self.handle_items_removed,
+            handle_items_updated=self.handle_items_updated,
+            owner=self,
+        )
 
     def canFetchMore(self, _parent):
-        return any(self.db_mngr.can_fetch_more(db_map, self) for db_map in self.db_maps)
+        result = False
+        for db_map in self.db_maps:
+            result |= self.db_mngr.can_fetch_more(db_map, self._fetch_parent)
+        return result
 
     def fetchMore(self, _parent):
         for db_map in self.db_maps:
-            self.db_mngr.fetch_more(db_map, self)
+            self.db_mngr.fetch_more(db_map, self._fetch_parent)
 
-    def filter_query(self, query, subquery, db_map):
-        return query.filter(getattr(subquery.c, self.entity_class_id_key).isnot(None)).order_by(
-            subquery.c.entity_class_name
-        )
+    def accepts_item(self, item, db_map):
+        return item.get(self.entity_class_id_key) is not None
+
+    def shows_item(self, item, db_map):
+        return any(m.db_map == db_map and m.filter_accepts_item(item) for m in self.accepted_single_models())
 
     def _make_header(self):
         raise NotImplementedError()
@@ -158,15 +167,9 @@ class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
         self._filter_class_ids = {}
         self._auto_filter = {}
         self.empty_model.fetchMore(QModelIndex())
-        self._make_auto_filter_menus()
-
-    def _make_auto_filter_menus(self):
-        """Makes auto filter menus."""
-        self._auto_filter_menus.clear()
-        for field in self.header:
-            # TODO: show_empty=True
-            self._auto_filter_menus[field] = menu = ParameterViewFilterMenu(self._parent, self, field, show_empty=False)
-            menu.filterChanged.connect(self.set_auto_filter)
+        while self._auto_filter_menus:
+            _, menu = self._auto_filter_menus.popitem()
+            menu.wipe_out()
 
     def get_auto_filter_menu(self, logical_index):
         """Returns auto filter menu for given logical index from header view.
@@ -177,35 +180,29 @@ class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
         Returns:
             ParameterViewFilterMenu
         """
-        return self._auto_filter_menus.get(self.header[logical_index], None)
+        return self._make_auto_filter_menu(self.header[logical_index])
 
-    def _modify_data_in_filter_menus(self, action, db_map, db_items):
-        """Modifies data in filter menus.
+    def _make_auto_filter_menu(self, field):
+        if field not in self._auto_filter_menus:
+            self._auto_filter_menus[field] = menu = ParameterViewFilterMenu(
+                self._parent,
+                self.db_mngr,
+                self.db_maps,
+                self.item_type,
+                self.entity_class_id_key,
+                field,
+                show_empty=False,
+            )
+            menu.filterChanged.connect(self.set_auto_filter)
+        return self._auto_filter_menus[field]
 
-        Args:
-            action (str): either 'add', 'remove', or 'update'
-            db_map (DiffDatabaseMapping)
-            db_items (list(dict))
-        """
-        for menu in self._auto_filter_menus.values():
-            menu.modify_menu_data(action, db_map, db_items)
-
-    def _do_add_data_to_filter_menus(self, db_map, db_items):
-        self._modify_data_in_filter_menus("add", db_map, db_items)
-
-    def _do_update_data_in_filter_menus(self, db_map, db_items):
-        self._modify_data_in_filter_menus("update", db_map, db_items)
-
-    def _do_remove_data_from_filter_menus(self, db_map, db_items):
-        self._modify_data_in_filter_menus("remove", db_map, db_items)
-
-    def headerData(self, section, orientation=Qt.Horizontal, role=Qt.DisplayRole):
+    def headerData(self, section, orientation=Qt.Orientation.Horizontal, role=Qt.ItemDataRole.DisplayRole):
         """Returns an italic font in case the given column has an autofilter installed."""
         italic_font = QFont()
         italic_font.setItalic(True)
         if (
-            role == Qt.FontRole
-            and orientation == Qt.Horizontal
+            role == Qt.ItemDataRole.FontRole
+            and orientation == Qt.Orientation.Horizontal
             and self._auto_filter.get(self.header[section], {}) != {}
         ):
             return italic_font
@@ -220,7 +217,7 @@ class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
         return self._empty_model_type(self, self.header, self.db_mngr)
 
     def filter_accepts_model(self, model):
-        """Returns a boolean indicating whether or not the given model passes the filter for compound model.
+        """Returns a boolean indicating whether the given model passes the filter for compound model.
 
         Args:
             model (SingleParameterModel, EmptyParameterModel)
@@ -244,12 +241,13 @@ class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
     def _auto_filter_accepts_model(self, model):
         if None in self._auto_filter.values():
             return False
-        for auto_filter in self._auto_filter.values():
-            if not auto_filter:
+        for values in self._auto_filter.values():
+            if not values:
                 continue
-            if model.db_map not in auto_filter:
-                return False
-            if model.entity_class_id not in auto_filter[model.db_map]:
+            for db_map, entity_class_id in values:
+                if model.db_map == db_map and (entity_class_id is None or model.entity_class_id == entity_class_id):
+                    break
+            else:  # nobreak
                 return False
         return True
 
@@ -279,13 +277,13 @@ class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
         self._auto_filter = {}
         self._invalidate_filter()
 
-    @Slot(str, dict)
+    @Slot(str, object)
     def set_auto_filter(self, field, values):
         """Updates and applies the auto filter.
 
         Args:
             field (str): the field name
-            values (dict): mapping db_map to entity_class id to accepted values for the field
+            values (dict): mapping (db_map, entity_class_id) to set of valid values
         """
         self._set_compound_auto_filter(field, values)
         for model in self.accepted_single_models():
@@ -296,7 +294,7 @@ class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
 
         Args:
             field (str): the field name
-            values (dict): maps tuple (database map, entity_class id) to list of accepted ids for the field
+            values (set): set of valid (db_map, item_type, id) tuples
         """
         if self._auto_filter.setdefault(field, {}) == values:
             return
@@ -313,7 +311,7 @@ class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
         Returns:
             bool: True if the auto-filtered values were updated, None otherwise
         """
-        values = self._auto_filter[field].get(model.db_map, {}).get(model.entity_class_id, {})
+        values = self._auto_filter[field].get((model.db_map, model.entity_class_id), set())
         if model.set_auto_filter(field, values):
             self._invalidate_filter()
 
@@ -343,23 +341,6 @@ class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
         """
         return [m for m in self.single_models if m.db_map == db_map]
 
-    def receive_entity_classes_removed(self, db_map_data):
-        """Runs when entity classes are removed from the dbs.
-        Removes sub-models for the given entity classes and dbs.
-
-        Args:
-            db_map_data (dict): list of removed dict-items keyed by DiffDatabaseMapping
-        """
-        self.layoutAboutToBeChanged.emit()
-        for db_map, data in db_map_data.items():
-            ids = {x["id"] for x in data}
-            for model in self._models_with_db_map(db_map):
-                if model.entity_class_id in ids:
-                    i = self.sub_models.index(model)
-                    self.sub_models.pop(i).deleteLater()
-        self._do_refresh()
-        self.layoutChanged.emit()
-
     def _items_per_class(self, items):
         """Returns a dict mapping entity_class ids to a set of items.
 
@@ -377,7 +358,7 @@ class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
             d.setdefault(entity_class_id, list()).append(item)
         return d
 
-    def receive_parameter_data_added(self, db_map_data):
+    def handle_items_added(self, db_map_data):
         """Runs when either parameter definitions or values are added to the dbs.
         Adds necessary sub-models and initializes them with data.
         Also notifies the empty model so it can remove rows that are already in.
@@ -393,7 +374,7 @@ class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
                 ids_committed = list()
                 ids_uncommitted = list()
                 for item in class_items:
-                    is_committed = item.get("commit_id") is not None
+                    is_committed = db_map.commit_id() is None or item["commit_id"] != db_map.commit_id()
                     item_id = item["id"]
                     if item_id in existing_ids:
                         continue
@@ -403,8 +384,7 @@ class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
                         ids_uncommitted.append(item_id)
                 self._add_parameter_data(db_map, entity_class_id, ids_committed, committed=True)
                 self._add_parameter_data(db_map, entity_class_id, ids_uncommitted, committed=False)
-                self._do_add_data_to_filter_menus(db_map, class_items)
-        self.empty_model.receive_parameter_data_added(db_map_data)
+        self.empty_model.handle_items_added(db_map_data)
 
     def _get_insert_position(self, model):
         if model.committed:
@@ -439,22 +419,18 @@ class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
         model = self._create_single_model(db_map, entity_class_id, committed)
         model.reset_model(ids)
 
-    def receive_parameter_data_updated(self, db_map_data):
+    def handle_items_updated(self, db_map_data):
         """Runs when either parameter definitions or values are updated in the dbs.
         Emits dataChanged so the parameter_name column is refreshed.
 
         Args:
             db_map_data (dict): list of updated dict-items keyed by DiffDatabaseMapping
         """
-        for db_map, items in db_map_data.items():
-            items_per_class = self._items_per_class(items)
-            for class_items in items_per_class.values():
-                self._do_update_data_in_filter_menus(db_map, class_items)
         self._emit_data_changed_for_column("parameter_name")
         # NOTE: parameter_definition names aren't refreshed unless we emit dataChanged,
         # whereas entity and class names are. Why?
 
-    def receive_parameter_data_removed(self, db_map_data):
+    def handle_items_removed(self, db_map_data):
         """Runs when either parameter definitions or values are removed from the dbs.
         Removes the affected rows from the corresponding single models.
 
@@ -471,8 +447,6 @@ class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
                 removed_rows = [row for row in range(model.rowCount()) if model._main_data[row] in removed_ids]
                 for row, count in sorted(rows_to_row_count_tuples(removed_rows), reverse=True):
                     del model._main_data[row : row + count]
-            for class_items in items_per_class.values():
-                self._do_remove_data_from_filter_menus(db_map, class_items)
         self._do_refresh()
         self.layoutChanged.emit()
 
@@ -486,7 +460,9 @@ class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
             column = self.header.index(field)
         except ValueError:
             return
-        self.dataChanged.emit(self.index(0, column), self.index(self.rowCount() - 1, column), [Qt.DisplayRole])
+        self.dataChanged.emit(
+            self.index(0, column), self.index(self.rowCount() - 1, column), [Qt.ItemDataRole.DisplayRole]
+        )
 
     def db_item(self, index):
         sub_index = self.map_to_sub(index)
@@ -519,7 +495,8 @@ class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
             },
             "parameter_value": {"object_class": "object_name", "relationship_class": "object_name_list"},
         }[self.item_type][self.entity_class_type]
-        names = item[name_key].split(",")
+        name = item[name_key]
+        names = [name] if not isinstance(name, tuple) else list(name)
         alternative_name = {"parameter_definition": lambda x: None, "parameter_value": lambda x: x["alternative_name"]}[
             self.item_type
         ](item)
@@ -555,18 +532,16 @@ class CompoundParameterModel(FetchParent, CompoundWithEmptyTableModel):
     def filter_by(self, rows_per_column):
         for column, rows in rows_per_column.items():
             field = self.headerData(column)
-            menu = self._auto_filter_menus[field]
-            accepted_values = {self.index(row, column).data(Qt.EditRole) for row in rows}
+            menu = self._make_auto_filter_menu(field)
+            accepted_values = {self.index(row, column).data(Qt.ItemDataRole.DisplayRole) for row in rows}
             menu.set_filter_accepted_values(accepted_values)
-            menu._filter._apply_filter()
 
     def filter_excluding(self, rows_per_column):
         for column, rows in rows_per_column.items():
             field = self.headerData(column)
-            menu = self._auto_filter_menus[field]
-            rejected_values = {self.index(row, column).data(Qt.EditRole) for row in rows}
+            menu = self._make_auto_filter_menu(field)
+            rejected_values = {self.index(row, column).data(Qt.ItemDataRole.DisplayRole) for row in rows}
             menu.set_filter_rejected_values(rejected_values)
-            menu._filter._apply_filter()
 
 
 class CompoundObjectParameterMixin:
@@ -638,14 +613,6 @@ class CompoundParameterValueMixin:
         model.set_filter_alternative_ids(self._filter_alternative_ids)
         return model
 
-    def receive_alternatives_updated(self, db_map_data):
-        """Updated alternative column
-
-        Args:
-            db_map_data (dict): list of updated dict-items keyed by DiffDatabaseMapping
-        """
-        self._emit_data_changed_for_column("alternative_id")
-
 
 class CompoundObjectParameterDefinitionModel(
     CompoundObjectParameterMixin, CompoundParameterDefinitionMixin, CompoundParameterModel
