@@ -15,7 +15,7 @@ Empty models for parameter definitions and values.
 :authors: M. Marin (KTH)
 :date:   28.6.2019
 """
-from PySide2.QtCore import Qt
+from PySide6.QtCore import Qt
 from ...mvcmodels.empty_row_model import EmptyRowModel
 from .parameter_mixins import (
     FillInParameterNameMixin,
@@ -28,7 +28,7 @@ from .parameter_mixins import (
     FillInValueListIdMixin,
 )
 from ...mvcmodels.shared import PARSED_ROLE, DB_MAP_ROLE
-from ...helpers import rows_to_row_count_tuples
+from ...helpers import rows_to_row_count_tuples, DB_ITEM_SEPARATOR
 
 
 class EmptyParameterModel(EmptyRowModel):
@@ -86,24 +86,25 @@ class EmptyParameterModel(EmptyRowModel):
     def item_id(self, _row):  # pylint: disable=no-self-use
         return None
 
-    def data(self, index, role=Qt.DisplayRole):
+    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
         if role == DB_MAP_ROLE:
-            return self.db_map
+            database = self.data(index, Qt.ItemDataRole.DisplayRole)
+            return next(iter(x for x in self.db_mngr.db_maps if x.codename == database), None)
         if self.header[index.column()] == self.value_field and role in (
-            Qt.DisplayRole,
-            Qt.ToolTipRole,
+            Qt.ItemDataRole.DisplayRole,
+            Qt.ItemDataRole.ToolTipRole,
             Qt.TextAlignmentRole,
             PARSED_ROLE,
         ):
-            data = super().data(index, role=Qt.EditRole)
+            data = super().data(index, role=Qt.ItemDataRole.EditRole)
             return self.db_mngr.get_value_from_data(data, role)
         return super().data(index, role)
 
     def _make_unique_id(self, item):
-        """Returns a unique id for the given model item (name-based). Used by receive_parameter_data_added."""
+        """Returns a unique id for the given model item (name-based). Used by handle_items_added."""
         return (item.get(self.entity_class_name_key), item.get("parameter_name"))
 
-    def receive_parameter_data_added(self, db_map_data):
+    def handle_items_added(self, db_map_data):
         """Runs when parameter definitions or values are added.
         Finds and removes model items that were successfully added to the db."""
         added_ids = set()
@@ -113,8 +114,8 @@ class EmptyParameterModel(EmptyRowModel):
                 unique_id = (database, *self._make_unique_id(item))
                 added_ids.add(unique_id)
         removed_rows = []
-        for row, data in enumerate(self._main_data):
-            item = dict(zip(self.header, data))
+        for row in range(self.rowCount()):
+            item = self._make_item(row)
             database = item.get("database")
             unique_id = (database, *self._make_unique_id(item))
             if unique_id in added_ids:
@@ -139,6 +140,9 @@ class EmptyParameterModel(EmptyRowModel):
         """
         raise NotImplementedError()
 
+    def _make_item(self, row):
+        return dict(zip(self.header, self._main_data[row]), row=row)
+
     def _make_db_map_data(self, rows):
         """
         Returns model data grouped by database map.
@@ -149,7 +153,7 @@ class EmptyParameterModel(EmptyRowModel):
         Returns:
             dict: mapping DiffDatabaseMapping instance to list of items
         """
-        items = [dict(zip(self.header, self._main_data[row]), row=row) for row in rows]
+        items = [self._make_item(row) for row in rows]
         db_map_data = dict()
         for item in items:
             database = item.pop("database")
@@ -252,8 +256,8 @@ class EmptyParameterValueModel(
         return {"object": "name", "relationship": "object_name_list"}[self.entity_type]
 
     def _make_unique_id(self, item):
-        """Returns a unique id for the given model item (name-based). Used by receive_parameter_data_added."""
-        return (*super()._make_unique_id(item), item.get(self.entity_name_key), item.get("alternative_name"))
+        """Returns a unique id for the given model item (name-based). Used by handle_items_added."""
+        return (*super()._make_unique_id(item), item.get("alternative_name"))
 
     def add_items_to_db(self, db_map_data):
         """See base class."""
@@ -294,6 +298,9 @@ class EmptyObjectParameterValueModel(EmptyParameterValueModel):
     def entity_type(self):
         return "object"
 
+    def _make_unique_id(self, item):
+        return (*super()._make_unique_id(item), item.get("name"))
+
 
 class EmptyRelationshipParameterValueModel(MakeRelationshipOnTheFlyMixin, EmptyParameterValueModel):
     """An empty relationship parameter_value model."""
@@ -308,6 +315,19 @@ class EmptyRelationshipParameterValueModel(MakeRelationshipOnTheFlyMixin, EmptyP
     def entity_type(self):
         return "relationship"
 
+    def _make_unique_id(self, item):
+        object_name_list = item.get("object_name_list")
+        return (
+            *super()._make_unique_id(item),
+            DB_ITEM_SEPARATOR.join(object_name_list) if object_name_list is not None else None,
+        )
+
+    def _make_item(self, row):
+        item = super()._make_item(row)
+        if item["object_name_list"]:
+            item["object_name_list"] = tuple(item["object_name_list"].split(DB_ITEM_SEPARATOR))
+        return item
+
     def add_items_to_db(self, db_map_data):
         """See base class."""
         # Call the super method to add whatever is ready.
