@@ -17,9 +17,10 @@ Contains :class:`ItemMetadataTableModel` and associated functionality.
 """
 from enum import auto, Enum, IntEnum, unique
 
-from PySide2.QtCore import QModelIndex
+from PySide6.QtCore import QModelIndex
 
 from spinetoolbox.helpers import rows_to_row_count_tuples
+from spinetoolbox.fetch_parent import FlexibleFetchParent
 from .metadata_table_model_base import Column, FLAGS_EDITABLE, FLAGS_FIXED, MetadataTableModelBase
 
 
@@ -45,16 +46,36 @@ class ItemMetadataTableModel(MetadataTableModelBase):
     _ITEM_NAME_KEY = "metadata_name"
     _ITEM_VALUE_KEY = "metadata_value"
 
-    def __init__(self, db_mngr, db_maps, parent=None):
+    def __init__(self, db_mngr, db_maps, db_editor):
         """
         Args:
             db_mngr (SpineDBManager): database manager
             db_maps (Iterable of DatabaseMappingBase): database maps
-            parent (QObject): parent object
+            db_editor (SpineDBEditor): DB editor
         """
-        super().__init__(db_mngr, db_maps, parent)
+        super().__init__(db_mngr, db_maps, db_editor)
         self._item_type = None
         self._item_ids = {}
+        self._entity_metadata_fetch_parent = FlexibleFetchParent(
+            "entity_metadata",
+            handle_items_added=self.add_item_metadata,
+            handle_items_removed=self.remove_item_metadata,
+            handle_items_updated=self.update_item_metadata,
+            accepts_item=self._accepts_entity_metadata_item,
+            owner=self,
+        )
+        self._parameter_value_metadata_fetch_parent = FlexibleFetchParent(
+            "parameter_value_metadata",
+            handle_items_added=self.add_item_metadata,
+            handle_items_removed=self.remove_item_metadata,
+            handle_items_updated=self.update_item_metadata,
+            accepts_item=self._accepts_parameter_value_metadata_item,
+            owner=self,
+        )
+
+    def _fetch_parents(self):
+        yield self._entity_metadata_fetch_parent
+        yield self._parameter_value_metadata_fetch_parent
 
     def clear(self):
         """Clears the model."""
@@ -69,16 +90,24 @@ class ItemMetadataTableModel(MetadataTableModelBase):
         """See base class."""
         return [None, None]
 
+    def _accepts_entity_metadata_item(self, item, db_map):
+        if self._item_type != ItemType.ENTITY:
+            return False
+        return item["entity_id"] == self._item_ids.get(db_map)
+
+    def _accepts_parameter_value_metadata_item(self, item, db_map):
+        if self._item_type != ItemType.VALUE:
+            return False
+        return item["parameter_value_id"] == self._item_ids.get(db_map)
+
     def set_entity_ids(self, db_map_ids):
         """Sets the model to show metadata from given entity.
 
         Args:
             db_map_ids (dict): mapping from database mapping to entity's id in that database
         """
-        metadata = {
-            db_map: self._db_mngr.get_entity_metadata(db_map, entity_id) for db_map, entity_id in db_map_ids.items()
-        }
-        self._reset_metadata(ItemType.ENTITY, db_map_ids, metadata)
+        self._reset_metadata(ItemType.ENTITY, db_map_ids)
+        self._reset_fetch_parents()
 
     def set_parameter_value_ids(self, db_map_ids):
         """Sets the model to show metadata from given parameter value.
@@ -86,18 +115,15 @@ class ItemMetadataTableModel(MetadataTableModelBase):
         Args:
             db_map_ids (dict): mapping from database mapping to value's id in that database
         """
-        metadata = {
-            db_map: self._db_mngr.get_parameter_value_metadata(db_map, id_) for db_map, id_ in db_map_ids.items()
-        }
-        self._reset_metadata(ItemType.VALUE, db_map_ids, metadata)
+        self._reset_metadata(ItemType.VALUE, db_map_ids)
+        self._reset_fetch_parents()
 
-    def _reset_metadata(self, item_type, db_map_ids, metadata):
+    def _reset_metadata(self, item_type, db_map_ids):
         """Resets model.
 
         Args:
             item_type (ItemType): current item type
             db_map_ids (dict): mapping from database mapping to value's id in that database
-            metadata (dict): mapping from database mapping to metadata records
         """
         self.beginResetModel()
         self._item_type = item_type
@@ -105,20 +131,15 @@ class ItemMetadataTableModel(MetadataTableModelBase):
         self._db_maps = set(db_map_ids.keys())
         default_db_map = next(iter(self._db_maps)) if self._db_maps else None
         self._adder_row = self._make_adder_row(default_db_map)
-        self._data = [
-            [record.metadata_name, record.metadata_value, db_map, record.id, record.metadata_id]
-            for db_map, records in metadata.items()
-            for record in records
-        ]
-        if db_map_ids:
-            db_map = next(iter(db_map_ids))
-        elif self._db_maps:
-            db_map = next(iter(self._db_maps))
-        else:
-            db_map = None
-        self._adder_row = self._make_adder_row(db_map)
+        self._data = []
         self.endResetModel()
 
+    def _reset_fetch_parents(self):
+        for parent in self._fetch_parents():
+            parent.reset_fetching(None)
+        if self.canFetchMore(None):
+            self.fetchMore(None)
+
     def _add_data_to_db_mngr(self, name, value, db_map):
         """See base class."""
         item_id = self._item_ids[db_map]
@@ -142,41 +163,16 @@ class ItemMetadataTableModel(MetadataTableModelBase):
                 {db_map: [{"id": id_, "metadata_name": name, "metadata_value": value}]}
             )
 
-    def roll_back(self, db_maps):
+    def rollback(self, _db_maps):
         """Rolls back changes in database.
 
         Args:
             db_maps (Iterable of DiffDatabaseMapping): database mappings that have been rolled back
         """
-        spans = rows_to_row_count_tuples(
-            i for db_map in db_maps for i, row in enumerate(self._data) if row[Column.DB_MAP] == db_map
-        )
-        for span in spans:
-            first = span[0]
-            last = span[0] + span[1] - 1
-            self.beginRemoveRows(QModelIndex(), first, last)
-            self._data = self._data[:first] + self._data[last + 1 :]
-            self.endRemoveRows()
-        if self._item_type == ItemType.ENTITY:
-            get_item_metadata = self._db_mngr.get_entity_metadata
-        else:
-            get_item_metadata = self._db_mngr.get_parameter_value_metadata
-        metadata = {}
-        for db_map in db_maps:
-            id_ = self._item_ids.get(db_map)
-            if id_ is None:
-                continue
-            metadata.update({db_map: get_item_metadata(db_map, id_)})
-        if not metadata:
-            return
-        rolled_back_data = [
-            [record.metadata_name, record.metadata_value, db_map, record.id, record.metadata_id]
-            for db_map, records in metadata.items()
-            for record in records
-        ]
-        self.beginInsertRows(QModelIndex(), len(self._data), len(self._data) + len(rolled_back_data) - 1)
-        self._data += rolled_back_data
-        self.endInsertRows()
+        self.beginResetModel()
+        self._data = []
+        self.endResetModel()
+        self._reset_fetch_parents()
 
     def flags(self, index):
         row = index.row()
@@ -239,19 +235,3 @@ class ItemMetadataTableModel(MetadataTableModelBase):
             db_map_data (dict): removed items keyed by database mapping
         """
         self._remove_data(db_map_data, ExtraColumn.ITEM_METADATA_ID)
-
-    def update_metadata(self, db_map_data):
-        """Updates metadata.
-
-        Args:
-            db_map_data (dict): updated items keyed by database mapping
-        """
-        self._update_data(db_map_data, ExtraColumn.METADATA_ID)
-
-    def remove_metadata(self, db_map_data):
-        """Removes entries that correspond to removed metadata.
-
-        Args:
-            db_map_data (dict): removed items keyed by database mapping
-        """
-        self._remove_data(db_map_data, ExtraColumn.METADATA_ID)
