@@ -16,7 +16,9 @@ Contains base class for metadata table models associated functionality.
 :date:   25.4.2022
 """
 from enum import IntEnum, unique
-from PySide2.QtCore import QAbstractTableModel, QModelIndex, Qt, Signal
+from operator import itemgetter
+
+from PySide6.QtCore import QAbstractTableModel, QModelIndex, Qt, Signal
 from spinetoolbox.helpers import rows_to_row_count_tuples
 from .colors import FIXED_FIELD_COLOR
 
@@ -48,14 +50,15 @@ class MetadataTableModelBase(QAbstractTableModel):
     _ITEM_NAME_KEY = None
     _ITEM_VALUE_KEY = None
 
-    def __init__(self, db_mngr, db_maps, parent=None):
+    def __init__(self, db_mngr, db_maps, db_editor):
         """
         Args:
             db_mngr (SpineDBManager): database manager
             db_maps (Iterable of DatabaseMappingBase): database maps
-            parent (QObject): parent object
+            db_editor (SpineDBEditor): DB editor
         """
-        super().__init__(parent)
+        super().__init__(db_editor)
+        self._db_editor = db_editor
         self._db_mngr = db_mngr
         self._data = []
         self._db_maps = db_maps
@@ -97,22 +100,42 @@ class MetadataTableModelBase(QAbstractTableModel):
         self._adder_row = self._make_adder_row(default_db_map)
         self.endResetModel()
 
+    def _fetch_parents(self):
+        """Yields fetch parents for this model.
+
+        Yields:
+            FetchParent
+        """
+        raise NotImplementedError()
+
+    def canFetchMore(self, _):
+        result = False
+        for fetch_parent in self._fetch_parents():
+            for db_map in self._db_maps:
+                result |= self._db_mngr.can_fetch_more(db_map, fetch_parent)
+        return result
+
+    def fetchMore(self, _):
+        for parent in self._fetch_parents():
+            for db_map in self._db_maps:
+                self._db_mngr.fetch_more(db_map, parent)
+
     def rowCount(self, parent=QModelIndex()):
         return len(self._data) + 1
 
     def columnCount(self, parent=QModelIndex()):
         return len(self._HEADER)
 
-    def data(self, index, role=Qt.DisplayRole):
+    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
         column = index.column()
         row = index.row()
-        if role == Qt.DisplayRole:
+        if role == Qt.ItemDataRole.DisplayRole:
             if column == Column.DB_MAP:
                 db_map = self._data[row][column] if row < len(self._data) else self._adder_row[column]
                 return db_map.codename if db_map is not None else ""
             return self._data[row][column] if row < len(self._data) else self._adder_row[column]
         if (
-            role == Qt.BackgroundRole
+            role == Qt.ItemDataRole.BackgroundRole
             and column == Column.DB_MAP
             and row < len(self._data)
             and self._row_id(self._data[row]) is not None
@@ -141,8 +164,8 @@ class MetadataTableModelBase(QAbstractTableModel):
         """
         raise NotImplementedError()
 
-    def setData(self, index, value, role=Qt.EditRole):
-        if role != Qt.EditRole:
+    def setData(self, index, value, role=Qt.ItemDataRole.EditRole):
+        if role != Qt.ItemDataRole.EditRole:
             return False
         column = index.column()
         row = index.row()
@@ -159,7 +182,7 @@ class MetadataTableModelBase(QAbstractTableModel):
         metadata_value = target_row[Column.VALUE]
         db_map = target_row[Column.DB_MAP]
         if not name or not metadata_value or db_map is None:
-            self.dataChanged.emit(index, index, [Qt.DisplayRole])
+            self.dataChanged.emit(index, index, [Qt.ItemDataRole.DisplayRole])
             return True
         if reserved.get(db_map, {}).get(name) == metadata_value:
             target_row[column] = previous_value
@@ -176,7 +199,7 @@ class MetadataTableModelBase(QAbstractTableModel):
             self._adder_row = self._make_adder_row(db_map)
             top_left = self.index(data_length, 0)
             bottom_right = self.index(data_length, Column.DB_MAP)
-            self.dataChanged.emit(top_left, bottom_right, [Qt.DisplayRole])
+            self.dataChanged.emit(top_left, bottom_right, [Qt.ItemDataRole.DisplayRole])
         return True
 
     def batch_set_data(self, indexes, values):
@@ -193,7 +216,7 @@ class MetadataTableModelBase(QAbstractTableModel):
         available_codenames = {db_map.codename: db_map for db_map in self._db_maps}
         reserved = self._reserved_metadata()
         for index, value in zip(indexes, values):
-            if not (self.flags(index) & Qt.ItemIsEditable):
+            if not self.flags(index) & Qt.ItemIsEditable:
                 continue
             if value is None:
                 value = ""
@@ -241,12 +264,12 @@ class MetadataTableModelBase(QAbstractTableModel):
         if rows:
             top_left = self.index(min(rows), min(columns))
             bottom_right = self.index(max(rows), max(columns))
-            self.dataChanged.emit(top_left, bottom_right, [Qt.DisplayRole])
+            self.dataChanged.emit(top_left, bottom_right, [Qt.ItemDataRole.DisplayRole])
         if duplicates_found:
             self.msg_error.emit("Duplicate metadata names and values.")
 
-    def headerData(self, section, orientation, role=Qt.DisplayRole):
-        if orientation != Qt.Horizontal or role != Qt.DisplayRole:
+    def headerData(self, section, orientation, role=Qt.ItemDataRole.DisplayRole):
+        if orientation != Qt.Orientation.Horizontal or role != Qt.ItemDataRole.DisplayRole:
             return None
         return self._HEADER[section]
 
@@ -282,13 +305,13 @@ class MetadataTableModelBase(QAbstractTableModel):
         """
         raise NotImplementedError()
 
-    def removeRows(self, row, count, parent=QModelIndex()):
-        if row == len(self._data):
+    def removeRows(self, first, count, parent=QModelIndex()):
+        if first == len(self._data):
             return False
-        count = min(count, len(self._data) - row)
+        count = min(count, len(self._data) - first)
         ids_to_remove = {}
         table_name = self._database_table_name()
-        for i, row in enumerate(self._data[row : row + count]):
+        for i, row in enumerate(self._data[first : first + count]):
             id_to_remove = self._row_id(row)
             if id_to_remove is not None:
                 ids_to_remove.setdefault(row[Column.DB_MAP], {}).setdefault(table_name, set()).add(id_to_remove)
@@ -368,7 +391,7 @@ class MetadataTableModelBase(QAbstractTableModel):
         if id_update_rows:
             top_left = self.index(min(id_update_rows), Column.DB_MAP)
             bottom_right = self.index(max(id_update_rows), Column.DB_MAP)
-            self.dataChanged.emit(top_left, bottom_right, [Qt.BackgroundRole])
+            self.dataChanged.emit(top_left, bottom_right, [Qt.ItemDataRole.BackgroundRole])
 
     def _update_data(self, db_map_data, id_column):
         """Update data table after database update.
@@ -377,7 +400,7 @@ class MetadataTableModelBase(QAbstractTableModel):
             db_map_data (dict): updated items keyed by database mapping
             id_column (int): column that contains item ids
         """
-        for db_map, items in db_map_data.items():
+        for items in db_map_data.values():
             items_by_id = {item["id"]: item for item in items}
             updated_rows = []
             for row_index, row in enumerate(self._data):
@@ -398,7 +421,7 @@ class MetadataTableModelBase(QAbstractTableModel):
             if updated_rows:
                 top_left = self.index(updated_rows[0], 0)
                 bottom_right = self.index(updated_rows[-1], Column.DB_MAP - 1)
-                self.dataChanged.emit(top_left, bottom_right, [Qt.DisplayRole])
+                self.dataChanged.emit(top_left, bottom_right, [Qt.ItemDataRole.DisplayRole])
 
     def _remove_data(self, db_map_data, id_column):
         """Removes data from model after it has been removed from databases.
@@ -407,7 +430,7 @@ class MetadataTableModelBase(QAbstractTableModel):
             db_map_data (dict): removed items keyed by database mapping
             id_column (int): column that contains item ids
         """
-        for db_map, items in db_map_data.items():
+        for items in db_map_data.values():
             ids_to_remove = {item["id"] for item in items}
             removed_rows = []
             for row_index, row in enumerate(self._data):
@@ -426,18 +449,15 @@ class MetadataTableModelBase(QAbstractTableModel):
         if not self._data or column < 0:
             return
 
-        def string_sort_key(row):
-            return row[column]
-
         def db_map_sort_key(row):
             db_map = row[Column.DB_MAP]
             return db_map.codename if db_map is not None else ""
 
-        sort_key = string_sort_key if column != Column.DB_MAP else db_map_sort_key
+        sort_key = itemgetter(column) if column != Column.DB_MAP else db_map_sort_key
         self._data.sort(key=sort_key, reverse=order == Qt.DescendingOrder)
         top_left = self.index(0, 0)
         bottom_right = self.index(len(self._data) - 1, Column.DB_MAP)
-        self.dataChanged.emit(top_left, bottom_right, [Qt.DisplayRole, Qt.BackgroundRole])
+        self.dataChanged.emit(top_left, bottom_right, [Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.BackgroundRole])
 
     def _find_db_map(self, codename):
         """Finds database mapping with given codename.
