@@ -17,12 +17,13 @@ Base classes to represent items from multiple databases in a tree.
 """
 from operator import attrgetter
 
-from PySide2.QtCore import Qt, QTimer
-from ...helpers import rows_to_row_count_tuples, bisect_chunks, FetchParent
+from PySide6.QtCore import Qt
+from ...helpers import rows_to_row_count_tuples, bisect_chunks
+from ...fetch_parent import FlexibleFetchParent
 from ...mvcmodels.minimal_tree_model import TreeItem
 
 
-class MultiDBTreeItem(FetchParent, TreeItem):
+class MultiDBTreeItem(TreeItem):
     """A tree item that may belong in multiple databases."""
 
     item_type = None
@@ -41,6 +42,15 @@ class MultiDBTreeItem(FetchParent, TreeItem):
             db_map_ids = {}
         self._db_map_ids = db_map_ids
         self._child_map = dict()  # Maps db_map to id to row number
+        self._fetch_parent = FlexibleFetchParent(
+            self.fetch_item_type,
+            accepts_item=self.accepts_item,
+            handle_items_added=self.handle_items_added,
+            handle_items_removed=self.handle_items_removed,
+            handle_items_updated=self.handle_items_updated,
+            will_have_children_change=self.will_have_children_change,
+            owner=self,
+        )
 
     def child_number(self):
         try:
@@ -130,13 +140,11 @@ class MultiDBTreeItem(FetchParent, TreeItem):
                 removed_rows.append(row)
         for row, count in reversed(rows_to_row_count_tuples(removed_rows)):
             self.remove_children(row, count)
-        changed_rows = []
         for row, child in enumerate(self.children):
             child._deep_refresh_children()
-            changed_rows.append(row)
-        if changed_rows:
-            top_row = changed_rows[0]
-            bottom_row = changed_rows[-1]
+        if self.children:
+            top_row = 0
+            bottom_row = self.child_count() - 1
             top_index = self.children[top_row].index().sibling(top_row, 1)
             bottom_index = self.children[bottom_row].index().sibling(bottom_row, 1)
             self.model.dataChanged.emit(top_index, bottom_index)
@@ -187,7 +195,7 @@ class MultiDBTreeItem(FetchParent, TreeItem):
         """Returns field from data for this item in given db_map or None if not found."""
         return self.db_map_data(db_map).get(field, default)
 
-    def _create_new_children(self, db_map, children_ids):
+    def _create_new_children(self, db_map, children_ids, **kwargs):
         """
         Creates new items from ids associated to a db map.
 
@@ -198,7 +206,7 @@ class MultiDBTreeItem(FetchParent, TreeItem):
         Returns:
             list of MultiDBTreeItem: new children
         """
-        return [self.child_item_class(self.model, {db_map: id_}) for id_ in children_ids]
+        return [self.child_item_class(self.model, {db_map: id_}, **kwargs) for id_ in children_ids]
 
     def _merge_children(self, new_children):
         """Merges new children into this item. Ensures that each child has a valid display id afterwards."""
@@ -229,11 +237,11 @@ class MultiDBTreeItem(FetchParent, TreeItem):
 
     @property
     def _children_sort_key(self):
-        return lambda item: item.display_id
+        return attrgetter("display_id")
 
-    def fetch_status_change(self):
+    def will_have_children_change(self):
         """Notifies the view that the model's layout has changed.
-        This triggers a repaint so this item may be painted gray if no children."""
+        This triggers a repaint so this item will be painted gray if no children."""
         self.model.layoutChanged.emit()
 
     @property
@@ -243,25 +251,38 @@ class MultiDBTreeItem(FetchParent, TreeItem):
     def can_fetch_more(self):
         if self.fetch_item_type is None:
             return False
-        return any(self.db_mngr.can_fetch_more(db_map, self) for db_map in self.db_maps)
+        result = False
+        for db_map in self.db_maps:
+            result |= self.db_mngr.can_fetch_more(db_map, self._fetch_parent)
+        return result
 
     def fetch_more(self):
         """Fetches children from all associated databases."""
         if self.fetch_item_type is None:
             return
         for db_map in self.db_maps:
-            self.db_mngr.fetch_more(db_map, self)
+            self.db_mngr.fetch_more(db_map, self._fetch_parent)
 
     def fetch_more_if_possible(self):
         if self.can_fetch_more():
             self.fetch_more()
 
-    def get_children_ids(self):
-        for db_map, ids in self._child_map.items():
-            for id_ in ids:
-                yield (db_map, id_)
+    def accepts_item(self, item, db_map):
+        return True
 
-    def append_children_by_id(self, db_map_ids):
+    def handle_items_added(self, db_map_data):
+        db_map_ids = {db_map: [x["id"] for x in data] for db_map, data in db_map_data.items()}
+        self.append_children_by_id(db_map_ids)
+
+    def handle_items_removed(self, db_map_data):
+        db_map_ids = {db_map: {x["id"] for x in data} for db_map, data in db_map_data.items()}
+        self.remove_children_by_id(db_map_ids)
+
+    def handle_items_updated(self, db_map_data):
+        db_map_ids = {db_map: {x["id"] for x in data} for db_map, data in db_map_data.items()}
+        self.update_children_by_id(db_map_ids)
+
+    def append_children_by_id(self, db_map_ids, **kwargs):
         """
         Appends children by id.
 
@@ -270,7 +291,7 @@ class MultiDBTreeItem(FetchParent, TreeItem):
         """
         new_children = []
         for db_map, ids in db_map_ids.items():
-            new_children += self._create_new_children(db_map, ids)
+            new_children += self._create_new_children(db_map, ids, **kwargs)
         self._merge_children(new_children)
 
     def remove_children_by_id(self, db_map_ids):
@@ -289,7 +310,7 @@ class MultiDBTreeItem(FetchParent, TreeItem):
         """Checks if the item is still valid after an update operation."""
         return True
 
-    def update_children_by_id(self, db_map_ids):
+    def update_children_by_id(self, db_map_ids, **kwargs):
         """
         Updates children by id. Essentially makes sure all children have a valid display id
         after updating the underlying data. These may require 'splitting' a child
@@ -316,11 +337,12 @@ class MultiDBTreeItem(FetchParent, TreeItem):
                     db_map_ids_to_add.setdefault(db_map, set()).add(id_)
         new_children = []  # List of new children to be inserted
         for db_map, ids in db_map_ids_to_add.items():
-            new_children += self._create_new_children(db_map, ids)
+            new_children += self._create_new_children(db_map, ids, **kwargs)
         # Check display ids
         display_ids = [child.display_id for child in self.children if child.display_id is not None]
         for row in sorted(rows_to_update, reverse=True):
             child = self.child(row)
+            child.update(**kwargs)
             if not child:
                 continue
             if not child.is_valid():
@@ -332,7 +354,7 @@ class MultiDBTreeItem(FetchParent, TreeItem):
                 db_map = child.first_db_map
                 new_child = child.deep_take_db_map(db_map)
                 new_children.append(new_child)
-            if child.display_id in display_ids[:row] + display_ids[row + 1 :]:
+            if child.display_id in display_ids[:row] + display_ids[row + 1 :] or child.should_be_merged():
                 # Take the child and put it in the list to be merged
                 new_children.append(child)
                 self.remove_children(row, 1)
@@ -341,7 +363,13 @@ class MultiDBTreeItem(FetchParent, TreeItem):
         self._merge_children(new_children)
         top_left = self.model.index(0, 0, self.index())
         bottom_right = self.model.index(self.child_count() - 1, 0, self.index())
-        self.model.dataChanged.emit(top_left, bottom_right, [Qt.DisplayRole])
+        self.model.dataChanged.emit(top_left, bottom_right)
+
+    def update(self, **kwargs):
+        pass
+
+    def should_be_merged(self):
+        return False
 
     def insert_children(self, position, children):
         """Insert new children at given position. Returns a boolean depending on how it went.
@@ -404,18 +432,22 @@ class MultiDBTreeItem(FetchParent, TreeItem):
                 if row != -1:
                     yield row
 
-    def data(self, column, role=Qt.DisplayRole):
+    def data(self, column, role=Qt.ItemDataRole.DisplayRole):
         """Returns data for given column and role."""
         if column == 0:
-            if role == Qt.DecorationRole:
+            if role == Qt.ItemDataRole.DecorationRole:
                 return self.display_icon
-            if role == Qt.DisplayRole:
+            if role == Qt.ItemDataRole.DisplayRole:
                 return self.display_data
-            if role == Qt.EditRole:
+            if role == Qt.ItemDataRole.EditRole:
                 return self.edit_data
-        if column and role == Qt.DisplayRole:
+        if column and role == Qt.ItemDataRole.DisplayRole:
             return self.display_database
 
     def default_parameter_data(self):
         """Returns data to set as default in a parameter table when this item is selected."""
         return {"database": self.first_db_map.codename}
+
+    def tear_down(self):
+        super().tear_down()
+        self._fetch_parent.set_obsolete(True)
