@@ -15,22 +15,23 @@ A base model class to represent items from multiple databases in a tree.
 :authors: P. Vennström (VTT), M. Marin (KTH)
 :date:    17.6.2020
 """
-from PySide2.QtCore import QModelIndex, Qt
+from PySide6.QtCore import QModelIndex, Qt
 from ...mvcmodels.minimal_tree_model import MinimalTreeModel, TreeItem
 
 
 class MultiDBTreeModel(MinimalTreeModel):
     """Base class for all tree models in Spine db editor."""
 
-    def __init__(self, parent, db_mngr, *db_maps):
+    def __init__(self, db_editor, db_mngr, *db_maps):
         """Init class.
 
         Args:
-            parent (SpineDBEditor)
+            db_editor (SpineDBEditor)
             db_mngr (SpineDBManager): A manager for the given db_maps
             db_maps (iter): DiffDatabaseMapping instances
         """
-        super().__init__(parent)
+        super().__init__(db_editor)
+        self.db_editor = db_editor
         self.db_mngr = db_mngr
         self.db_maps = db_maps
         self._root_item = None
@@ -59,8 +60,8 @@ class MultiDBTreeModel(MinimalTreeModel):
     def columnCount(self, parent=QModelIndex()):
         return 2
 
-    def headerData(self, section, orientation, role=Qt.DisplayRole):
-        if orientation == Qt.Horizontal and role == Qt.DisplayRole:
+    def headerData(self, section, orientation, role=Qt.ItemDataRole.DisplayRole):
+        if orientation == Qt.Orientation.Horizontal and role == Qt.ItemDataRole.DisplayRole:
             return ("name", "database")[section]
         return None
 
