@@ -16,9 +16,8 @@ Tree items for parameter_value lists.
 :date:   28.6.2019
 """
 
-from PySide2.QtCore import Qt
-from PySide2.QtGui import QIcon
-from spinedb_api import to_database
+from PySide6.QtCore import Qt
+from PySide6.QtGui import QIcon
 from spinetoolbox.mvcmodels.shared import PARSED_ROLE
 from .tree_item_utility import (
     EmptyChildMixin,
@@ -27,8 +26,8 @@ from .tree_item_utility import (
     EditableMixin,
     StandardDBItem,
     FetchMoreMixin,
+    SortChildrenMixin,
     LeafItem,
-    ListValueFetchParent,
 )
 from ...helpers import CharIconEngine
 
@@ -47,36 +46,50 @@ class DBItem(EmptyChildMixin, FetchMoreMixin, StandardDBItem):
     def empty_child(self):
         return ListItem()
 
+    def _make_child(self, id_):
+        return ListItem(id_)
 
-class ListItem(GrayIfLastMixin, EditableMixin, EmptyChildMixin, BoldTextMixin, FetchMoreMixin, LeafItem):
+
+class ListItem(
+    GrayIfLastMixin, EditableMixin, EmptyChildMixin, SortChildrenMixin, BoldTextMixin, FetchMoreMixin, LeafItem
+):
     """A list item."""
 
     def __init__(self, identifier=None, name=None):
         super().__init__(identifier=identifier)
         self._name = name
-        self._list_value_fetch_parent = ListValueFetchParent(identifier)
 
     @property
     def item_type(self):
         return "parameter_value_list"
 
-    def _fetch_parents(self):
-        yield self._list_value_fetch_parent
+    @property
+    def fetch_item_type(self):
+        return "list_value"
 
     def _make_item_data(self):
         return {"name": "Type new list name here..." if self._name is None else self._name}
 
-    def _do_finalize(self):
+    def _do_set_up(self):
         if not self.id and not self._name:
             return
-        super()._do_finalize()
+        super()._do_set_up()
 
     # pylint: disable=no-self-use
     def empty_child(self):
         return ValueItem()
 
-    def data(self, column, role=Qt.DisplayRole):
-        if role == Qt.DecorationRole:
+    def _make_child(self, id_):
+        return ValueItem(id_)
+
+    def accepts_item(self, item, db_map):
+        return item["parameter_value_list_id"] == self.id
+
+    def _children_sort_key(self, child):
+        return self.db_mngr.get_item(self.db_map, "list_value", child.id)["index"]
+
+    def data(self, column, role=Qt.ItemDataRole.DisplayRole):
+        if role == Qt.ItemDataRole.DecorationRole:
             engine = CharIconEngine("\uf022", 0)
             return QIcon(engine.pixmap())
         return super().data(column, role)
@@ -96,11 +109,11 @@ class ValueItem(GrayIfLastMixin, EditableMixin, LeafItem):
     def item_type(self):
         return "list_value"
 
-    def data(self, column, role=Qt.DisplayRole):
-        if role == Qt.DisplayRole and not self.id:
+    def data(self, column, role=Qt.ItemDataRole.DisplayRole):
+        if role == Qt.ItemDataRole.DisplayRole and not self.id:
             return "Enter new list value here..."
-        if role in (Qt.DisplayRole, Qt.EditRole, Qt.ToolTipRole, PARSED_ROLE):
-            return self.db_mngr.get_value(self.db_map, self.item_type, self.id, role)
+        if role in (Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.EditRole, Qt.ItemDataRole.ToolTipRole, PARSED_ROLE):
+            return self.db_mngr.get_value(self.db_map, self.item_type, self.id, role=role)
         return super().data(column, role)
 
     def list_index(self):
