@@ -16,8 +16,7 @@ Single models for parameter definitions and values (as 'for a single entity').
 :date:   28.6.2019
 """
 
-from PySide2.QtCore import Qt
-from PySide2.QtGui import QGuiApplication
+from PySide6.QtCore import Qt
 from spinetoolbox.helpers import DB_ITEM_SEPARATOR
 from ...mvcmodels.minimal_table_model import MinimalTableModel
 from ..mvcmodels.parameter_mixins import (
@@ -214,39 +213,39 @@ class SingleParameterModel(HalfSortedTableModel):
         item_id = db_item.get(id_key)
         return self.db_mngr.get_item(self.db_map, item_type, item_id)
 
-    def data(self, index, role=Qt.DisplayRole):
+    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
         """Gets the id and database for the row, and reads data from the db manager
         using the item_type property.
         Paint the object_class icon next to the name.
         Also paint background of fixed indexes gray and apply custom format to JSON fields."""
         field = self.header[index.column()]
         # Background role
-        if role == Qt.BackgroundRole and field in self.fixed_fields:
+        if role == Qt.ItemDataRole.BackgroundRole and field in self.fixed_fields:
             return FIXED_FIELD_COLOR
         # Display, edit, tool tip, alignment role of 'json fields'
         if field == self.value_field and role in (
-            Qt.DisplayRole,
-            Qt.EditRole,
-            Qt.ToolTipRole,
+            Qt.ItemDataRole.DisplayRole,
+            Qt.ItemDataRole.EditRole,
+            Qt.ItemDataRole.ToolTipRole,
             Qt.TextAlignmentRole,
             PARSED_ROLE,
         ):
             id_ = self._main_data[index.row()]
             return self.db_mngr.get_value(self.db_map, self.item_type, id_, role)
-        if role in (Qt.DisplayRole, Qt.EditRole, Qt.ToolTipRole):
+        if role in (Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.EditRole, Qt.ItemDataRole.ToolTipRole):
             if field == "database":
                 return self.db_map.codename
             id_ = self._main_data[index.row()]
             item = self.db_mngr.get_item(self.db_map, self.item_type, id_)
-            if role == Qt.ToolTipRole:
+            if role == Qt.ItemDataRole.ToolTipRole:
                 description = self.get_field_item(field, item).get("description", None)
                 if description not in (None, ""):
                     return description
             data = item.get(field)
-            if role == Qt.DisplayRole and data and field in self.group_fields:
-                data = data.replace(",", DB_ITEM_SEPARATOR)
+            if data and field in self.group_fields:
+                data = DB_ITEM_SEPARATOR.join(data)
             return data
-        if role == Qt.DecorationRole and field == self.entity_class_name_field:
+        if role == Qt.ItemDataRole.DecorationRole and field == self.entity_class_name_field:
             return self.db_mngr.entity_class_icon(self.db_map, self.entity_class_type, self.entity_class_id)
         if role == DB_MAP_ROLE:
             return self.db_map
@@ -257,11 +256,17 @@ class SingleParameterModel(HalfSortedTableModel):
         Sets data directly in database using db mngr. If successful, updated data will be
         automatically seen by the data method.
         """
+
+        def split_value(value, column):
+            if self.header[column] in self.group_fields:
+                return tuple(value.split(DB_ITEM_SEPARATOR))
+            return value
+
         if not indexes or not data:
             return False
         row_data = dict()
         for index, value in zip(indexes, data):
-            row_data.setdefault(index.row(), {})[self.header[index.column()]] = value
+            row_data.setdefault(index.row(), {})[self.header[index.column()]] = split_value(value, index.column())
         items = [dict(id=self._main_data[row], **data) for row, data in row_data.items()]
         self.update_items_in_db(items)
         return True
@@ -278,7 +283,7 @@ class SingleParameterModel(HalfSortedTableModel):
         return self._auto_filter_accepts_item(item)
 
     def set_auto_filter(self, field, values):
-        if values == self._auto_filter.get(field, {}):
+        if values == self._auto_filter.get(field, set()):
             return False
         self._auto_filter[field] = values
         return True
@@ -287,12 +292,8 @@ class SingleParameterModel(HalfSortedTableModel):
         """Returns the result of the auto filter."""
         if self._auto_filter is None:
             return False
-        try:
-            item_id = item["id"]
-        except KeyError as error:
-            raise error
-        for accepted_ids in self._auto_filter.values():
-            if accepted_ids and item_id not in accepted_ids:
+        for field, values in self._auto_filter.items():
+            if values and item.get(field) not in values:
                 return False
         return True
 
@@ -353,7 +354,7 @@ class SingleParameterDefinitionMixin(FillInParameterNameMixin, FillInValueListId
         """Update items in db.
 
         Args:
-            item (list): dictionary-items
+            items (list): dictionary-items
         """
         self.build_lookup_dictionary({self.db_map: items})
         param_defs = list()
@@ -402,7 +403,7 @@ class SingleParameterValueMixin(
 
     def _sort_key(self, element):
         item = self.db_item_from_id(element)
-        return item[self.entity_name_key], item["parameter_name"], item["alternative_name"]
+        return tuple(item[k] for k in (self.entity_name_key, "parameter_name", "alternative_name"))
 
     def set_filter_entity_ids(self, db_map_class_entity_ids):
         if self._filter_db_map_class_entity_ids == db_map_class_entity_ids:
@@ -496,7 +497,7 @@ class SingleRelationshipParameterValueModel(
         """Update items in db.
 
         Args:
-            item (list): dictionary-items
+            items (list): dictionary-items
         """
         for item in items:
             item["relationship_class_name"] = self.entity_class_name
