@@ -15,15 +15,8 @@ Classes to represent tool and feature items in a tree.
 :authors: M. Marin (KTH)
 :date:    1.9.2020
 """
-from PySide2.QtCore import Qt
-from .tree_item_utility import (
-    GrayIfLastMixin,
-    EditableMixin,
-    EmptyChildRootItem,
-    LeafItem,
-    StandardTreeItem,
-    ListValueFetchParent,
-)
+from PySide6.QtCore import Qt
+from .tree_item_utility import GrayIfLastMixin, EditableMixin, EmptyChildRootItem, LeafItem, StandardTreeItem
 
 _FEATURE_ICON = "\uf5bc"  # splotch
 _TOOL_ICON = "\uf6e3"  # hammer
@@ -48,6 +41,9 @@ class FeatureRootItem(EmptyChildRootItem):
     def empty_child(self):
         return FeatureLeafItem()
 
+    def _make_child(self, id_):
+        return FeatureLeafItem(id_)
+
 
 class ToolRootItem(EmptyChildRootItem):
     """A tool root item."""
@@ -67,6 +63,9 @@ class ToolRootItem(EmptyChildRootItem):
     def empty_child(self):
         return ToolLeafItem()
 
+    def _make_child(self, id_):
+        return ToolLeafItem(id_)
+
 
 class FeatureLeafItem(GrayIfLastMixin, EditableMixin, LeafItem):
     """A feature leaf item."""
@@ -85,8 +84,10 @@ class FeatureLeafItem(GrayIfLastMixin, EditableMixin, LeafItem):
         item_data = self.db_mngr.get_item(self.db_map, self.item_type, self.id)
         if not item_data:
             return {}
-        name = self.model.make_feature_name(item_data["entity_class_name"], item_data["parameter_definition_name"])
-        return dict(name=name, **item_data)
+        item_data["name"] = self.model.make_feature_name(
+            item_data["entity_class_name"], item_data["parameter_definition_name"]
+        )
+        return item_data
 
     @property
     def tool_tip(self):
@@ -151,10 +152,10 @@ class ToolLeafItem(GrayIfLastMixin, EditableMixin, LeafItem):
     def update_item_in_db(self, db_item):
         self.db_mngr.update_tools({self.db_map: [db_item]})
 
-    def _do_finalize(self):
+    def _do_set_up(self):
         if not self.id:
             return
-        super()._do_finalize()
+        super()._do_set_up()
         self.append_children([ToolFeatureRootItem()])
 
 
@@ -187,6 +188,12 @@ class ToolFeatureRootItem(EmptyChildRootItem):
     def empty_child(self):
         return ToolFeatureLeafItem()
 
+    def _make_child(self, id_):
+        return ToolFeatureLeafItem(id_)
+
+    def accepts_item(self, item, db_map):
+        return item["tool_id"] == self.parent_item.id
+
 
 class ToolFeatureLeafItem(GrayIfLastMixin, LeafItem):
     """A tool feature leaf item."""
@@ -203,15 +210,15 @@ class ToolFeatureLeafItem(GrayIfLastMixin, LeafItem):
         if not item_data:
             return {}
         feature_data = self.db_mngr.get_item(self.db_map, "feature", item_data["feature_id"])
-        name = self.model.make_feature_name(
+        item_data["name"] = self.model.make_feature_name(
             feature_data["entity_class_name"], feature_data["parameter_definition_name"]
         )
-        return dict(name=name, **item_data)
+        return item_data
 
-    def _do_finalize(self):
+    def _do_set_up(self):
         if not self.id:
             return
-        super()._do_finalize()
+        super()._do_set_up()
         self.append_children([ToolFeatureRequiredItem(), ToolFeatureMethodRootItem()])
 
     def _make_item_to_add(self, value):
@@ -248,16 +255,16 @@ class ToolFeatureRequiredItem(StandardTreeItem):
             flags |= Qt.ItemIsEditable
         return flags
 
-    def data(self, column, role=Qt.DisplayRole):
-        if column == 0 and role in (Qt.DisplayRole, Qt.EditRole):
+    def data(self, column, role=Qt.ItemDataRole.DisplayRole):
+        if column == 0 and role in (Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.EditRole):
             if not self.parent_item.item_data:
                 return None
             required = "yes" if self.parent_item.item_data["required"] else "no"
             return "required: " + required
         return super().data(column, role)
 
-    def set_data(self, column, value, role=Qt.EditRole):
-        if role == Qt.EditRole and column == 0:
+    def set_data(self, column, value, role=Qt.ItemDataRole.EditRole):
+        if role == Qt.ItemDataRole.EditRole and column == 0:
             required = {"yes": True, "no": False}.get(value)
             if required is None:
                 return False
@@ -273,19 +280,6 @@ class ToolFeatureRequiredItem(StandardTreeItem):
 class ToolFeatureMethodRootItem(EmptyChildRootItem):
     """A tool_feature_method root item."""
 
-    def __init__(self, *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self._list_value_fetch_parent = None
-
-    def _do_finalize(self):
-        super()._do_finalize()
-        parameter_value_list_id = self.parent_item.item_data["parameter_value_list_id"]
-        self._list_value_fetch_parent = ListValueFetchParent(parameter_value_list_id)
-
-    def _fetch_parents(self):
-        yield from super()._fetch_parents()
-        yield self._list_value_fetch_parent
-
     @property
     def item_type(self):
         return "tool_feature_method"
@@ -301,6 +295,12 @@ class ToolFeatureMethodRootItem(EmptyChildRootItem):
     def empty_child(self):
         return ToolFeatureMethodLeafItem()
 
+    def _make_child(self, id_):
+        return ToolFeatureMethodLeafItem(id_)
+
+    def accepts_item(self, item, db_map):
+        return item["tool_feature_id"] == self.parent_item.id
+
 
 class ToolFeatureMethodLeafItem(GrayIfLastMixin, LeafItem):
     """A tool_feature_method leaf item."""
@@ -320,10 +320,10 @@ class ToolFeatureMethodLeafItem(GrayIfLastMixin, LeafItem):
         item_data = self.db_mngr.get_item(self.db_map, self.item_type, self.id)
         if not item_data:
             return {}
-        name = self.db_mngr.get_value_list_item(
+        item_data["name"] = self.db_mngr.get_value_list_item(
             self.db_map, item_data["parameter_value_list_id"], item_data["method_index"]
         )
-        return dict(name=name, **item_data)
+        return item_data
 
     def _make_item_data(self):
         return {"name": "Enter new method here...", "description": ""}
