@@ -16,10 +16,11 @@ A tree model for parameter_value lists.
 :date:   28.6.2019
 """
 
-from PySide2.QtCore import Qt
-from PySide2.QtGui import QBrush, QFont, QIcon, QGuiApplication
+from PySide6.QtCore import Qt
+from PySide6.QtGui import QBrush, QFont, QIcon, QGuiApplication
 from spinetoolbox.mvcmodels.minimal_tree_model import TreeItem
-from spinetoolbox.helpers import CharIconEngine, FetchParent, ItemTypeFetchParent, bisect_chunks
+from spinetoolbox.helpers import CharIconEngine, bisect_chunks
+from spinetoolbox.fetch_parent import FlexibleFetchParent
 
 
 class StandardTreeItem(TreeItem):
@@ -52,18 +53,18 @@ class StandardTreeItem(TreeItem):
         engine = CharIconEngine(self.icon_code, 0)
         return QIcon(engine.pixmap())
 
-    def data(self, column, role=Qt.DisplayRole):
+    def data(self, column, role=Qt.ItemDataRole.DisplayRole):
         if column != 0:
             return None
-        if role in (Qt.DisplayRole, Qt.EditRole):
+        if role in (Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.EditRole):
             return self.display_data
-        if role == Qt.DecorationRole:
+        if role == Qt.ItemDataRole.DecorationRole:
             return self.display_icon
-        if role == Qt.ToolTipRole:
+        if role == Qt.ItemDataRole.ToolTipRole:
             return self.tool_tip
         return super().data(column, role)
 
-    def set_data(self, column, value, role=Qt.DisplayRole):
+    def set_data(self, column, value, role=Qt.ItemDataRole.DisplayRole):
         return False
 
     @property
@@ -88,7 +89,7 @@ class EditableMixin:
 class GrayIfLastMixin:
     """Paints the item gray if it's the last."""
 
-    def data(self, column, role=Qt.DisplayRole):
+    def data(self, column, role=Qt.ItemDataRole.DisplayRole):
         if role == Qt.ForegroundRole and self.child_number() == self.parent_item.child_count() - 1:
             gray_color = QGuiApplication.palette().text().color()
             gray_color.setAlpha(128)
@@ -100,8 +101,8 @@ class GrayIfLastMixin:
 class BoldTextMixin:
     """Bolds text."""
 
-    def data(self, column, role=Qt.DisplayRole):
-        if role == Qt.FontRole:
+    def data(self, column, role=Qt.ItemDataRole.DisplayRole):
+        if role == Qt.ItemDataRole.FontRole:
             bold_font = QFont()
             bold_font.setBold(True)
             return bold_font
@@ -118,29 +119,48 @@ class EmptyChildMixin:
     def empty_child(self):
         raise NotImplementedError()
 
-    def _do_finalize(self):
-        super()._do_finalize()
+    def _do_set_up(self):
+        super()._do_set_up()
         empty_child = self.empty_child()
         self.append_children([empty_child])
 
 
-class SortsChildrenMixin:
+class SortChildrenMixin:
+    def _children_sort_key(self, child):
+        return child.data(0)
+
     def insert_children_sorted(self, children):
         for child in children:
             child.parent_item = self
-        for chunk, pos in bisect_chunks(self.non_empty_children, children, key=lambda x: x.data(0)):
+        for chunk, pos in bisect_chunks(self.non_empty_children, children, key=self._children_sort_key):
             if not super().insert_children(pos, chunk):
                 return False
         return True
 
+    def _resort(self):
+        # FIXME MM Needed?
+        non_empty_children, empty_child = self.children[:-1], self.children[-1]
+        non_empty_children.sort(key=self._children_sort_key)
+        self.children = non_empty_children + [empty_child]
+        top = self.model.index_from_item(self.child(0))
+        bottom = self.model.index_from_item(self.child(-1))
+        self.model.dataChanged.emit(top, bottom)
+
 
 class FetchMoreMixin:
-    # FIXME: Use parent for calls to fetch_more can_fetch_more
-    # and also insert items from db map cache in case they were already fetched
-
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self._natural_fetch_parent = ItemTypeFetchParent(self.fetch_item_type)
+        self._natural_fetch_parent = FlexibleFetchParent(
+            self.fetch_item_type,
+            handle_items_added=self.handle_items_added,
+            handle_items_removed=self.handle_items_removed,
+            handle_items_updated=self.handle_items_updated,
+            accepts_item=self.accepts_item,
+        )
+
+    def tear_down(self):
+        super().tear_down()
+        self._natural_fetch_parent.set_obsolete(True)
 
     @property
     def fetch_item_type(self):
@@ -150,14 +170,65 @@ class FetchMoreMixin:
         yield self._natural_fetch_parent
 
     def can_fetch_more(self):
-        return any(self.db_mngr.can_fetch_more(self.db_map, parent) for parent in self._fetch_parents())
+        result = False
+        for parent in self._fetch_parents():
+            result |= self.db_mngr.can_fetch_more(self.db_map, parent)
+        return result
 
     def fetch_more(self):
         for parent in self._fetch_parents():
             self.db_mngr.fetch_more(self.db_map, parent)
 
+    def _make_child(self, id_):
+        raise NotImplementedError()
 
-class StandardDBItem(SortsChildrenMixin, StandardTreeItem):
+    def accepts_item(self, item, db_map):
+        return True
+
+    def handle_items_added(self, db_map_data):
+        """Inserts items at right positions. Items with commit_id are kept sorted.
+        Items without a commit_id are put at the end.
+
+        Args:
+            db_map_data (dict): mapping db_map to list of dict corresponding to db items
+        """
+        db_items = db_map_data.get(self.db_map, [])
+        ids_committed = []
+        ids_uncommitted = []
+        for item in db_items:
+            if item["id"] in self.children_ids:
+                continue
+            ids = ids_committed if item.get("commit_id") is not None else ids_uncommitted
+            ids.append(item["id"])
+        children_committed = [self._make_child(id_) for id_ in ids_committed]
+        children_uncommitted = [self._make_child(id_) for id_ in ids_uncommitted]
+        self.insert_children_sorted(children_committed)
+        self.insert_children(len(self.non_empty_children), children_uncommitted)
+
+    def handle_items_removed(self, db_map_data):
+        ids = {x["id"] for x in db_map_data.get(self.db_map, [])}
+        removed_rows = []
+        for row, leaf_item in enumerate(self.children):
+            if leaf_item.id and leaf_item.id in ids:
+                removed_rows.append(row)
+        for row in sorted(removed_rows, reverse=True):
+            self.remove_children(row, 1)
+
+    def handle_items_updated(self, db_map_data):
+        leaf_items = {leaf_item.id: leaf_item for leaf_item in self.children if leaf_item.id}
+        ids = {x["id"] for x in db_map_data.get(self.db_map, [])}
+        for id_ in set(ids).intersection(leaf_items):
+            leaf_item = leaf_items[id_]
+            leaf_item.handle_updated_in_db()
+            index = self.model.index_from_item(leaf_item)
+            self.model.dataChanged.emit(index, index)
+            if leaf_item.children:
+                top_left = self.model.index_from_item(leaf_item.child(0))
+                bottom_right = self.model.index_from_item(leaf_item.child(-1))
+                self.model.dataChanged.emit(top_left, bottom_right)
+
+
+class StandardDBItem(SortChildrenMixin, StandardTreeItem):
     """An item representing a db."""
 
     def __init__(self, db_map):
@@ -174,17 +245,17 @@ class StandardDBItem(SortsChildrenMixin, StandardTreeItem):
     def item_type(self):
         return "db"
 
-    def data(self, column, role=Qt.DisplayRole):
+    def data(self, column, role=Qt.ItemDataRole.DisplayRole):
         """Shows Spine icon for fun."""
         if column != 0:
             return None
-        if role == Qt.DecorationRole:
+        if role == Qt.ItemDataRole.DecorationRole:
             return QIcon(":/symbols/Spine_symbol.png")
-        if role in (Qt.DisplayRole, Qt.EditRole):
+        if role in (Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.EditRole):
             return self.db_map.codename
 
 
-class RootItem(SortsChildrenMixin, BoldTextMixin, FetchMoreMixin, StandardTreeItem):
+class RootItem(SortChildrenMixin, BoldTextMixin, FetchMoreMixin, StandardTreeItem):
     """A root item."""
 
     @property
@@ -203,6 +274,10 @@ class EmptyChildRootItem(EmptyChildMixin, RootItem):
 
 class LeafItem(StandardTreeItem):
     def __init__(self, identifier=None):
+        """
+        Args:
+            identifier (int, optional): item's database id
+        """
         super().__init__()
         self._id = identifier
 
@@ -238,18 +313,18 @@ class LeafItem(StandardTreeItem):
         raise NotImplementedError()
 
     def header_data(self, column):
-        return self.model.headerData(column, Qt.Horizontal)
+        return self.model.headerData(column, Qt.Orientation.Horizontal)
 
-    def data(self, column, role=Qt.DisplayRole):
-        if role in (Qt.DisplayRole, Qt.EditRole):
+    def data(self, column, role=Qt.ItemDataRole.DisplayRole):
+        if role in (Qt.ItemDataRole.DisplayRole, Qt.ItemDataRole.EditRole):
             data = self.item_data.get(self.header_data(column))
             if data is None:
                 data = ""
             return data
         return super().data(column, role)
 
-    def set_data(self, column, value, role=Qt.EditRole):
-        if role != Qt.EditRole or value == self.data(column, role):
+    def set_data(self, column, value, role=Qt.ItemDataRole.EditRole):
+        if role != Qt.ItemDataRole.EditRole or value == self.data(column, role):
             return False
         if self.id:
             db_item = self._make_item_to_update(column, value)
@@ -274,16 +349,3 @@ class LeafItem(StandardTreeItem):
 
     def can_fetch_more(self):
         return False
-
-
-class ListValueFetchParent(FetchParent):
-    def __init__(self, parameter_value_list_id):
-        super().__init__()
-        self._parameter_value_list_id = parameter_value_list_id
-
-    @property
-    def fetch_item_type(self):
-        return "list_value"
-
-    def filter_query(self, query, subquery, db_map):
-        return query.filter(subquery.c.parameter_value_list_id == self._parameter_value_list_id)
