@@ -17,7 +17,7 @@ Classes for custom QDialogs to add items to databases.
 """
 
 from itertools import product
-from PySide2.QtWidgets import (
+from PySide6.QtWidgets import (
     QHBoxLayout,
     QVBoxLayout,
     QGridLayout,
@@ -36,8 +36,8 @@ from PySide2.QtWidgets import (
     QSplitter,
     QDialogButtonBox,
 )
-from PySide2.QtCore import Slot, Qt, QSize, QModelIndex
-from PySide2.QtGui import QIcon
+from PySide6.QtCore import Slot, Qt, QSize, QModelIndex
+from PySide6.QtGui import QIcon
 from ...mvcmodels.empty_row_model import EmptyRowModel
 from ...mvcmodels.compound_table_model import CompoundTableModel
 from ...mvcmodels.minimal_table_model import MinimalTableModel
@@ -55,7 +55,6 @@ from .manage_items_dialogs import (
     ManageItemsDialog,
     ManageItemsDialogBase,
 )
-from ...helpers import default_icon_id
 from ...spine_db_commands import AgedUndoCommand, AddItemsCommand, RemoveItemsCommand
 
 
@@ -89,16 +88,16 @@ class AddReadyRelationshipsDialog(ManageItemsDialogBase):
         return QTableWidget(self)
 
     def populate_table_view(self):
-        object_class_name_list = self.relationship_class["object_class_name_list"].split(",")
+        object_class_name_list = self.relationship_class["object_class_name_list"]
         self.table_view.setRowCount(len(self.relationships))
         self.table_view.setColumnCount(len(object_class_name_list) + 1)
-        labels = [""] + object_class_name_list
+        labels = ("",) + object_class_name_list
         self.table_view.setHorizontalHeaderLabels(labels)
         self.table_view.verticalHeader().hide()
         for row, relationship in enumerate(self.relationships):
             item = QTableWidgetItem()
             item.setFlags(Qt.ItemIsEnabled)
-            item.setCheckState(Qt.Checked)
+            item.setCheckState(Qt.CheckState.Checked)
             self.table_view.setItem(row, 0, item)
             for column, object_name in enumerate(relationship):
                 item = QTableWidgetItem(object_name)
@@ -114,7 +113,7 @@ class AddReadyRelationshipsDialog(ManageItemsDialogBase):
 
     def _handle_table_view_cell_clicked(self, row, column):
         item = self.table_view.item(row, 0)
-        check_state = Qt.Unchecked if item.checkState() == Qt.Checked else Qt.Checked
+        check_state = Qt.CheckState.Unchecked if item.checkState() == Qt.CheckState.Checked else Qt.CheckState.Checked
         item.setCheckState(check_state)
 
     def _handle_table_view_current_changed(self, current, _previous):
@@ -125,7 +124,7 @@ class AddReadyRelationshipsDialog(ManageItemsDialogBase):
         super().accept()
         data = []
         for row in range(self.table_view.rowCount()):
-            if self.table_view.item(row, 0).checkState() != Qt.Checked:
+            if self.table_view.item(row, 0).checkState() != Qt.CheckState.Checked:
                 continue
             relationship = self.relationships[row]
             data.append([self.relationship_class["name"], relationship])
@@ -385,7 +384,7 @@ class AddRelationshipClassesDialog(ShowIconColorEditorMixin, GetObjectClassesMix
 
     @Slot(QModelIndex, QModelIndex, list)
     def _handle_model_data_changed(self, top_left, bottom_right, roles):
-        if Qt.EditRole not in roles:
+        if Qt.ItemDataRole.EditRole not in roles:
             return
         top = top_left.row()
         left = top_left.column()
@@ -543,8 +542,7 @@ class AddRelationshipsDialog(AddOrManageRelationshipsDialog):
     def reset_model(self, index):
         """Setup model according to current relationship_class selected in combobox."""
         self.class_name, self.object_class_name_list = self.relationship_class_keys[index]
-        object_class_name_list = self.object_class_name_list.split(",")
-        header = object_class_name_list + ['relationship name', 'databases']
+        header = self.object_class_name_list + ('relationship name', 'databases')
         self.model.set_horizontal_header_labels(header)
         default_db_maps = [
             db_map
@@ -559,7 +557,7 @@ class AddRelationshipsDialog(AddOrManageRelationshipsDialog):
 
     @Slot(QModelIndex, QModelIndex, list)
     def _handle_model_data_changed(self, top_left, bottom_right, roles):
-        if Qt.EditRole not in roles:
+        if Qt.ItemDataRole.EditRole not in roles:
             return
         header = self.model.horizontal_header_labels()
         top = top_left.row()
@@ -609,7 +607,6 @@ class AddRelationshipsDialog(AddOrManageRelationshipsDialog):
                 rel_cls = relationship_classes[self.class_name, self.object_class_name_list]
                 class_id = rel_cls["id"]
                 object_class_id_list = rel_cls["object_class_id_list"]
-                object_class_id_list = [int(x) for x in object_class_id_list.split(",")]
                 objects = self.db_map_obj_lookup[db_map]
                 object_id_list = list()
                 for object_class_id, object_name in zip(object_class_id_list, object_name_list):
@@ -640,7 +637,6 @@ class ManageRelationshipsDialog(AddOrManageRelationshipsDialog):
             parent_item (MultiDBTreeItem)
             db_mngr (SpineDBManager): the manager to do the removal
             *db_maps: DiffDatabaseMapping instances
-            relationship_class_key (str, optional): relationships class name, object_class name list string.
         """
         super().__init__(parent, db_mngr, *db_maps)
         self.setWindowTitle("Manage relationships")
@@ -716,9 +712,11 @@ class ManageRelationshipsDialog(AddOrManageRelationshipsDialog):
     @Slot(bool)
     def add_relationships(self, checked=True):
         object_names = [[item.text(0) for item in wg.selectedItems()] for wg in self.splitter_widgets()]
-        candidate = list(product(*object_names))
-        existing = self.new_items_model._main_data + self.existing_items_model._main_data
-        to_add = list(set(candidate) - set(existing))
+        candidate = set(product(*object_names))
+        existing = {
+            tuple(objects) for objects in self.new_items_model._main_data + self.existing_items_model._main_data
+        }
+        to_add = candidate - existing
         count = len(to_add)
         self.new_items_model.insertRows(0, count)
         self.new_items_model._main_data[0:count] = to_add
@@ -728,7 +726,7 @@ class ManageRelationshipsDialog(AddOrManageRelationshipsDialog):
     def reset_model(self, index):
         """Setup model according to current relationship_class selected in combobox."""
         self.class_name, self.object_class_name_list = self.relationship_class_keys[index]
-        object_class_name_list = self.object_class_name_list.split(",")
+        object_class_name_list = self.object_class_name_list
         self.model.set_horizontal_header_labels(object_class_name_list)
         self.existing_items_model.set_horizontal_header_labels(object_class_name_list)
         self.new_items_model.set_horizontal_header_labels(object_class_name_list)
@@ -741,9 +739,9 @@ class ManageRelationshipsDialog(AddOrManageRelationshipsDialog):
             for relationship in self.db_mngr.get_items_by_field(
                 db_map, "relationship", "class_id", rel_cls["id"], only_visible=False
             ):
-                key = tuple(relationship["object_name_list"].split(","))
+                key = relationship["object_name_list"]
                 self.relationship_ids[key] = relationship["id"]
-        existing_items = sorted(self.relationship_ids)
+        existing_items = sorted(map(list, self.relationship_ids))
         self.existing_items_model.reset_model(existing_items)
         self.model.refresh()
         self.model.modelReset.emit()
@@ -751,7 +749,7 @@ class ManageRelationshipsDialog(AddOrManageRelationshipsDialog):
             wg.deleteLater()
         for name in object_class_name_list:
             tree_widget = QTreeWidget(self)
-            tree_widget.setSelectionMode(QAbstractItemView.ExtendedSelection)
+            tree_widget.setSelectionMode(QAbstractItemView.SelectionMode.ExtendedSelection)
             tree_widget.setColumnCount(1)
             tree_widget.setIndentation(0)
             header_item = QTreeWidgetItem([name])
@@ -786,7 +784,9 @@ class ManageRelationshipsDialog(AddOrManageRelationshipsDialog):
     @Slot()
     def accept(self):
         """Collect info from dialog and try to add items."""
-        keys_to_remove = set(self.relationship_ids) - set(self.existing_items_model._main_data)
+        keys_to_remove = set(self.relationship_ids) - {
+            tuple(objects) for objects in self.existing_items_model._main_data
+        }
         to_remove = [self.relationship_ids[key] for key in keys_to_remove]
         self.db_mngr.remove_items({self.db_map: {"relationship": to_remove}})
         to_add = [[self.class_name, object_name_list] for object_name_list in self.new_items_model._main_data]
@@ -820,12 +820,12 @@ class ObjectGroupDialogBase(QDialog):
         header_layout.addWidget(self.db_combo_box)
         self.non_members_tree = QTreeWidget(self)
         self.non_members_tree.setHeaderLabel("Non members")
-        self.non_members_tree.setSelectionMode(QTreeWidget.ExtendedSelection)
+        self.non_members_tree.setSelectionMode(QTreeWidget.SelectionMode.ExtendedSelection)
         self.non_members_tree.setColumnCount(1)
         self.non_members_tree.setIndentation(0)
         self.members_tree = QTreeWidget(self)
         self.members_tree.setHeaderLabel("Members")
-        self.members_tree.setSelectionMode(QTreeWidget.ExtendedSelection)
+        self.members_tree.setSelectionMode(QTreeWidget.SelectionMode.ExtendedSelection)
         self.members_tree.setColumnCount(1)
         self.members_tree.setIndentation(0)
         self.add_button = QToolButton()
@@ -847,7 +847,7 @@ class ObjectGroupDialogBase(QDialog):
         vertical_button_layout.addWidget(self.remove_button)
         vertical_button_layout.addStretch()
         self.button_box = QDialogButtonBox(self)
-        self.button_box.setStandardButtons(QDialogButtonBox.Cancel | QDialogButtonBox.Ok)
+        self.button_box.setStandardButtons(QDialogButtonBox.StandardButton.Cancel | QDialogButtonBox.StandardButton.Ok)
         layout = QGridLayout(self)
         layout.addWidget(self.header_widget, 0, 0, 1, 3, Qt.AlignHCenter)
         layout.addWidget(self.non_members_tree, 1, 0)
@@ -1025,6 +1025,6 @@ class ManageMembersDialog(ObjectGroupDialogBase):
         if items_to_add:
             AddItemsCommand(self.db_mngr, self.db_map, items_to_add, "entity_group", parent=macro)
         if ids_to_remove:
-            RemoveItemsCommand(self.db_mngr, self.db_map, {"entity_group": ids_to_remove}, parent=macro)
+            RemoveItemsCommand(self.db_mngr, self.db_map, ids_to_remove, "entity_group", parent=macro)
         self.db_mngr.undo_stack[self.db_map].push(macro)
         super().accept()
