@@ -17,29 +17,25 @@ Custom item delegates.
 """
 
 from numbers import Number
-from PySide2.QtCore import QModelIndex, QPoint, Qt, Signal
-from PySide2.QtWidgets import QStyledItemDelegate, QComboBox
-from PySide2.QtGui import QFontMetrics
+from PySide6.QtCore import QModelIndex, QPoint, Qt, Signal
+from PySide6.QtWidgets import QStyledItemDelegate, QComboBox
+from PySide6.QtGui import QFontMetrics, QFont
 from spinedb_api import to_database
 from spinedb_api.parameter_value import join_value_and_type
-from ..mvcmodels.single_parameter_models import SingleParameterModel
 from ...widgets.custom_editors import CustomLineEditor, SearchBarEditor, CheckListEditor, ParameterValueLineEditor
 from ...mvcmodels.shared import PARSED_ROLE, DB_MAP_ROLE
 from ...widgets.custom_delegates import CheckBoxDelegate, RankDelegate
 from ...helpers import object_icon
 from ..mvcmodels.metadata_table_model_base import Column as MetadataColumn
 
-# FIXME: only_visible=False ???
-
 
 class RelationshipPivotTableDelegate(CheckBoxDelegate):
-
     data_committed = Signal(QModelIndex, object)
 
     def __init__(self, parent):
         """
         Args:
-            parent (SpineDBEditor)
+            parent (SpineDBEditor): parent widget, i.e. the database editor
         """
         super().__init__(parent)
         self.data_committed.connect(parent._set_model_data)
@@ -47,11 +43,14 @@ class RelationshipPivotTableDelegate(CheckBoxDelegate):
     @staticmethod
     def _is_relationship_index(index):
         """
-        Checks whether or not the given index corresponds to a relationship,
+        Checks whether the given index corresponds to a relationship,
         in which case we need to use the check box delegate.
 
+        Args:
+            index (QModelIndex): index to check
+
         Returns:
-            bool
+            bool: True if index corresponds to relationship, False otherwise
         """
         return index.model().sourceModel().index_in_data(index)
 
@@ -80,18 +79,17 @@ class RelationshipPivotTableDelegate(CheckBoxDelegate):
         if self._is_relationship_index(index):
             return super().createEditor(parent, option, index)
         editor = CustomLineEditor(parent)
-        editor.set_data(index.data(Qt.EditRole))
+        editor.set_data(index.data(Qt.ItemDataRole.EditRole))
         return editor
 
 
 class ScenarioAlternativeTableDelegate(RankDelegate):
-
     data_committed = Signal(QModelIndex, object)
 
     def __init__(self, parent):
         """
         Args:
-            parent (SpineDBEditor)
+            parent (SpineDBEditor): database editor
         """
         super().__init__(parent)
         self.data_committed.connect(parent._set_model_data)
@@ -132,19 +130,18 @@ class ScenarioAlternativeTableDelegate(RankDelegate):
         if self._is_scenario_alternative_index(index):
             return super().createEditor(parent, option, index)
         editor = CustomLineEditor(parent)
-        editor.set_data(index.data(Qt.EditRole))
+        editor.set_data(index.data(Qt.ItemDataRole.EditRole))
         return editor
 
 
 class ParameterPivotTableDelegate(QStyledItemDelegate):
-
     parameter_value_editor_requested = Signal(QModelIndex)
     data_committed = Signal(QModelIndex, object)
 
     def __init__(self, parent):
         """
         Args:
-            parent (SpineDBEditor)
+            parent (SpineDBEditor): parent widget, i.e. database editor
         """
         super().__init__(parent)
         self.data_committed.connect(parent._set_model_data)
@@ -170,7 +167,7 @@ class ParameterPivotTableDelegate(QStyledItemDelegate):
             self.parameter_value_editor_requested.emit(index.model().mapToSource(index))
             return None
         editor = CustomLineEditor(parent)
-        editor.set_data(index.data(Qt.EditRole))
+        editor.set_data(index.data(Qt.ItemDataRole.EditRole))
         return editor
 
 
@@ -203,7 +200,7 @@ class ParameterValueElementDelegate(QStyledItemDelegate):
         Returns:
             ParameterValueLineEditor: editor widget
         """
-        value = index.data(Qt.EditRole)
+        value = index.data(Qt.ItemDataRole.EditRole)
         if value is None or isinstance(value, (Number, str)) and not isinstance(value, bool):
             editor = ParameterValueLineEditor(parent)
             editor.set_data(value)
@@ -263,8 +260,8 @@ class DatabaseNameDelegate(ParameterDelegate):
     def createEditor(self, parent, option, index):
         """Returns editor."""
         editor = SearchBarEditor(self.parent(), parent)
-        editor.set_data(index.data(Qt.DisplayRole), [x.codename for x in self.db_mngr.db_maps])
-        editor.data_committed.connect(lambda editor=editor, index=index: self._close_editor(editor, index))
+        editor.set_data(index.data(Qt.ItemDataRole.DisplayRole), [x.codename for x in self.db_mngr.db_maps])
+        editor.data_committed.connect(lambda *_: self._close_editor(editor, index))
         return editor
 
 
@@ -274,6 +271,11 @@ class ParameterValueOrDefaultValueDelegate(ParameterDelegate):
     parameter_value_editor_requested = Signal(QModelIndex)
 
     def __init__(self, parent, db_mngr):
+        """
+        Args:
+            parent (QWidget): parent widget
+            db_mngr (SpineDatabaseManager): database manager
+        """
         super().__init__(parent, db_mngr)
         self._db_value_list_lookup = {}
 
@@ -286,8 +288,16 @@ class ParameterValueOrDefaultValueDelegate(ParameterDelegate):
             value = join_value_and_type(*to_database(display_value))
         self.data_committed.emit(index, value)
 
-    def _create_or_request_parameter_value_editor(self, parent, option, index, db_map):
-        """Emits the signal to request a standalone `ParameterValueEditor` from parent widget."""
+    def _create_or_request_parameter_value_editor(self, parent, index):
+        """Emits the signal to request a standalone `ParameterValueEditor` from parent widget.
+
+        Args:
+            parent (QWidget): editor's parent widget
+            index (QModelIndex): index to parameter value model
+
+        Returns:
+            ParameterValueLineEditor: editor or None if ``parameter_value_editor_request`` signal was emitted
+        """
         value = index.data(PARSED_ROLE)
         if value is None or isinstance(value, (Number, str)) and not isinstance(value, bool):
             editor = ParameterValueLineEditor(parent)
@@ -295,6 +305,18 @@ class ParameterValueOrDefaultValueDelegate(ParameterDelegate):
             return editor
         self.parameter_value_editor_requested.emit(index)
 
+    def _get_value_list_id(self, index, db_map):
+        """Returns a value list id for the given index and db_map.
+
+        Args:
+            index (QModelIndex): value list's index
+            db_map (DiffDatabaseMapping): database mapping
+
+        Returns:
+            int: value list id
+        """
+        raise NotImplementedError()
+
     def createEditor(self, parent, option, index):
         """If the parameter has associated a value list, returns a SearchBarEditor.
         Otherwise returns or requests a dedicated parameter_value editor.
@@ -306,24 +328,24 @@ class ParameterValueOrDefaultValueDelegate(ParameterDelegate):
         value_list_id = self._get_value_list_id(index, db_map)
         if value_list_id:
             display_value_list = self.db_mngr.get_parameter_value_list(
-                db_map, value_list_id, Qt.DisplayRole, only_visible=False
+                db_map, value_list_id, Qt.ItemDataRole.DisplayRole, only_visible=False
             )
             db_value_list = self.db_mngr.get_parameter_value_list(
-                db_map, value_list_id, Qt.EditRole, only_visible=False
+                db_map, value_list_id, Qt.ItemDataRole.EditRole, only_visible=False
             )
             self._db_value_list_lookup = dict(zip(display_value_list, db_value_list))
             editor = SearchBarEditor(self.parent(), parent)
             editor.set_data(index.data(), self._db_value_list_lookup)
-            editor.data_committed.connect(lambda editor=editor, index=index: self._close_editor(editor, index))
+            editor.data_committed.connect(lambda *_: self._close_editor(editor, index))
             return editor
-        return self._create_or_request_parameter_value_editor(parent, option, index, db_map)
+        return self._create_or_request_parameter_value_editor(parent, index)
 
 
 class ParameterDefaultValueDelegate(ParameterValueOrDefaultValueDelegate):
     """A delegate for the default value."""
 
     def _get_value_list_id(self, index, db_map):
-        """Returns a value list item for the given index and db_map."""
+        """See base class"""
         h = index.model().header.index
         value_list_name = index.sibling(index.row(), h("value_list_name")).data()
         value_lists = self.db_mngr.get_items_by_field(
@@ -337,7 +359,7 @@ class ParameterValueDelegate(ParameterValueOrDefaultValueDelegate):
     """A delegate for the parameter_value."""
 
     def _get_value_list_id(self, index, db_map):
-        """Returns a value list item for the given index and db_map."""
+        """See base class."""
         h = index.model().header.index
         parameter_name = index.sibling(index.row(), h("parameter_name")).data()
         parameters = self.db_mngr.get_items_by_field(
@@ -363,8 +385,8 @@ class ValueListDelegate(ParameterDelegate):
             return None
         editor = SearchBarEditor(self.parent(), parent)
         name_list = [x["name"] for x in self.db_mngr.get_items(db_map, "parameter_value_list", only_visible=False)]
-        editor.set_data(index.data(Qt.EditRole), name_list)
-        editor.data_committed.connect(lambda editor=editor, index=index: self._close_editor(editor, index))
+        editor.set_data(index.data(Qt.ItemDataRole.EditRole), name_list)
+        editor.data_committed.connect(lambda *_: self._close_editor(editor, index))
         return editor
 
 
@@ -378,8 +400,8 @@ class ObjectClassNameDelegate(ParameterDelegate):
             return None
         editor = SearchBarEditor(self.parent(), parent)
         object_classes = self.db_mngr.get_items(db_map, "object_class", only_visible=False)
-        editor.set_data(index.data(Qt.EditRole), [x["name"] for x in object_classes])
-        editor.data_committed.connect(lambda editor=editor, index=index: self._close_editor(editor, index))
+        editor.set_data(index.data(Qt.ItemDataRole.EditRole), [x["name"] for x in object_classes])
+        editor.data_committed.connect(lambda *_: self._close_editor(editor, index))
         return editor
 
 
@@ -393,8 +415,8 @@ class RelationshipClassNameDelegate(ParameterDelegate):
             return None
         editor = SearchBarEditor(self.parent(), parent)
         relationship_classes = self.db_mngr.get_items(db_map, "relationship_class", only_visible=False)
-        editor.set_data(index.data(Qt.EditRole), [x["name"] for x in relationship_classes])
-        editor.data_committed.connect(lambda editor=editor, index=index: self._close_editor(editor, index))
+        editor.set_data(index.data(Qt.ItemDataRole.EditRole), [x["name"] for x in relationship_classes])
+        editor.data_committed.connect(lambda *_: self._close_editor(editor, index))
         return editor
 
 
@@ -415,8 +437,8 @@ class ParameterNameDelegate(ParameterDelegate):
         else:
             parameter_definitions = self.db_mngr.get_items(db_map, "parameter_definition", only_visible=False)
         name_list = list({x["parameter_name"]: None for x in parameter_definitions})
-        editor.set_data(index.data(Qt.EditRole), name_list)
-        editor.data_committed.connect(lambda editor=editor, index=index: self._close_editor(editor, index))
+        editor.set_data(index.data(Qt.ItemDataRole.EditRole), name_list)
+        editor.data_committed.connect(lambda *_: self._close_editor(editor, index))
         return editor
 
 
@@ -435,8 +457,8 @@ class ObjectNameDelegate(ParameterDelegate):
         else:
             objects = self.db_mngr.get_items(db_map, "object", only_visible=False)
         name_list = list({x["name"]: None for x in objects})
-        editor.set_data(index.data(Qt.EditRole), name_list)
-        editor.data_committed.connect(lambda editor=editor, index=index: self._close_editor(editor, index))
+        editor.set_data(index.data(Qt.ItemDataRole.EditRole), name_list)
+        editor.data_committed.connect(lambda *_: self._close_editor(editor, index))
         return editor
 
 
@@ -450,8 +472,8 @@ class AlternativeNameDelegate(ParameterDelegate):
             return None
         editor = SearchBarEditor(self.parent(), parent)
         name_list = [x["name"] for x in self.db_mngr.get_items(db_map, "alternative", only_visible=False)]
-        editor.set_data(index.data(Qt.EditRole), name_list)
-        editor.data_committed.connect(lambda editor=editor, index=index: self._close_editor(editor, index))
+        editor.set_data(index.data(Qt.ItemDataRole.EditRole), name_list)
+        editor.data_committed.connect(lambda *_: self._close_editor(editor, index))
         return editor
 
 
@@ -468,7 +490,7 @@ class ObjectNameListDelegate(ParameterDelegate):
         relationship_class_id = index.model().get_entity_class_id(index, db_map)
         if not relationship_class_id:
             editor = CustomLineEditor(parent)
-            editor.set_data(index.data(Qt.EditRole))
+            editor.set_data(index.data(Qt.ItemDataRole.EditRole))
             return editor
         self.object_name_list_editor_requested.emit(index, relationship_class_id, db_map)
 
@@ -479,10 +501,24 @@ class ToolFeatureDelegate(QStyledItemDelegate):
     data_committed = Signal(QModelIndex, object)
 
     def __init__(self, *args, **kwargs):
+        """
+        Args:
+            *args: arguments passed to QStyledItemDelegate
+            **kwargs: keyword arguments passed to QStyledItemDelegate
+        """
         super().__init__(*args, **kwargs)
         self._feature_ids = {}
 
     def _get_names(self, item, model):
+        """Collects names under given tree item.
+
+        Args:
+            item (Standard tree item): A non-leaf item
+            model (ToolFeatureModel): model
+
+        Returns:
+            list of str: names or None if nothing was found
+        """
         if item.item_type == "feature":
             names = model.get_all_feature_names(item.db_map)
             if not names:
@@ -509,7 +545,16 @@ class ToolFeatureDelegate(QStyledItemDelegate):
 
     @staticmethod
     def _get_index_data(item, index):
-        index_data = index.data(Qt.EditRole)
+        """Returns formatted model data from given index.
+
+        Args:
+            item (StandardTreeItem): item corresponding to index
+            index (QModelIndex): index
+
+        Returns:
+            str: index data
+        """
+        index_data = index.data(Qt.ItemDataRole.EditRole)
         if item.item_type == "tool_feature required":
             return index_data.split(": ")[1]
         return index_data
@@ -541,10 +586,10 @@ class ToolFeatureDelegate(QStyledItemDelegate):
             if names is None:
                 return None
             editor.set_data(index_data, names)
-            editor.data_committed.connect(lambda editor=editor, index=index: self._close_editor(editor, index))
+            editor.data_committed.connect(lambda *_: self._close_editor(editor, index))
         else:
             editor = CustomLineEditor(parent)
-            editor.set_data(index.data(Qt.EditRole))
+            editor.set_data(index.data(Qt.ItemDataRole.EditRole))
         return editor
 
     def updateEditorGeometry(self, editor, option, index):
@@ -552,29 +597,95 @@ class ToolFeatureDelegate(QStyledItemDelegate):
         item = index.model().item_from_index(index)
         if item.item_type in ("feature", "tool_feature", "tool_feature required", "tool_feature_method"):
             if item.item_type == "tool_feature required":
-                dx = QFontMetrics(index.data(Qt.FontRole)).horizontalAdvance("required:")
+                font = index.data(Qt.ItemDataRole.FontRole)
+                if not font:
+                    font = QFont()  # app default
+                dx = QFontMetrics(font).horizontalAdvance("required:")
                 editor.set_base_offset(QPoint(dx, 0))
             editor.update_geometry(option)
 
     def _close_editor(self, editor, index):
-        """Closes editor. Needed by SearchBarEditor."""
+        """Closes editor.
+
+        Needed by SearchBarEditor.
+
+        Args:
+            editor (QWidget): editor widget
+            index (QModelIndex): index that is being edited
+        """
         self.closeEditor.emit(editor)
         self.setModelData(editor, index.model(), index)
 
 
-class AlternativeScenarioDelegate(QStyledItemDelegate):
-    """A delegate for the alternative scenario tree."""
+class AlternativeDelegate(QStyledItemDelegate):
+    """A delegate for the alternative tree."""
 
     data_committed = Signal(QModelIndex, object)
 
     def __init__(self, *args, **kwargs):
+        """
+        Args:
+            *args: arguments passed to QStyledItemDelegate
+            **kwargs: keyword arguments passed to QStyledItemDelegate
+        """
         super().__init__(*args, **kwargs)
         self._alternative_ids = {}
 
     def setModelData(self, editor, model, index):
         """Send signal."""
         item = index.model().item_from_index(index)
-        index_data = self._get_index_data(item, index)
+        index_data = index.data(Qt.ItemDataRole.EditRole)
+        editor_data = editor.data()
+        if editor_data == index_data:
+            return
+        self.data_committed.emit(index, editor_data)
+
+    def setEditorData(self, editor, index):
+        """Do nothing. We're setting editor data right away in createEditor."""
+
+    def createEditor(self, parent, option, index):
+        """Returns editor."""
+        model = index.model()
+        item = model.item_from_index(index)
+        editor = CustomLineEditor(parent)
+        editor.set_data(index.data(Qt.ItemDataRole.EditRole))
+        return editor
+
+    def updateEditorGeometry(self, editor, option, index):
+        super().updateEditorGeometry(editor, option, index)
+        item = index.model().item_from_index(index)
+
+    def _close_editor(self, editor, index):
+        """Closes editor.
+
+        Needed by SearchBarEditor.
+
+        Args:
+            editor (QWidget): editor widget
+            index (QModelIndex): index that is being edited
+        """
+        self.closeEditor.emit(editor)
+        self.setModelData(editor, index.model(), index)
+
+
+class ScenarioDelegate(QStyledItemDelegate):
+    """A delegate for the scenario tree."""
+
+    data_committed = Signal(QModelIndex, object)
+
+    def __init__(self, *args, **kwargs):
+        """
+        Args:
+            *args: arguments passed to QStyledItemDelegate
+            **kwargs: keyword arguments passed to QStyledItemDelegate
+        """
+        super().__init__(*args, **kwargs)
+        self._alternative_ids = {}
+
+    def setModelData(self, editor, model, index):
+        """Send signal."""
+        item = index.model().item_from_index(index)
+        index_data = index.data(Qt.ItemDataRole.EditRole)
         editor_data = editor.data()
         if editor_data == index_data:
             return
@@ -587,52 +698,55 @@ class AlternativeScenarioDelegate(QStyledItemDelegate):
     def setEditorData(self, editor, index):
         """Do nothing. We're setting editor data right away in createEditor."""
 
-    def _get_names(self, item, model):
-        if item.item_type == "scenario_alternative":
-            self._alternative_ids = {
-                x["name"]: x["id"]
-                for x in item.db_mngr.get_items(item.db_map, "alternative", only_visible=False)
-                if x["id"] not in item.parent_item.alternative_id_list
-            }
-            return list(self._alternative_ids)
-        if item.item_type == "scenario active":
-            return ["yes", "no"]
+    def _get_names(self, item):
+        """Collects available alternative names avoiding duplicates in a scenario.
 
-    @staticmethod
-    def _get_index_data(item, index):
-        index_data = index.data(Qt.EditRole)
-        if item.item_type == "scenario active":
-            return index_data.split(": ")[1]
-        return index_data
+        Excludes alternatives that are already in the scenario
+
+        Args:
+            item (ScenarioAlternativeItem): one of scenario's scenario alternatives
+
+        Returns:
+            list of str: available alternative names
+        """
+        excluded_ids = set(item.parent_item.alternative_id_list)
+        self._alternative_ids = {
+            x["name"]: x["id"]
+            for x in item.db_mngr.get_items(item.db_map, "alternative", only_visible=False)
+            if x["id"] not in excluded_ids
+        }
+        return list(self._alternative_ids)
 
     def createEditor(self, parent, option, index):
         """Returns editor."""
         model = index.model()
         item = model.item_from_index(index)
-        if item.item_type in ("scenario active", "scenario_alternative"):
+        if item.item_type == "scenario_alternative":
             editor = SearchBarEditor(self.parent(), parent)
-            index_data = self._get_index_data(item, index)
-            names = self._get_names(item, model)
-            if names is None:
-                return None
+            index_data = index.data(Qt.ItemDataRole.EditRole)
+            names = self._get_names(item)
             editor.set_data(index_data, names)
-            editor.data_committed.connect(lambda editor=editor, index=index: self._close_editor(editor, index))
+            editor.data_committed.connect(lambda *_: self._close_editor(editor, index))
         else:
             editor = CustomLineEditor(parent)
-            editor.set_data(index.data(Qt.EditRole))
+            editor.set_data(index.data(Qt.ItemDataRole.EditRole))
         return editor
 
     def updateEditorGeometry(self, editor, option, index):
         super().updateEditorGeometry(editor, option, index)
         item = index.model().item_from_index(index)
-        if item.item_type in ("scenario active", "scenario_alternative"):
-            if item.item_type == "scenario active":
-                dx = QFontMetrics(index.data(Qt.FontRole)).horizontalAdvance("active:")
-                editor.set_base_offset(QPoint(dx, 0))
+        if item.item_type == "scenario_alternative":
             editor.update_geometry(option)
 
     def _close_editor(self, editor, index):
-        """Closes editor. Needed by SearchBarEditor."""
+        """Closes editor.
+
+        Needed by SearchBarEditor.
+
+        Args:
+            editor (QWidget): editor widget
+            index (QModelIndex): index that is being edited
+        """
         self.closeEditor.emit(editor)
         self.setModelData(editor, index.model(), index)
 
@@ -661,7 +775,7 @@ class ParameterValueListDelegate(QStyledItemDelegate):
         item = model.item_from_index(index)
         if item.item_type != "list_value":
             editor = CustomLineEditor(parent)
-            editor.set_data(index.data(Qt.EditRole))
+            editor.set_data(index.data(Qt.ItemDataRole.EditRole))
             return editor
         value = index.data(PARSED_ROLE)
         if value is None or isinstance(value, (Number, str)) and not isinstance(value, bool):
@@ -671,7 +785,14 @@ class ParameterValueListDelegate(QStyledItemDelegate):
         self.parameter_value_editor_requested.emit(index)
 
     def _close_editor(self, editor, index):
-        """Closes editor."""
+        """Closes editor.
+
+        Needed by SearchBarEditor.
+
+        Args:
+            editor (QWidget): editor widget
+            index (QModelIndex): index that is being edited
+        """
         self.closeEditor.emit(editor)
         self.setModelData(editor, index.model(), index)
 
@@ -685,9 +806,17 @@ class ManageItemsDelegate(QStyledItemDelegate):
         """Send signal."""
         self.data_committed.emit(index, editor.data())
 
-    def close_editor(self, editor, index, model):
+    def close_editor(self, editor, index):
+        """Closes editor.
+
+        Needed by SearchBarEditor.
+
+        Args:
+            editor (QWidget): editor widget
+            index (QModelIndex): index that is being edited
+        """
         self.closeEditor.emit(editor)
-        self.setModelData(editor, model, index)
+        self.setModelData(editor, index.model(), index)
 
     def updateEditorGeometry(self, editor, option, index):
         super().updateEditorGeometry(editor, option, index)
@@ -695,15 +824,28 @@ class ManageItemsDelegate(QStyledItemDelegate):
             editor.update_geometry(option)
 
     def connect_editor_signals(self, editor, index):
-        """Connect editor signals if necessary."""
-        if isinstance(editor, SearchBarEditor):
-            model = index.model()
-            editor.data_committed.connect(lambda e=editor, i=index, m=model: self.close_editor(e, i, m))
+        """Connect editor signals if necessary.
 
-    def _create_database_editor(self, parent, option, index):
+        Args:
+            editor (QWidget): editor widget
+            index (QModelIndex): index being edited
+        """
+        if isinstance(editor, SearchBarEditor):
+            editor.data_committed.connect(lambda *_: self.close_editor(editor, index))
+
+    def _create_database_editor(self, parent, index):
+        """Creates an editor.
+
+        Args:
+            parent (QWidget): parent widget
+            index (QModelIndex): index being edited
+
+        Returns:
+            QWidget: editor
+        """
         editor = CheckListEditor(parent)
         all_databases = self.parent().all_databases(index.row())
-        databases = index.data(Qt.DisplayRole).split(",")
+        databases = index.data(Qt.ItemDataRole.DisplayRole).split(",")
         editor.set_data(all_databases, databases)
         return editor
 
@@ -711,10 +853,10 @@ class ManageItemsDelegate(QStyledItemDelegate):
         """Returns an editor."""
         header = index.model().horizontal_header_labels()
         if header[index.column()] == 'databases':
-            editor = self._create_database_editor(parent, option, index)
+            editor = self._create_database_editor(parent, index)
         else:
             editor = CustomLineEditor(parent)
-            editor.set_data(index.data(Qt.EditRole))
+            editor.set_data(index.data(Qt.ItemDataRole.EditRole))
         self.connect_editor_signals(editor, index)
         return editor
 
@@ -742,10 +884,10 @@ class ManageObjectClassesDelegate(ManageEntityClassesDelegate):
             self.icon_color_editor_requested.emit(index)
             editor = None
         elif header[index.column()] == 'databases':
-            editor = self._create_database_editor(parent, option, index)
+            editor = self._create_database_editor(parent, index)
         else:
             editor = CustomLineEditor(parent)
-            editor.set_data(index.data(Qt.EditRole))
+            editor.set_data(index.data(Qt.ItemDataRole.EditRole))
         self.connect_editor_signals(editor, index)
         return editor
 
@@ -759,12 +901,12 @@ class ManageObjectsDelegate(ManageItemsDelegate):
         if header[index.column()] == 'object_class name':
             editor = SearchBarEditor(parent)
             object_class_name_list = self.parent().object_class_name_list(index.row())
-            editor.set_data(index.data(Qt.EditRole), object_class_name_list)
+            editor.set_data(index.data(Qt.ItemDataRole.EditRole), object_class_name_list)
         elif header[index.column()] == 'databases':
-            editor = self._create_database_editor(parent, option, index)
+            editor = self._create_database_editor(parent, index)
         else:
             editor = CustomLineEditor(parent)
-            editor.set_data(index.data(Qt.EditRole))
+            editor.set_data(index.data(Qt.ItemDataRole.EditRole))
         self.connect_editor_signals(editor, index)
         return editor
 
@@ -782,13 +924,13 @@ class ManageRelationshipClassesDelegate(ManageEntityClassesDelegate):
             editor = None
         elif header[index.column()] in ('relationship_class name', 'description'):
             editor = CustomLineEditor(parent)
-            editor.set_data(index.data(Qt.EditRole))
+            editor.set_data(index.data(Qt.ItemDataRole.EditRole))
         elif header[index.column()] == 'databases':
-            editor = self._create_database_editor(parent, option, index)
+            editor = self._create_database_editor(parent, index)
         else:
             editor = SearchBarEditor(parent)
             object_class_name_list = self.parent().object_class_name_list(index.row())
-            editor.set_data(index.data(Qt.EditRole), object_class_name_list)
+            editor.set_data(index.data(Qt.ItemDataRole.EditRole), object_class_name_list)
         self.connect_editor_signals(editor, index)
         return editor
 
@@ -801,14 +943,14 @@ class ManageRelationshipsDelegate(ManageItemsDelegate):
         header = index.model().horizontal_header_labels()
         if header[index.column()] == 'relationship name':
             editor = CustomLineEditor(parent)
-            data = index.data(Qt.EditRole)
+            data = index.data(Qt.ItemDataRole.EditRole)
             editor.set_data(data)
         elif header[index.column()] == 'databases':
-            editor = self._create_database_editor(parent, option, index)
+            editor = self._create_database_editor(parent, index)
         else:
             editor = SearchBarEditor(parent)
             object_name_list = self.parent().object_name_list(index.row(), index.column())
-            editor.set_data(index.data(Qt.EditRole), object_name_list)
+            editor.set_data(index.data(Qt.ItemDataRole.EditRole), object_name_list)
         self.connect_editor_signals(editor, index)
         return editor
 
@@ -820,7 +962,7 @@ class RemoveEntitiesDelegate(ManageItemsDelegate):
         """Return editor."""
         header = index.model().horizontal_header_labels()
         if header[index.column()] == 'databases':
-            editor = self._create_database_editor(parent, option, index)
+            editor = self._create_database_editor(parent, index)
             self.connect_editor_signals(editor, index)
             return editor
 
