@@ -15,12 +15,11 @@ Custom QTableView classes that support copy-paste and the like.
 :author: M. Marin (KTH)
 :date:   18.5.2018
 """
+
 from dataclasses import replace
-
-from PySide2.QtCore import Qt, Signal, Slot, QTimer, QModelIndex, QPoint
-from PySide2.QtWidgets import QAction, QTableView, QHeaderView, QMenu
-from PySide2.QtGui import QKeySequence
-
+from PySide6.QtCore import Qt, Signal, Slot, QTimer, QModelIndex, QPoint, QItemSelection, QItemSelectionModel
+from PySide6.QtWidgets import QTableView, QMenu
+from PySide6.QtGui import QKeySequence, QAction
 from .scenario_generator import ScenarioGenerator
 from ..mvcmodels.pivot_table_models import (
     ParameterValuePivotTableModel,
@@ -32,7 +31,7 @@ from ..mvcmodels.metadata_table_model_base import Column as MetadataColumn
 from ...widgets.report_plotting_failure import report_plotting_failure
 from ...widgets.plot_widget import PlotWidget, prepare_plot_in_window_menu
 from ...widgets.custom_qtableview import CopyPasteTableView, AutoFilterCopyPasteTableView
-from ...widgets.custom_qwidgets import TitleWidgetAction
+from ...widgets.custom_qwidgets import TitleWidgetAction, ResizingViewMixin
 from ...plotting import (
     PlottingError,
     ParameterTableHeaderSection,
@@ -67,7 +66,7 @@ def _set_parameter_data(index, new_value):
     index.model().setData(index, new_value)
 
 
-class ParameterTableView(AutoFilterCopyPasteTableView):
+class ParameterTableView(ResizingViewMixin, AutoFilterCopyPasteTableView):
     value_column_header: str = NotImplemented
     """Either "default value" or "value". Used to identify the value column for advanced editing and plotting."""
 
@@ -174,7 +173,7 @@ class ParameterTableView(AutoFilterCopyPasteTableView):
         self._menu.addAction("Clear all filters", self._spine_db_editor.clear_all_filters)
         self._menu.addSeparator()
         # Shortcuts
-        remove_rows_action.setShortcut(QKeySequence(Qt.CTRL + Qt.Key_Delete))
+        remove_rows_action.setShortcut(QKeySequence(Qt.CTRL | Qt.Key_Delete))
         remove_rows_action.setShortcutContext(Qt.WidgetShortcut)
         self.addAction(remove_rows_action)
 
@@ -188,7 +187,7 @@ class ParameterTableView(AutoFilterCopyPasteTableView):
         if not index.isValid():
             return
         model = self.model()
-        is_value = model.headerData(index.column(), Qt.Horizontal) == self.value_column_header
+        is_value = model.headerData(index.column(), Qt.Orientation.Horizontal) == self.value_column_header
         self._open_in_editor_action.setEnabled(is_value)
         self._plot_action.setEnabled(is_value)
         if is_value:
@@ -196,7 +195,7 @@ class ParameterTableView(AutoFilterCopyPasteTableView):
             plot_in_window_menu.triggered.connect(self.plot_in_window)
             prepare_plot_in_window_menu(plot_in_window_menu)
             self._menu.insertMenu(self._plot_separator, plot_in_window_menu)
-        self._menu.exec_(event.globalPos())
+        self._menu.exec(event.globalPos())
         if is_value:
             plot_in_window_menu.deleteLater()
 
@@ -238,7 +237,7 @@ class ParameterTableView(AutoFilterCopyPasteTableView):
         selection = self.selectionModel().selection()
         rows = list()
         while not selection.isEmpty():
-            current = selection.takeFirst()
+            current = selection.takeAt(0)
             top = current.top()
             bottom = current.bottom()
             rows += range(top, bottom + 1)
@@ -258,8 +257,7 @@ class ParameterTableView(AutoFilterCopyPasteTableView):
         model.db_mngr.remove_items(db_map_typed_data)
         self.selectionModel().clearSelection()
 
-    def rowsInserted(self, parent, start, end):
-        super().rowsInserted(parent, start, end)
+    def _do_resize(self):
         self.resizeColumnsToContents()
 
     @Slot(QModelIndex, QModelIndex)
@@ -407,7 +405,7 @@ class RelationshipParameterValueTableView(RelationshipParameterTableMixin, Param
         )
 
 
-class PivotTableView(CopyPasteTableView):
+class PivotTableView(ResizingViewMixin, CopyPasteTableView):
     """Custom QTableView class with pivot capabilities.
 
     Uses 'contexts' to provide different UI elements (table headers, context menus,...) depending on what
@@ -416,33 +414,6 @@ class PivotTableView(CopyPasteTableView):
 
     header_changed = Signal()
 
-    def __init__(self, parent=None):
-        """
-        Args:
-            parent (QWidget, optional): parent widget
-        """
-        super().__init__(parent)
-        self.setHorizontalScrollMode(QTableView.ScrollPerPixel)
-        self.setVerticalScrollMode(QTableView.ScrollPerPixel)
-        # NOTE: order of creation of header tables is important for them to stack properly
-        self._left_header_table = CopyPasteTableView(self)
-        self._top_header_table = CopyPasteTableView(self)
-        self._top_left_header_table = CopyPasteTableView(self)
-        self._left_header_table.setObjectName("left")
-        self._top_header_table.setObjectName("top")
-        self._top_left_header_table.setObjectName("top-left")
-        self._spine_db_editor = None
-        self._context = None
-        self._fetch_more_timer = QTimer(self)
-        self._fetch_more_timer.setSingleShot(True)
-        self._fetch_more_timer.setInterval(100)
-        self._fetch_more_timer.timeout.connect(self._fetch_more_visible)
-        self._left_header_table.verticalScrollBar().valueChanged.connect(self.verticalScrollBar().setValue)
-        self.verticalScrollBar().valueChanged.connect(self._left_header_table.verticalScrollBar().setValue)
-        self._top_header_table.horizontalScrollBar().valueChanged.connect(self.horizontalScrollBar().setValue)
-        self.horizontalScrollBar().valueChanged.connect(self._top_header_table.horizontalScrollBar().setValue)
-        self._init_header_tables()
-
     class _ContextBase:
         """Base class for pivot table view's contexts."""
 
@@ -502,7 +473,7 @@ class PivotTableView(CopyPasteTableView):
             """Shows the context menu."""
             self._refresh_selected_indexes()
             self._update_actions_availability()
-            self._menu.exec_(position)
+            self._menu.exec(position)
 
         def _to_selection_lists(self, index):
             """Caches given index to corresponding selected index list.
@@ -604,8 +575,8 @@ class PivotTableView(CopyPasteTableView):
             self._remove_relationships_action = None
             self._plot_action = None
             self._plot_in_window_menu = None
-            horizontal_header = ParameterValuePivotHeaderView(Qt.Horizontal, "columns", view)
-            vertical_header = ParameterValuePivotHeaderView(Qt.Vertical, "rows", view)
+            horizontal_header = ParameterValuePivotHeaderView(Qt.Orientation.Horizontal, "columns", view)
+            vertical_header = ParameterValuePivotHeaderView(Qt.Orientation.Vertical, "rows", view)
             super().__init__(view, db_editor, horizontal_header, vertical_header)
             self._header_selection_lists["parameter"] = self._selected_parameter_indexes
 
@@ -738,8 +709,8 @@ class PivotTableView(CopyPasteTableView):
                 db_editor (SpineDBEditor): database editor
             """
             self._remove_objects_action = None
-            horizontal_header = PivotTableHeaderView(Qt.Horizontal, "columns", view)
-            vertical_header = PivotTableHeaderView(Qt.Vertical, "rows", view)
+            horizontal_header = PivotTableHeaderView(Qt.Orientation.Horizontal, "columns", view)
+            vertical_header = PivotTableHeaderView(Qt.Orientation.Vertical, "rows", view)
             super().__init__(view, db_editor, horizontal_header, vertical_header)
 
         def populate_context_menu(self):
@@ -769,9 +740,9 @@ class PivotTableView(CopyPasteTableView):
             self._toggle_alternatives_checked.triggered.connect(self._toggle_checked_state)
             self._remove_scenarios_action = None
             self._duplicate_scenario_action = None
-            horizontal_header = ScenarioAlternativePivotHeaderView(Qt.Horizontal, "columns", view)
+            horizontal_header = ScenarioAlternativePivotHeaderView(Qt.Orientation.Horizontal, "columns", view)
             horizontal_header.context_menu_requested.connect(self.show_context_menu)
-            vertical_header = ScenarioAlternativePivotHeaderView(Qt.Vertical, "rows", view)
+            vertical_header = ScenarioAlternativePivotHeaderView(Qt.Orientation.Vertical, "rows", view)
             vertical_header.context_menu_requested.connect(self.show_context_menu)
             super().__init__(view, db_editor, horizontal_header, vertical_header)
             self._header_selection_lists["scenario"] = self._selected_scenario_indexes
@@ -852,6 +823,50 @@ class PivotTableView(CopyPasteTableView):
             checked = len(selected) * [not all(selected)]
             source_model.batch_set_data(self._selected_scenario_alternative_indexes, checked)
 
+    def __init__(self, parent=None):
+        """
+        Args:
+            parent (QWidget, optional): parent widget
+        """
+        super().__init__(parent)
+        self.setHorizontalScrollMode(QTableView.ScrollMode.ScrollPerPixel)
+        self.setVerticalScrollMode(QTableView.ScrollMode.ScrollPerPixel)
+        # NOTE: order of creation of header tables is important for them to stack properly
+        self._left_header_table = CopyPasteTableView(self)
+        self._top_header_table = CopyPasteTableView(self)
+        self._top_left_header_table = CopyPasteTableView(self)
+        self._left_header_table.setObjectName("left")
+        self._top_header_table.setObjectName("top")
+        self._top_left_header_table.setObjectName("top-left")
+        self._spine_db_editor = None
+        self._context = None
+        self._fetch_more_timer = QTimer(self)
+        self._fetch_more_timer.setSingleShot(True)
+        self._fetch_more_timer.setInterval(100)
+        self._fetch_more_timer.timeout.connect(self._fetch_more_visible)
+        self._left_header_table.verticalScrollBar().valueChanged.connect(self.verticalScrollBar().setValue)
+        self.verticalScrollBar().valueChanged.connect(self._left_header_table.verticalScrollBar().setValue)
+        self._top_header_table.horizontalScrollBar().valueChanged.connect(self.horizontalScrollBar().setValue)
+        self.horizontalScrollBar().valueChanged.connect(self._top_header_table.horizontalScrollBar().setValue)
+        # NOTE: order of the iteration below is important for calls to stackUnder
+        for header_table in (self._top_left_header_table, self._top_header_table, self._left_header_table):
+            header_table.setFocusPolicy(Qt.NoFocus)
+            header_table.setStyleSheet(self.styleSheet())
+            header_table.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
+            header_table.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
+            header_table.show()
+            header_table.verticalHeader().hide()
+            header_table.horizontalHeader().hide()
+            header_table.setHorizontalScrollMode(QTableView.ScrollMode.ScrollPerPixel)
+            header_table.setVerticalScrollMode(QTableView.ScrollMode.ScrollPerPixel)
+            header_table.setStyleSheet("QTableView { border: none;}")
+            self.viewport().stackUnder(header_table)
+        for header_table in (self._top_header_table, self._left_header_table):
+            header_table.setAttribute(Qt.WA_TransparentForMouseEvents)
+
+    def _do_resize(self):
+        self.resizeColumnsToContents()
+
     @property
     def source_model(self):
         return self.model().sourceModel()
@@ -894,7 +909,7 @@ class PivotTableView(CopyPasteTableView):
             title = TitleWidgetAction("Pivot", self._spine_db_editor)
             pivot_menu.addAction(title)
             pivot_menu.addActions(self._spine_db_editor.pivot_action_group.actions())
-            pivot_menu.exec_(event.globalPos())
+            pivot_menu.exec(event.globalPos())
             return
         self._context.show_context_menu(event.globalPos())
 
@@ -902,12 +917,22 @@ class PivotTableView(CopyPasteTableView):
         old_model = self.model()
         if old_model:
             old_model.model_data_changed.disconnect(self._fetch_more_timer.start)
+            old_model.model_data_changed.disconnect(self._update_header_tables)
             old_model.modelReset.disconnect(self._update_header_tables)
+            self.selectionModel().selectionChanged.disconnect(self._synch_selection_with_header_tables)
         super().setModel(model)
         for header_table in (self._left_header_table, self._top_header_table, self._top_left_header_table):
             header_table.setModel(model)
         model.model_data_changed.connect(self._fetch_more_timer.start)
+        model.model_data_changed.connect(self._update_header_tables)
         model.modelReset.connect(self._update_header_tables)
+        self.selectionModel().selectionChanged.connect(self._synch_selection_with_header_tables)
+
+    @Slot(QItemSelection, QItemSelection)
+    def _synch_selection_with_header_tables(self, selected, deselected):
+        for header_table in (self._left_header_table, self._top_header_table):
+            header_table.selectionModel().select(selected, QItemSelectionModel.Select)
+            header_table.selectionModel().select(deselected, QItemSelectionModel.Deselect)
 
     def setIndexWidget(self, proxy_index, widget):
         self._top_left_header_table.setIndexWidget(proxy_index, widget)
@@ -935,35 +960,20 @@ class PivotTableView(CopyPasteTableView):
         if scrollbar_at_max and model.canFetchMore(QModelIndex()):
             model.fetchMore(QModelIndex())
 
-    def _init_header_tables(self):
-        # NOTE: order of the iteration below is important for calls to stackUnder
-        for header_table in (self._top_left_header_table, self._top_header_table, self._left_header_table):
-            header_table.setFocusPolicy(Qt.NoFocus)
-            header_table.setStyleSheet(self.styleSheet())
-            header_table.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
-            header_table.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
-            header_table.show()
-            header_table.verticalHeader().hide()
-            header_table.horizontalHeader().hide()
-            header_table.setHorizontalScrollMode(QTableView.ScrollPerPixel)
-            header_table.setVerticalScrollMode(QTableView.ScrollPerPixel)
-            header_table.setStyleSheet("QTableView { border: none;}")
-            self.viewport().stackUnder(header_table)
-
     def _update_header_tables(self):
         # Top
         for header_table in (self._top_header_table, self._top_left_header_table):
             for i in range(0, self.source_model.headerRowCount()):
                 header_table.setRowHeight(i, self.rowHeight(i))
                 header_table.setRowHidden(i, False)
-            for i in range(self.source_model.headerRowCount(), self.source_model.rowCount() - 1):
+            for i in range(self.source_model.headerRowCount(), self.source_model.rowCount()):
                 header_table.setRowHidden(i, True)
         # Left
         for header_table in (self._left_header_table, self._top_left_header_table):
             for j in range(0, self.source_model.headerColumnCount()):
                 header_table.setColumnWidth(j, self.columnWidth(j))
                 header_table.setColumnHidden(j, False)
-            for j in range(self.source_model.headerColumnCount(), self.source_model.columnCount() - 1):
+            for j in range(self.source_model.headerColumnCount(), self.source_model.columnCount()):
                 header_table.setColumnHidden(j, True)
         self._update_header_tables_geometry()
 
@@ -998,7 +1008,6 @@ class PivotTableView(CopyPasteTableView):
 
 
 class FrozenTableView(QTableView):
-
     header_dropped = Signal(object, object)
 
     @property
@@ -1044,7 +1053,7 @@ class MetadataTableViewBase(CopyPasteTableView):
 
     def contextMenuEvent(self, event):
         menu_position = event.globalPos()
-        self._menu.exec_(menu_position)
+        self._menu.exec(menu_position)
 
     def _remove_selected(self):
         """Removes selected rows from view's model."""
@@ -1063,7 +1072,7 @@ class MetadataTableViewBase(CopyPasteTableView):
             db_editor (SpineDBEditor): database editor
         """
 
-    def _populate_context_menu(self,):
+    def _populate_context_menu(self):
         """Fills context menu with actions."""
         self._menu.addAction(self.copy_action)
         self._menu.addAction(self.paste_action)
