@@ -15,24 +15,37 @@ Classes for custom QTreeView.
 :author: M. Marin (KTH)
 :date:   25.4.2018
 """
+import pickle
 
-from PySide2.QtWidgets import QMenu
-from PySide2.QtCore import Signal, Slot, Qt, QEvent, QTimer, QModelIndex, QItemSelection
-from PySide2.QtGui import QMouseEvent, QIcon
-from spinetoolbox.widgets.custom_qtreeview import CopyTreeView
+from PySide6.QtWidgets import QApplication, QMenu, QAbstractItemView
+from PySide6.QtCore import Signal, Slot, Qt, QEvent, QTimer, QModelIndex, QItemSelection
+from PySide6.QtGui import QMouseEvent, QIcon
+
+from spinetoolbox.widgets.custom_qtreeview import CopyPasteTreeView
 from spinetoolbox.helpers import busy_effect, CharIconEngine
-from .custom_delegates import ToolFeatureDelegate, AlternativeScenarioDelegate, ParameterValueListDelegate
+from spinetoolbox.widgets.custom_qwidgets import ResizingViewMixin
+from .custom_delegates import ScenarioDelegate, ToolFeatureDelegate, AlternativeDelegate, ParameterValueListDelegate
 from .scenario_generator import ScenarioGenerator
-from ..mvcmodels.alternative_scenario_item import AlternativeLeafItem
+from ..mvcmodels import mime_types
+from ..mvcmodels.alternative_item import AlternativeItem
+from ..mvcmodels.scenario_item import ScenarioDBItem, ScenarioAlternativeItem, ScenarioItem
 
 
-class EntityTreeView(CopyTreeView):
+class ResizableTreeView(ResizingViewMixin, CopyPasteTreeView):
+    def _do_resize(self):
+        self.resizeColumnToContents(0)
+
+
+class EntityTreeView(ResizableTreeView):
     """Tree view base class for object and relationship tree views."""
 
     tree_selection_changed = Signal(dict)
 
     def __init__(self, parent):
-        """Initialize the view."""
+        """
+        Args:
+            parent (QWidget): parent widget
+        """
         super().__init__(parent=parent)
         self._context_item = None
         self._selected_indexes = {}
@@ -98,7 +111,7 @@ class EntityTreeView(CopyTreeView):
     @Slot(QModelIndex, int, QEvent)
     def edit(self, index, trigger, event):
         """Edit all selected items."""
-        if trigger == self.EditKeyPressed:
+        if trigger == QAbstractItemView.EditTrigger.EditKeyPressed:
             self.edit_selected()
             return True
         return super().edit(index, trigger, event)
@@ -110,7 +123,6 @@ class EntityTreeView(CopyTreeView):
     def rowsInserted(self, parent, start, end):
         super().rowsInserted(parent, start, end)
         self._refresh_selected_indexes()
-        self.resizeColumnToContents(0)
 
     def rowsRemoved(self, parent, start, end):
         super().rowsRemoved(parent, start, end)
@@ -201,7 +213,7 @@ class EntityTreeView(CopyTreeView):
             return
         self._context_item = self.model().item_from_index(index)
         self.update_actions_availability()
-        self._menu.exec_(event.globalPos())
+        self._menu.exec(event.globalPos())
 
     def mousePressEvent(self, event):
         """Overrides selection behaviour if the user has selected sticky selection in Settings.
@@ -264,7 +276,10 @@ class ObjectTreeView(EntityTreeView):
     """Custom QTreeView class for the object tree in SpineDBEditor."""
 
     def __init__(self, parent):
-        """Initialize the view."""
+        """
+        Args:
+            parent (QWidget): parent widget
+        """
         super().__init__(parent=parent)
         self._add_objects_action = None
         self._add_object_classes_action = None
@@ -282,7 +297,7 @@ class ObjectTreeView(EntityTreeView):
         self._add_object_group_action.setEnabled(item.item_type == "object_class")
         self._add_relationship_classes_action.setEnabled(item.item_type in ("root", "object_class"))
         self._manage_members_action.setEnabled(item.item_type == "members")
-        self._duplicate_object_action.setEnabled(item.item_type == "object" and not item.is_group())
+        self._duplicate_object_action.setEnabled(item.item_type == "object" and not item.is_group)
         self._find_next_action.setEnabled(item.item_type == "relationship")
 
     def _add_middle_actions(self):
@@ -373,11 +388,14 @@ class RelationshipTreeView(EntityTreeView):
         self._spine_db_editor.show_add_relationships_form(self._context_item)
 
 
-class ItemTreeView(CopyTreeView):
+class ItemTreeView(ResizableTreeView):
     """Base class for all non-entity tree views."""
 
     def __init__(self, parent):
-        """Initialize the view."""
+        """
+        Args:
+            parent (QWidget): parent widget
+        """
         super().__init__(parent=parent)
         self._spine_db_editor = None
         self._menu = QMenu(self)
@@ -399,6 +417,11 @@ class ItemTreeView(CopyTreeView):
         raise NotImplementedError()
 
     def connect_spine_db_editor(self, spine_db_editor):
+        """Prepares the view to work with the DB editor.
+
+        Args:
+            spine_db_editor (SpineDBEditor): editor instance
+        """
         self._spine_db_editor = spine_db_editor
         self.populate_context_menu()
         self.connect_signals()
@@ -406,6 +429,7 @@ class ItemTreeView(CopyTreeView):
     def populate_context_menu(self):
         """Creates a context menu for this view."""
         self._menu.addAction(self._spine_db_editor.ui.actionCopy)
+        self._menu.addAction(self._spine_db_editor.ui.actionPaste)
         self._menu.addAction("Remove", self.remove_selected)
 
     def contextMenuEvent(self, event):
@@ -419,7 +443,7 @@ class ItemTreeView(CopyTreeView):
             return
         item = index.model().item_from_index(index)
         self.update_actions_availability(item)
-        self._menu.exec_(event.globalPos())
+        self._menu.exec(event.globalPos())
 
     @Slot(QModelIndex, QModelIndex)
     def _refresh_copy_paste_actions(self, _, __):
@@ -476,7 +500,7 @@ class ToolFeatureTreeView(ItemTreeView):
 
     def dragMoveEvent(self, event):
         super().dragMoveEvent(event)
-        index = self.indexAt(event.pos())
+        index = self.indexAt(event.position().toPoint())
         item = self.model().item_from_index(index)
         if item and item.item_type == "tool":
             self.expand(index)
@@ -487,21 +511,27 @@ class ToolFeatureTreeView(ItemTreeView):
             event.accept()
 
 
-class AlternativeScenarioTreeView(ItemTreeView):
-    """Custom QTreeView class for the alternative scenario tree in SpineDBEditor."""
+class AlternativeTreeView(ItemTreeView):
+    """Custom QTreeView for the alternative tree in SpineDBEditor."""
 
     alternative_selection_changed = Signal(dict)
 
     def __init__(self, parent):
-        """Initialize the view."""
+        """
+        Args:
+            parent (QWidget): parent widget
+        """
         super().__init__(parent=parent)
         self._selected_alternative_ids = dict()
         self._generate_scenarios_action = None
-        self.setMouseTracking(True)
+
+    @property
+    def selected_alternative_ids(self):
+        return self._selected_alternative_ids
 
     def reset(self):
         super().reset()
-        self._selected_alternative_ids = dict()
+        self._selected_alternative_ids.clear()
 
     def connect_signals(self):
         """Connects signals."""
@@ -511,7 +541,7 @@ class AlternativeScenarioTreeView(ItemTreeView):
     def connect_spine_db_editor(self, spine_db_editor):
         """see base class"""
         super().connect_spine_db_editor(spine_db_editor)
-        delegate = AlternativeScenarioDelegate(self._spine_db_editor)
+        delegate = AlternativeDelegate(self._spine_db_editor)
         delegate.data_committed.connect(self.model().setData)
         self.setItemDelegateForColumn(0, delegate)
 
@@ -522,25 +552,21 @@ class AlternativeScenarioTreeView(ItemTreeView):
         super().populate_context_menu()
 
     def _db_map_alt_ids_from_selection(self, selection):
-        db_map_ids = {}
-        for index in selection.indexes():
-            if index.column() != 0:
-                continue
-            item = self.model().item_from_index(index)
-            if item.item_type == "alternative" and hasattr(item, "id") and item.id:
-                db_map_ids.setdefault(item.db_map, set()).add(item.id)
-            if item.item_type == "scenario_alternative" and hasattr(item, "alternative_id") and item.alternative_id:
-                db_map_ids.setdefault(item.db_map, set()).add(item.alternative_id)
-        return db_map_ids
+        """Gather alternative ids per database map from selection.
 
-    def _db_map_scen_alt_ids_from_selection(self, selection):
+        Args:
+            selection (QItemSelection): selection
+
+        Returns:
+            dict: mapping from database map to set of alternative ids
+        """
         db_map_ids = {}
         for index in selection.indexes():
             if index.column() != 0:
                 continue
             item = self.model().item_from_index(index)
-            if item.item_type == "scenario_alternative" and hasattr(item, "alternative_id_list"):
-                db_map_ids.setdefault(item.db_map, set()).update(item.alternative_id_list)
+            if isinstance(item, AlternativeItem) and item.id is not None:
+                db_map_ids.setdefault(item.db_map, set()).add(item.id)
         return db_map_ids
 
     @Slot(QItemSelection, QItemSelection)
@@ -548,17 +574,10 @@ class AlternativeScenarioTreeView(ItemTreeView):
         """Emits alternative_selection_changed with the current selection."""
         selected_db_map_alt_ids = self._db_map_alt_ids_from_selection(selected)
         deselected_db_map_alt_ids = self._db_map_alt_ids_from_selection(deselected)
-        selected_db_map_scen_alt_ids = self._db_map_scen_alt_ids_from_selection(selected)
-        deselected_db_map_scen_alt_ids = self._db_map_scen_alt_ids_from_selection(deselected)
-        # NOTE: remove deselected scenario alternatives *before* adding selected alternatives, for obvious reasons
         for db_map, ids in deselected_db_map_alt_ids.items():
             self._selected_alternative_ids[db_map].difference_update(ids)
-        for db_map, ids in deselected_db_map_scen_alt_ids.items():
-            self._selected_alternative_ids[db_map].difference_update(ids)
         for db_map, ids in selected_db_map_alt_ids.items():
             self._selected_alternative_ids.setdefault(db_map, set()).update(ids)
-        for db_map, ids in selected_db_map_scen_alt_ids.items():
-            self._selected_alternative_ids.setdefault(db_map, set()).update(ids)
         self.alternative_selection_changed.emit(self._selected_alternative_ids)
 
     def remove_selected(self):
@@ -566,56 +585,25 @@ class AlternativeScenarioTreeView(ItemTreeView):
         if not self.selectionModel().hasSelection():
             return
         db_map_typed_data_to_rm = {}
-        db_map_scen_alt_data = {}
         items = [self.model().item_from_index(index) for index in self.selectionModel().selectedIndexes()]
         for db_item in self.model()._invisible_root_item.children:
-            db_map_typed_data_to_rm[db_item.db_map] = {"alternative": set(), "scenario": set()}
-            db_map_scen_alt_data[db_item.db_map] = []
-            for alt_item in reversed(db_item.child(0).children[:-1]):
+            db_map_typed_data_to_rm[db_item.db_map] = {"alternative": set()}
+            for alt_item in db_item.children[:-1]:
                 if alt_item in items:
                     db_map_typed_data_to_rm[db_item.db_map]["alternative"].add(alt_item.id)
-            for scen_item in reversed(db_item.child(1).children[:-1]):
-                if scen_item in items:
-                    db_map_typed_data_to_rm[db_item.db_map]["scenario"].add(scen_item.id)
-                    continue
-                scen_alt_root_item = scen_item.scenario_alternative_root_item
-                if not scen_alt_root_item.non_empty_children:
-                    continue
-                curr_alt_id_list = scen_alt_root_item.alternative_id_list
-                new_alt_id_list = [
-                    id_
-                    for alt_item, id_ in zip(scen_alt_root_item.non_empty_children, curr_alt_id_list)
-                    if alt_item not in items
-                ]
-                if new_alt_id_list != curr_alt_id_list:
-                    item = {"id": scen_item.id, "alternative_id_list": ",".join([str(id_) for id_ in new_alt_id_list])}
-                    db_map_scen_alt_data[db_item.db_map].append(item)
-        self.model().db_mngr.set_scenario_alternatives(db_map_scen_alt_data)
         self.model().db_mngr.remove_items(db_map_typed_data_to_rm)
         self.selectionModel().clearSelection()
 
     def update_actions_availability(self, item):
         """See base class."""
         self._generate_scenarios_action.setEnabled(
-            isinstance(item, AlternativeLeafItem) and bool(self._selected_alternative_ids.get(item.db_map))
+            isinstance(item, AlternativeItem) and bool(self._selected_alternative_ids.get(item.db_map))
         )
 
-    def dragMoveEvent(self, event):
-        super().dragMoveEvent(event)
-        index = self.indexAt(event.pos())
-        item = self.model().item_from_index(index)
-        if item and item.item_type == "scenario":
-            self.expand(index)
-
-    def dragEnterEvent(self, event):
-        super().dragEnterEvent(event)
-        if event.source() is self:
-            event.accept()
-
     def _open_scenario_generator(self):
         """Opens the scenario generator dialog."""
         item = self.model().item_from_index(self.currentIndex())
-        if not isinstance(item, AlternativeLeafItem):
+        if not isinstance(item, AlternativeItem):
             return
         included_ids = set()
         alternatives = list()
@@ -627,12 +615,258 @@ class AlternativeScenarioTreeView(ItemTreeView):
         generator = ScenarioGenerator(self, db_map, alternatives, self._spine_db_editor)
         generator.show()
 
+    def can_copy(self):
+        """See base class."""
+        selection = self.selectionModel().selection()
+        if selection.isEmpty():
+            return False
+        model = self.model()
+        for index in selection.indexes():
+            item = model.item_from_index(index)
+            if isinstance(item, AlternativeItem) and item.id is not None:
+                return True
+        return False
+
+    def can_paste(self):
+        """See base class."""
+        clipboard = QApplication.clipboard()
+        mime_data = clipboard.mimeData()
+        if mime_data is None or not mime_data.hasFormat(mime_types.ALTERNATIVE_DATA):
+            return False
+        return True
+
+    def copy(self):
+        """See base class."""
+        selection = self.selectionModel().selection()
+        if not selection:
+            return False
+        model = self.model()
+        indexes = []
+        for index in selection.indexes():
+            item = model.item_from_index(index)
+            if not isinstance(item, AlternativeItem) or item.id is None:
+                continue
+            indexes.append(index)
+        if not indexes:
+            return False
+        mime_data = self.model().mimeData(indexes)
+        clipboard = QApplication.clipboard()
+        clipboard.setMimeData(mime_data)
+        return True
+
+    def paste(self):
+        """Pastes alternatives from clipboard to the tree.
+
+        This makes sense only when pasting alternatives from one database to another.
+        """
+        clipboard = QApplication.clipboard()
+        mime_data = clipboard.mimeData()
+        if mime_data is None or not mime_data.hasFormat(mime_types.ALTERNATIVE_DATA):
+            return
+        index = self.selectionModel().currentIndex()
+        model = self.model()
+        item = model.item_from_index(index)
+        if isinstance(item, AlternativeItem):
+            item = item.parent_item
+        model.paste_alternative_mime_data(mime_data, item)
+
+
+class ScenarioTreeView(ItemTreeView):
+    """Custom QTreeView for the scenario tree in SpineDBEditor."""
+
+    alternative_selection_changed = Signal(dict)
+
+    def __init__(self, parent):
+        """
+        Args:
+            parent (QWidget): parent widget
+        """
+        super().__init__(parent=parent)
+        self._selected_alternative_ids = dict()
+        self._duplicate_scenario_action = None
+
+    @property
+    def selected_alternative_ids(self):
+        return self._selected_alternative_ids
+
+    def reset(self):
+        super().reset()
+        self._selected_alternative_ids.clear()
+
+    def connect_signals(self):
+        """Connects signals."""
+        super().connect_signals()
+        self.selectionModel().selectionChanged.connect(self._handle_selection_changed)
+
+    def connect_spine_db_editor(self, spine_db_editor):
+        """see base class"""
+        super().connect_spine_db_editor(spine_db_editor)
+        delegate = ScenarioDelegate(self._spine_db_editor)
+        delegate.data_committed.connect(self.model().setData)
+        self.setItemDelegateForColumn(0, delegate)
+
+    def populate_context_menu(self):
+        """See base class."""
+        super().populate_context_menu()
+        self._duplicate_scenario_action = self._menu.addAction("Duplicate", self._duplicate_scenario)
+
+    def _db_map_alternative_ids_from_selection(self, selection):
+        """Collects database maps and alternative ids within given selection.
+
+        Args:
+            selection (Sequence of QModelIndex): selection indices
+
+        Returns:
+            dict: mapping from database map to set of alternative ids
+        """
+        db_map_ids = {}
+        for index in selection.indexes():
+            if index.column() != 0:
+                continue
+            item = self.model().item_from_index(index)
+            if isinstance(item, ScenarioItem) and item.id is not None:
+                db_map_ids.setdefault(item.db_map, set()).update(item.alternative_id_list)
+            elif isinstance(item, ScenarioAlternativeItem) and item.alternative_id is not None:
+                db_map_ids.setdefault(item.db_map, set()).add(item.alternative_id)
+        return db_map_ids
+
+    @Slot(QItemSelection, QItemSelection)
+    def _handle_selection_changed(self, selected, deselected):
+        """Emits alternative_selection_changed with the current selection."""
+        self._selected_alternative_ids.clear()
+        for index in self.selectionModel().selectedRows(column=0):
+            item = self.model().item_from_index(index)
+            if isinstance(item, ScenarioItem) and item.id is not None:
+                self._selected_alternative_ids.setdefault(item.db_map, set()).update(item.alternative_id_list)
+            elif isinstance(item, ScenarioAlternativeItem) and item.alternative_id is not None:
+                self._selected_alternative_ids.setdefault(item.db_map, set()).add(item.alternative_id)
+        self.alternative_selection_changed.emit(self._selected_alternative_ids)
+
+    def remove_selected(self):
+        """See base class."""
+        if not self.selectionModel().hasSelection():
+            return
+        db_map_typed_data_to_rm = {}
+        db_map_scen_alt_data = {}
+        items = [self.model().item_from_index(index) for index in self.selectionModel().selectedIndexes()]
+        for db_item in self.model()._invisible_root_item.children:
+            db_map_typed_data_to_rm[db_item.db_map] = {"scenario": set()}
+            db_map_scen_alt_data[db_item.db_map] = []
+            for scen_item in db_item.children[:-1]:
+                if scen_item in items:
+                    db_map_typed_data_to_rm[db_item.db_map]["scenario"].add(scen_item.id)
+                    continue
+                if not scen_item.non_empty_children:
+                    continue
+                curr_alt_id_list = list(scen_item.alternative_id_list)
+                new_alt_id_list = [
+                    id_
+                    for alt_item, id_ in zip(scen_item.non_empty_children, curr_alt_id_list)
+                    if alt_item not in items
+                ]
+                if new_alt_id_list != curr_alt_id_list:
+                    item = {"id": scen_item.id, "alternative_id_list": new_alt_id_list}
+                    db_map_scen_alt_data[db_item.db_map].append(item)
+        self.model().db_mngr.set_scenario_alternatives(db_map_scen_alt_data)
+        self.model().db_mngr.remove_items(db_map_typed_data_to_rm)
+        self.selectionModel().clearSelection()
+
+    def dragMoveEvent(self, event):
+        super().dragMoveEvent(event)
+        index = self.indexAt(event.position().toPoint())
+        item = self.model().item_from_index(index)
+        if item and item.item_type == "scenario":
+            self.expand(index)
+
+    def dragEnterEvent(self, event):
+        super().dragEnterEvent(event)
+        if event.source() is self:
+            event.accept()
+
+    def update_actions_availability(self, item):
+        """See base class"""
+        self._duplicate_scenario_action.setEnabled(isinstance(item, ScenarioItem) and item.id is not None)
+
+    def copy(self):
+        """See base class."""
+        selection = self.selectionModel().selection()
+        if not selection:
+            return False
+        model = self.model()
+        mime_data = model.mimeData(selection.indexes())
+        clipboard = QApplication.clipboard()
+        clipboard.setMimeData(mime_data)
+        return True
+
+    def can_paste(self):
+        """See base class."""
+        index = self.selectionModel().currentIndex()
+        model = self.model()
+        item = model.item_from_index(index)
+        clipboard = QApplication.clipboard()
+        mime_data = clipboard.mimeData()
+        if mime_data is None:
+            return False
+        if mime_data.hasFormat(mime_types.ALTERNATIVE_DATA):
+            if isinstance(item, ScenarioItem):
+                return item.id is not None
+            return isinstance(item, ScenarioAlternativeItem)
+        if mime_data.hasFormat(mime_types.SCENARIO_DATA):
+            return isinstance(item, (ScenarioDBItem, ScenarioItem))
+        return False
+
+    def paste(self):
+        """Pastes alternatives and scenarios from clipboard to the tree."""
+        clipboard = QApplication.clipboard()
+        mime_data = clipboard.mimeData()
+        if mime_data is None:
+            return
+        index = self.selectionModel().currentIndex()
+        model = self.model()
+        item = model.item_from_index(index)
+        if mime_data.hasFormat(mime_types.ALTERNATIVE_DATA):
+            if isinstance(item, ScenarioAlternativeItem):
+                target_row = index.row()
+                scenario_item = item.parent_item
+            elif isinstance(item, ScenarioItem):
+                target_row = -1
+                scenario_item = item
+            else:
+                return
+            if scenario_item.id is None:
+                return
+            model.paste_alternative_mime_data(mime_data, target_row, scenario_item)
+        elif mime_data.hasFormat(mime_types.SCENARIO_DATA):
+            if isinstance(item, ScenarioItem):
+                database_item = item.parent_item
+            elif isinstance(item, ScenarioDBItem):
+                database_item = item
+            else:
+                return
+            model.paste_scenario_mime_data(mime_data, database_item)
+
+    def _duplicate_scenario(self):
+        """Duplicates selected scenarios."""
+        selection = self.selectionModel().selection()
+        if selection.isEmpty():
+            return
+        model = self.model()
+        # Remove duplicates while keeping the order.
+        items = list(dict.fromkeys(model.item_from_index(index) for index in selection.indexes()))
+        for item in items:
+            if not isinstance(item, ScenarioItem) or item.id is None:
+                continue
+            model.duplicate_scenario(item)
+
 
 class ParameterValueListTreeView(ItemTreeView):
     """Custom QTreeView class for parameter_value_list in SpineDBEditor."""
 
     def __init__(self, parent):
-        """Initialize the view."""
+        """
+        Args:
+            parent(QWidget): parent widget
+        """
         super().__init__(parent=parent)
         self._open_in_editor_action = None
 
@@ -672,9 +906,8 @@ class ParameterValueListTreeView(ItemTreeView):
                     continue
                 if list_item in items:
                     db_map_typed_data_to_rm[db_item.db_map]["parameter_value_list"].add(list_item.id)
-                    removed_value_item_ids = {x.id for x in list_item.children[:-1]}
-                else:
-                    removed_value_item_ids = {x.id for x in list_item.children[:-1] if x in items}
+                    continue
+                removed_value_item_ids = {x.id for x in list_item.children[:-1] if x in items}
                 db_map_typed_data_to_rm[db_item.db_map]["list_value"].update(removed_value_item_ids)
         self.model().db_mngr.remove_items(db_map_typed_data_to_rm)
         self.selectionModel().clearSelection()
