@@ -17,8 +17,9 @@ Classes for custom QDialogs to add edit and remove database items.
 """
 
 from functools import reduce
-from PySide2.QtWidgets import QDialog, QDialogButtonBox, QHeaderView, QGridLayout, QAction
-from PySide2.QtCore import Slot, Qt, QModelIndex
+from PySide6.QtWidgets import QDialog, QDialogButtonBox, QHeaderView, QGridLayout
+from PySide6.QtCore import Slot, Qt, QModelIndex
+from PySide6.QtGui import QAction
 from ...widgets.custom_editors import IconColorEditor
 from ...widgets.custom_qtableview import CopyPasteTableView
 from ...helpers import busy_effect, preferred_row_height
@@ -33,10 +34,10 @@ class ManageItemsDialogBase(QDialog):
             db_mngr (SpineDBManager)
         """
         super().__init__(parent)
-        self.setWindowFlag(Qt.WindowContextHelpButtonHint, False)
+        self.setWindowFlag(Qt.WindowType.WindowContextHelpButtonHint, False)
         self.db_mngr = db_mngr
         self.table_view = self.make_table_view()
-        self.table_view.horizontalHeader().setSectionResizeMode(QHeaderView.Interactive)
+        self.table_view.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeMode.Interactive)
         self.table_view.horizontalHeader().setStretchLastSection(True)
         self.table_view.horizontalHeader().setMinimumSectionSize(120)
         self.table_view.verticalHeader().setDefaultSectionSize(preferred_row_height(self))
@@ -44,14 +45,16 @@ class ManageItemsDialogBase(QDialog):
         self._accept_action.setShortcut("Ctrl+Return")
         self.addAction(self._accept_action)
         self.button_box = QDialogButtonBox(self)
-        self.button_box.setStandardButtons(QDialogButtonBox.Cancel | QDialogButtonBox.Ok)
+        self.button_box.setStandardButtons(QDialogButtonBox.StandardButton.Cancel | QDialogButtonBox.StandardButton.Ok)
         layout = QGridLayout(self)
         layout.addWidget(self.table_view)
         layout.addWidget(self.button_box)
         self.setAttribute(Qt.WA_DeleteOnClose)
 
     def make_table_view(self):
-        return CopyPasteTableView(self)
+        table_view = CopyPasteTableView(self)
+        table_view.init_copy_and_paste_actions()
+        return table_view
 
     def connect_signals(self):
         """Connect signals to slots."""
@@ -109,7 +112,7 @@ class ManageItemsDialog(ManageItemsDialogBase):
         """Update model data."""
         if data is None:
             return
-        self.model.setData(index, data, Qt.EditRole)
+        self.model.setData(index, data, Qt.ItemDataRole.EditRole)
 
     @Slot()
     def _handle_model_reset(self):
@@ -171,7 +174,6 @@ class GetObjectsMixin:
                 continue
             rel_cls = relationship_classes[rel_cls_key]
             object_class_id_list = rel_cls["object_class_id_list"]
-            object_class_id_list = [int(x) for x in object_class_id_list.split(",")]
             object_class_id = object_class_id_list[column]
             objects = self.db_map_obj_lookup[db_map]
             object_name_lists.append([name for (class_id, name) in objects if class_id == object_class_id])
@@ -199,6 +201,6 @@ class ShowIconColorEditorMixin:
     @busy_effect
     def show_icon_color_editor(self, index):
         editor = IconColorEditor(self)
-        editor.set_data(index.data(Qt.DisplayRole))
+        editor.set_data(index.data(Qt.ItemDataRole.DisplayRole))
         editor.accepted.connect(lambda index=index, editor=editor: self.set_model_data(index, editor.data()))
         editor.show()
