@@ -15,7 +15,7 @@ Contains machinery to deal with metadata editor.
 :author: A. Soininen (VTT)
 :date:   7.2.2022
 """
-from PySide2.QtCore import QModelIndex, Qt
+from PySide6.QtCore import Qt
 from ..mvcmodels.metadata_table_model import MetadataTableModel
 
 
@@ -31,7 +31,7 @@ class MetadataEditor:
         """
         self._db_editor = db_editor
         self._metadata_table_view = metadata_table_view
-        self._metadata_table_model = MetadataTableModel(db_mngr, db_editor.db_maps, self._metadata_table_view)
+        self._metadata_table_model = MetadataTableModel(db_mngr, db_editor.db_maps, db_editor)
         self._metadata_table_view.sortByColumn(-1, Qt.AscendingOrder)
         self._metadata_table_view.setModel(self._metadata_table_model)
         self._metadata_table_view.connect_spine_db_editor(db_editor)
@@ -51,7 +51,6 @@ class MetadataEditor:
             db_maps (Iterable of DiffDatabaseMapping): database mappings
         """
         self._metadata_table_model.set_db_maps(db_maps)
-        self._metadata_table_model.fetchMore(QModelIndex())
 
     def metadata_model(self):
         """Returns metadata model.
@@ -61,44 +60,10 @@ class MetadataEditor:
         """
         return self._metadata_table_model
 
-    def add_metadata(self, db_map_data):
-        """Adds metadata.
-
-        Args:
-            db_map_data (dict): added metadata records
-        """
-        self._metadata_table_model.add_metadata(db_map_data)
-
-    def update_metadata(self, db_map_data):
-        """Updates metadata.
-
-        Args:
-            db_map_data (dict): updated metadata records
-        """
-        self._metadata_table_model.update_metadata(db_map_data)
-
-    def remove_metadata(self, db_map_data):
-        """Removes entries corresponding to removed metadata from the model.
-
-        Args:
-            db_map_data (dict): removed metadata records
-        """
-        self._metadata_table_model.remove_metadata(db_map_data)
-
-    def add_and_update_metadata(self, db_map_data):
-        """Adds and updates metadata.
-
-        Combined metadata additions and updates may happen when item metadata has been updated.
-
-        Args:
-            db_map_data (dict): removed metadata records
-        """
-        self._metadata_table_model.add_and_update_metadata(db_map_data)
-
-    def roll_back(self, db_maps):
+    def rollback(self, db_maps):
         """Rolls back database changes.
 
         Args:
             db_maps (Iterable of DiffDatabaseMapping): rolled back databases
         """
-        self._metadata_table_model.roll_back(db_maps)
+        self._metadata_table_model.rollback(db_maps)
