@@ -17,9 +17,9 @@ Contains the MultiSpineDBEditor class.
 """
 
 import os
-from PySide2.QtWidgets import QMessageBox, QMenu, QStatusBar, QToolButton
-from PySide2.QtCore import Slot, QPoint
-from PySide2.QtGui import QIcon, QFont
+from PySide6.QtWidgets import QMessageBox, QMenu, QStatusBar, QToolButton
+from PySide6.QtCore import Slot, QPoint
+from PySide6.QtGui import QIcon, QFont
 from .spine_db_editor import SpineDBEditor
 from .custom_qwidgets import ShootingLabel, OpenFileButton, OpenSQLiteFileButton
 from ...widgets.multi_tab_window import MultiTabWindow
@@ -55,19 +55,33 @@ class MultiSpineDBEditor(MultiTabWindow):
     def _make_other(self):
         return MultiSpineDBEditor(self.db_mngr)
 
-    def others(self):
-        return [w for w in self.db_mngr.get_all_multi_spine_db_editors() if w is not self]
-
     def _connect_tab_signals(self, tab):
+        """Connects Spine Db editor window (tab) signals.
+
+        Args:
+            tab (SpineDBEditor): Spine Db editor window
+
+        Returns:
+            bool: True if ok, False otherwise
+        """
         if not super()._connect_tab_signals(tab):
             return False
         tab.file_exported.connect(self.insert_file_open_button)
         tab.sqlite_file_exported.connect(self.insert_sqlite_file_open_button)
         tab.ui.actionUser_guide.triggered.connect(self.show_user_guide)
         tab.ui.actionSettings.triggered.connect(self.settings_form.show)
+        tab.ui.actionClose.triggered.connect(self.handle_close_request_from_tab)
         return True
 
     def _disconnect_tab_signals(self, index):
+        """Disconnects signals of Spine Db editor window (tab) in given index.
+
+        Args:
+            index (int): Tab index
+
+        Returns:
+            bool: True if ok, False otherwise
+        """
         if not super()._disconnect_tab_signals(index):
             return False
         tab = self.tab_widget.widget(index)
@@ -75,6 +89,7 @@ class MultiSpineDBEditor(MultiTabWindow):
         tab.sqlite_file_exported.disconnect(self.insert_sqlite_file_open_button)
         tab.ui.actionUser_guide.triggered.disconnect(self.show_user_guide)
         tab.ui.actionSettings.triggered.disconnect(self.settings_form.show)
+        tab.ui.actionClose.triggered.disconnect(self.handle_close_request_from_tab)
         return True
 
     def _make_new_tab(self, db_url_codenames=None):  # pylint: disable=arguments-differ
@@ -86,13 +101,15 @@ class MultiSpineDBEditor(MultiTabWindow):
         toolbox = self.db_mngr.parent()
         if toolbox is None:
             return
-        ds_urls = {ds.name: ds.project_item.sql_alchemy_url() for ds in toolbox.project_item_model.items("Data Stores")}
+        data_stores = tuple(ds_item.project_item for ds_item in toolbox.project_item_model.items("Data Stores"))
+        ds_urls = {ds.name: ds.sql_alchemy_url() for ds in data_stores}
+        is_url_validated = {ds.name: ds.is_url_validated() for ds in data_stores}
         if not ds_urls:
             return
         menu = QMenu(self)
         for name, url in ds_urls.items():
-            action = menu.addAction(name, lambda name=name, url=url: self.add_new_tab({url: name}))
-            action.setEnabled(bool(url))
+            action = menu.addAction(name, lambda: self.add_new_tab({url: name}))
+            action.setEnabled(url is not None and is_url_validated[name])
         menu.popup(global_pos)
         menu.aboutToHide.connect(menu.deleteLater)
 
@@ -160,9 +177,13 @@ class MultiSpineDBEditor(MultiTabWindow):
         if self._waiting_box is not None:
             return
         self._waiting_box = QMessageBox(
-            QMessageBox.Information, "Closing database", "Waiting for database to close...", QMessageBox.Ok, parent=self
+            QMessageBox.Information,
+            "Closing database",
+            "Waiting for database to close...",
+            QMessageBox.StandardButton.Ok,
+            parent=self,
         )
-        self._waiting_box.button(QMessageBox.Ok).setText("I wait patiently")
+        self._waiting_box.button(QMessageBox.StandardButton.Ok).setText("I wait patiently")
         self._waiting_box.open()
 
     @Slot()
