@@ -19,8 +19,7 @@ Contains the SpineDBEditor class.
 import os
 import json
 from sqlalchemy.engine.url import URL
-from PySide2.QtWidgets import (
-    QAction,
+from PySide6.QtWidgets import (
     QMainWindow,
     QErrorMessage,
     QDockWidget,
@@ -31,15 +30,15 @@ from PySide2.QtWidgets import (
     QCheckBox,
     QDialog,
     QInputDialog,
+    QToolButton,
 )
-from PySide2.QtCore import QModelIndex, Qt, Signal, Slot, QTimer
-from PySide2.QtGui import QGuiApplication, QKeySequence, QIcon
+from PySide6.QtCore import QModelIndex, Qt, Signal, Slot, QTimer
+from PySide6.QtGui import QGuiApplication, QKeySequence, QIcon, QAction
 from spinedb_api import export_data, DatabaseMapping, SpineDBAPIError, SpineDBVersionError, Asterisk
 from spinedb_api.spine_io.importers.excel_reader import get_mapped_data_from_xlsx
 from spinedb_api.helpers import vacuum
 from .custom_menus import MainMenu
 from .commit_viewer import CommitViewer
-from .item_metadata_editor import ItemMetadataEditor
 from .mass_select_items_dialogs import MassRemoveItemsDialog, MassExportItemsDialog
 from .parameter_view_mixin import ParameterViewMixin
 from .tree_view_mixin import TreeViewMixin
@@ -47,6 +46,7 @@ from .graph_view_mixin import GraphViewMixin
 from .tabular_view_mixin import TabularViewMixin
 from .url_toolbar import UrlToolBar
 from .metadata_editor import MetadataEditor
+from .item_metadata_editor import ItemMetadataEditor
 from ...widgets.notification import ChangeNotifier, Notification
 from ...widgets.parameter_value_editor import ParameterValueEditor
 from ...widgets.custom_qwidgets import ToolBarWidgetAction
@@ -101,7 +101,7 @@ class SpineDBEditorBase(QMainWindow):
         self.qsettings = self.db_mngr.qsettings
         self.err_msg = QErrorMessage(self)
         self.err_msg.setWindowTitle("Error")
-        self.err_msg.setWindowFlag(Qt.WindowContextHelpButtonHint, False)
+        self.err_msg.setWindowFlag(Qt.WindowType.WindowContextHelpButtonHint, False)
         self.silenced = False
         max_screen_height = max([s.availableSize().height() for s in QGuiApplication.screens()])
         self.visible_rows = int(max_screen_height / preferred_row_height(self))
@@ -269,7 +269,8 @@ class SpineDBEditorBase(QMainWindow):
         menu.addSeparator()
         menu.addAction(self.ui.dockWidget_tool_feature_tree.toggleViewAction())
         menu.addAction(self.ui.dockWidget_parameter_value_list.toggleViewAction())
-        menu.addAction(self.ui.dockWidget_alternative_scenario_tree.toggleViewAction())
+        menu.addAction(self.ui.alternative_dock_widget.toggleViewAction())
+        menu.addAction(self.ui.scenario_dock_widget.toggleViewAction())
         menu.addAction(self.ui.metadata_dock_widget.toggleViewAction())
         menu.addAction(self.ui.item_metadata_dock_widget.toggleViewAction())
         menu.addSeparator()
@@ -301,7 +302,7 @@ class SpineDBEditorBase(QMainWindow):
         docks_menu = self._make_docks_menu()
         docks_menu_action.setMenu(docks_menu)
         docks_menu_button = view_action.tool_bar.widgetForAction(docks_menu_action)
-        docks_menu_button.setPopupMode(docks_menu_button.InstantPopup)
+        docks_menu_button.setPopupMode(QToolButton.ToolButtonPopupMode.InstantPopup)
         session_action = ToolBarWidgetAction("Session", menu)
         session_action.tool_bar.addActions([self.ui.actionCommit, self.ui.actionRollback])
         session_action.tool_bar.addSeparator()
@@ -316,6 +317,8 @@ class SpineDBEditorBase(QMainWindow):
         menu.addSeparator()
         menu.addAction(self.ui.actionUser_guide)
         menu.addAction(self.ui.actionSettings)
+        self.ui.actionClose.setShortcut(QKeySequence.Close)
+        menu.addAction(self.ui.actionClose)
         menu_action = self.url_toolbar.add_main_menu(menu)
         actions = [
             self.ui.actionNew_db_file,
@@ -352,11 +355,13 @@ class SpineDBEditorBase(QMainWindow):
         # Message signals
         self.msg.connect(self.add_message)
         self.msg_error.connect(self.err_msg.showMessage)
+        self.db_mngr.items_added.connect(self._handle_items_added)
+        self.db_mngr.items_updated.connect(self._handle_items_updated)
+        self.db_mngr.items_removed.connect(self._handle_items_removed)
         # Menu actions
         self.ui.actionCommit.triggered.connect(self.commit_session)
         self.ui.actionRollback.triggered.connect(self.rollback_session)
         self.ui.actionView_history.triggered.connect(self._browse_commits)
-        self.ui.actionClose.triggered.connect(self.close)
         self.ui.actionNew_db_file.triggered.connect(self.create_db_file)
         self.ui.actionOpen_db_file.triggered.connect(self.open_db_file)
         self.ui.actionAdd_db_file.triggered.connect(self.add_db_file)
@@ -377,8 +382,8 @@ class SpineDBEditorBase(QMainWindow):
         msg += "</ul>"
         self.msg.emit(msg)
 
-    @Slot(int)
-    def update_undo_redo_actions(self, index):
+    @Slot(bool)
+    def update_undo_redo_actions(self, _):
         undo_db_map = max(self.db_maps, key=lambda db_map: self.db_mngr.undo_stack[db_map].undo_age)
         redo_db_map = max(self.db_maps, key=lambda db_map: self.db_mngr.undo_stack[db_map].redo_age)
         new_undo_action = self.db_mngr.undo_action[undo_db_map]
@@ -396,7 +401,7 @@ class SpineDBEditorBase(QMainWindow):
                 self.ui.actionRedo.triggered.disconnect(self.redo_action.triggered)
             self.ui.actionRedo.triggered.connect(new_redo_action.triggered)
             self.redo_action = new_redo_action
-        QTimer.singleShot(0, self._refresh_undo_redo_actions)
+        self._refresh_undo_redo_actions()
 
     @Slot()
     def _refresh_undo_redo_actions(self):
@@ -767,171 +772,25 @@ class SpineDBEditorBase(QMainWindow):
         """Update export enabled."""
         # TODO: check if db_mngr has any cache or something like that
 
-    def _receive_items_changed(self, action, item_type, db_map_data):
+    def _log_items_change(self, msg):
         """Enables or disables actions and informs the user about what just happened."""
-        count = sum(len(data) for data in db_map_data.values())
-        msg = f"Successfully {action} {count} {item_type} item(s)"
         self._changelog.append(msg)
         self._update_export_enabled()
 
-    def receive_scenarios_added(self, db_map_data):
-        self._receive_items_changed("added", "scenario", db_map_data)
+    def _handle_items_added(self, item_type, db_map_data):
+        count = sum(len(data) for data in db_map_data.values())
+        msg = f"Successfully added {count} {item_type} item(s)"
+        self._log_items_change(msg)
 
-    def receive_alternatives_added(self, db_map_data):
-        self._receive_items_changed("added", "alternative", db_map_data)
+    def _handle_items_updated(self, item_type, db_map_data):
+        count = sum(len(data) for data in db_map_data.values())
+        msg = f"Successfully updated {count} {item_type} item(s)"
+        self._log_items_change(msg)
 
-    def receive_object_classes_added(self, db_map_data):
-        self._receive_items_changed("added", "object_class", db_map_data)
-
-    def receive_objects_added(self, db_map_data):
-        self._receive_items_changed("added", "object", db_map_data)
-
-    def receive_relationship_classes_added(self, db_map_data):
-        self._receive_items_changed("added", "relationship_class", db_map_data)
-
-    def receive_relationships_added(self, db_map_data):
-        self._receive_items_changed("added", "relationship", db_map_data)
-
-    def receive_entity_groups_added(self, db_map_data):
-        self._receive_items_changed("added", "entity_group", db_map_data)
-
-    def receive_parameter_definitions_added(self, db_map_data):
-        self._receive_items_changed("added", "parameter_definition", db_map_data)
-
-    def receive_parameter_values_added(self, db_map_data):
-        self._receive_items_changed("added", "parameter_value", db_map_data)
-
-    def receive_parameter_value_lists_added(self, db_map_data):
-        self._receive_items_changed("added", "parameter_value_list", db_map_data)
-
-    def receive_list_values_added(self, db_map_data):
-        self._receive_items_changed("added", "list_value", db_map_data)
-
-    def receive_features_added(self, db_map_data):
-        self._receive_items_changed("added", "feature", db_map_data)
-
-    def receive_tools_added(self, db_map_data):
-        self._receive_items_changed("added", "tool", db_map_data)
-
-    def receive_tool_features_added(self, db_map_data):
-        self._receive_items_changed("added", "tool_feature", db_map_data)
-
-    def receive_tool_feature_methods_added(self, db_map_data):
-        self._receive_items_changed("added", "tool_feature_method", db_map_data)
-
-    def receive_metadata_added(self, db_map_data):
-        self._receive_items_changed("added", "metadata", db_map_data)
-
-    def receive_entity_metadata_added(self, db_map_data):
-        self._receive_items_changed("added", "entity_metadata", db_map_data)
-
-    def receive_parameter_value_metadata_added(self, db_map_data):
-        self._receive_items_changed("added", "parameter_value_metadata", db_map_data)
-
-    def receive_scenarios_updated(self, db_map_data):
-        self._receive_items_changed("updated", "scenario", db_map_data)
-
-    def receive_alternatives_updated(self, db_map_data):
-        self._receive_items_changed("updated", "alternative", db_map_data)
-
-    def receive_object_classes_updated(self, db_map_data):
-        self._receive_items_changed("updated", "object_class", db_map_data)
-
-    def receive_objects_updated(self, db_map_data):
-        self._receive_items_changed("updated", "object", db_map_data)
-
-    def receive_relationship_classes_updated(self, db_map_data):
-        self._receive_items_changed("updated", "relationship_class", db_map_data)
-
-    def receive_relationships_updated(self, db_map_data):
-        self._receive_items_changed("updated", "relationship", db_map_data)
-
-    def receive_parameter_definitions_updated(self, db_map_data):
-        self._receive_items_changed("updated", "parameter_definition", db_map_data)
-
-    def receive_parameter_values_updated(self, db_map_data):
-        self._receive_items_changed("updated", "parameter_value", db_map_data)
-
-    def receive_parameter_value_lists_updated(self, db_map_data):
-        self._receive_items_changed("updated", "parameter_value_list", db_map_data)
-
-    def receive_list_values_updated(self, db_map_data):
-        self._receive_items_changed("updated", "list_value", db_map_data)
-
-    def receive_features_updated(self, db_map_data):
-        self._receive_items_changed("updated", "feature", db_map_data)
-
-    def receive_tools_updated(self, db_map_data):
-        self._receive_items_changed("updated", "tool", db_map_data)
-
-    def receive_tool_features_updated(self, db_map_data):
-        self._receive_items_changed("updated", "tool_feature", db_map_data)
-
-    def receive_tool_feature_methods_updated(self, db_map_data):
-        self._receive_items_changed("updated", "tool_feature_method", db_map_data)
-
-    def receive_metadata_updated(self, db_map_data):
-        self._receive_items_changed("updated", "metadata", db_map_data)
-
-    def receive_entity_metadata_updated(self, db_map_data):
-        self._receive_items_changed("updated", "entity_metadata", db_map_data)
-
-    def receive_parameter_value_metadata_updated(self, db_map_data):
-        self._receive_items_changed("updated", "parameter_value_metadata", db_map_data)
-
-    def receive_scenarios_removed(self, db_map_data):
-        self._receive_items_changed("removed", "scenarios", db_map_data)
-
-    def receive_alternatives_removed(self, db_map_data):
-        self._receive_items_changed("removed", "alternatives", db_map_data)
-
-    def receive_object_classes_removed(self, db_map_data):
-        self._receive_items_changed("removed", "object_class", db_map_data)
-
-    def receive_objects_removed(self, db_map_data):
-        self._receive_items_changed("removed", "object", db_map_data)
-
-    def receive_relationship_classes_removed(self, db_map_data):
-        self._receive_items_changed("removed", "relationship_class", db_map_data)
-
-    def receive_relationships_removed(self, db_map_data):
-        self._receive_items_changed("removed", "relationship", db_map_data)
-
-    def receive_entity_groups_removed(self, db_map_data):
-        self._receive_items_changed("removed", "entity_group", db_map_data)
-
-    def receive_parameter_definitions_removed(self, db_map_data):
-        self._receive_items_changed("removed", "parameter_definition", db_map_data)
-
-    def receive_parameter_values_removed(self, db_map_data):
-        self._receive_items_changed("removed", "parameter_value", db_map_data)
-
-    def receive_parameter_value_lists_removed(self, db_map_data):
-        self._receive_items_changed("removed", "parameter_value_list", db_map_data)
-
-    def receive_list_values_removed(self, db_map_data):
-        self._receive_items_changed("removed", "list_value", db_map_data)
-
-    def receive_features_removed(self, db_map_data):
-        self._receive_items_changed("removed", "feature", db_map_data)
-
-    def receive_tools_removed(self, db_map_data):
-        self._receive_items_changed("removed", "tool", db_map_data)
-
-    def receive_tool_features_removed(self, db_map_data):
-        self._receive_items_changed("removed", "tool_feature", db_map_data)
-
-    def receive_tool_feature_methods_removed(self, db_map_data):
-        self._receive_items_changed("removed", "tool_feature_method", db_map_data)
-
-    def receive_metadata_removed(self, db_map_data):
-        self._receive_items_changed("removed", "metadata", db_map_data)
-
-    def receive_entity_metadata_removed(self, db_map_data):
-        self._receive_items_changed("removed", "entity_metadata", db_map_data)
-
-    def receive_parameter_value_metadata_removed(self, db_map_data):
-        self._receive_items_changed("removed", "parameter_value_metadata", db_map_data)
+    def _handle_items_removed(self, item_type, db_map_data):
+        count = sum(len(data) for data in db_map_data.values())
+        msg = f"Successfully removed {count} {item_type} item(s)"
+        self._log_items_change(msg)
 
     def restore_ui(self):
         """Restore UI state from previous session."""
@@ -957,22 +816,24 @@ class SpineDBEditorBase(QMainWindow):
         Returns:
             bool: True if editor is ready to close, False otherwise
         """
-        needs_committing = self.db_mngr.dirty_and_without_editors(self, *self.db_maps)
+        dirty_db_maps = self.db_mngr.dirty_and_without_editors(self, *self.db_maps)
         commit_dirty = False
         commit_msg = ""
-        if needs_committing:
+        if dirty_db_maps:
             answer = self._prompt_to_commit_changes()
-            if answer == QMessageBox.Cancel:
+            if answer == QMessageBox.StandardButton.Cancel:
                 return False
-            db_names = ", ".join([db_map.codename for db_map in needs_committing])
-            if answer == QMessageBox.Save:
+            db_names = ", ".join([db_map.codename for db_map in dirty_db_maps])
+            if answer == QMessageBox.StandardButton.Save:
                 commit_dirty = True
                 commit_msg = self._get_commit_msg(db_names)
                 if not commit_msg:
                     return False
         self._purge_change_notifiers()
         self._torn_down = True
-        self.db_mngr.unregister_listener(self, *self.db_maps, commit_dirty=commit_dirty, commit_msg=commit_msg)
+        self.db_mngr.unregister_listener(
+            self, *self.db_maps, dirty_db_maps=dirty_db_maps, commit_dirty=commit_dirty, commit_msg=commit_msg
+        )
         return True
 
     def _prompt_to_commit_changes(self):
@@ -984,28 +845,30 @@ class SpineDBEditorBase(QMainWindow):
         commit_at_exit = int(self.qsettings.value("appSettings/commitAtExit", defaultValue="1"))
         if commit_at_exit == 0:
             # Don't commit session and don't show message box
-            return QMessageBox.Discard
+            return QMessageBox.StandardButton.Discard
         if commit_at_exit == 1:  # Default
             # Show message box
             msg = QMessageBox(self)
-            msg.setIcon(QMessageBox.Question)
+            msg.setIcon(QMessageBox.Icon.Question)
             msg.setWindowTitle(self.windowTitle())
             msg.setText("The current session has uncommitted changes. Do you want to commit them now?")
-            msg.setStandardButtons(QMessageBox.Save | QMessageBox.Discard | QMessageBox.Cancel)
-            msg.button(QMessageBox.Save).setText("Commit and close ")
-            msg.button(QMessageBox.Discard).setText("Discard changes and close")
+            msg.setStandardButtons(
+                QMessageBox.StandardButton.Save | QMessageBox.StandardButton.Discard | QMessageBox.StandardButton.Cancel
+            )
+            msg.button(QMessageBox.StandardButton.Save).setText("Commit and close ")
+            msg.button(QMessageBox.StandardButton.Discard).setText("Discard changes and close")
             chkbox = QCheckBox()
             chkbox.setText("Do not ask me again")
             msg.setCheckBox(chkbox)
-            answer = msg.exec_()
-            if answer != QMessageBox.Cancel and chkbox.checkState() == 2:
+            answer = msg.exec()
+            if answer != QMessageBox.StandardButton.Cancel and chkbox.checkState() == 2:
                 # Save preference
-                preference = "2" if answer == QMessageBox.Save else "0"
+                preference = "2" if answer == QMessageBox.StandardButton.Save else "0"
                 self.qsettings.setValue("appSettings/commitAtExit", preference)
             return answer
         if commit_at_exit == 2:
             # Commit session and don't show message box
-            return QMessageBox.Save
+            return QMessageBox.StandardButton.Save
 
     def _get_commit_msg(self, db_names):
         """Prompts user for commit message.
@@ -1017,8 +880,8 @@ class SpineDBEditorBase(QMainWindow):
             str: commit message
         """
         dialog = CommitDialog(self, db_names)
-        answer = dialog.exec_()
-        if answer == QDialog.Accepted:
+        answer = dialog.exec()
+        if answer == QDialog.DialogCode.Accepted:
             return dialog.commit_msg
 
     def _get_rollback_confirmation(self, db_names):
@@ -1031,16 +894,16 @@ class SpineDBEditorBase(QMainWindow):
             bool: True if user confirmed, False otherwise
         """
         message_box = QMessageBox(
-            QMessageBox.Question,
+            QMessageBox.Icon.Question,
             f"Rollback changes in {db_names}",
             "Are you sure? "
             "All your changes since the last commit will be reverted and removed from the undo/redo stack.",
-            QMessageBox.Ok | QMessageBox.Cancel,
+            QMessageBox.StandardButton.Ok | QMessageBox.StandardButton.Cancel,
             parent=self,
         )
-        message_box.button(QMessageBox.Ok).setText("Rollback")
-        answer = message_box.exec_()
-        return answer == QMessageBox.Ok
+        message_box.button(QMessageBox.StandardButton.Ok).setText("Rollback")
+        answer = message_box.exec()
+        return answer == QMessageBox.StandardButton.Ok
 
     def _purge_change_notifiers(self):
         """Tears down change notifiers."""
@@ -1061,30 +924,6 @@ class SpineDBEditorBase(QMainWindow):
         self.save_window_state()
         super().closeEvent(event)
 
-    def scenario_items(self, db_map):
-        """Gathers scenario items from alternative scenario tree for given database.
-
-        Args:
-            db_map (DiffDatabaseMapping): database map
-
-        Returns:
-            list of CachedItem: scenario items
-        """
-        model = self.ui.treeView_alternative_scenario.model()
-        db_index = None
-        for row in range(model.rowCount()):
-            db_index = model.index(row, 0)
-            item = model.item_from_index(db_index)
-            if item.db_map is db_map:
-                break
-        if db_index is None:
-            raise RuntimeError("Database item not found in alternative/scenario model.")
-        scenario_root_index = model.index(1, 0, db_index)
-        return [
-            model.item_from_index(model.index(row, 0, scenario_root_index))
-            for row in range(model.rowCount(scenario_root_index))
-        ]
-
 
 class SpineDBEditor(TabularViewMixin, GraphViewMixin, ParameterViewMixin, TreeViewMixin, SpineDBEditorBase):
     """A widget to visualize Spine dbs."""
@@ -1207,24 +1046,30 @@ class SpineDBEditor(TabularViewMixin, GraphViewMixin, ParameterViewMixin, TreeVi
     def apply_stacked_style(self, checked=False):
         """Applies the stacked style, inspired in the former tree view."""
         self.begin_style_change()
-        self.splitDockWidget(self.ui.dockWidget_object_tree, self.ui.dockWidget_object_parameter_value, Qt.Horizontal)
         self.splitDockWidget(
-            self.ui.dockWidget_object_parameter_value, self.ui.dockWidget_alternative_scenario_tree, Qt.Horizontal
+            self.ui.dockWidget_object_tree, self.ui.dockWidget_object_parameter_value, Qt.Orientation.Horizontal
+        )
+        self.splitDockWidget(
+            self.ui.dockWidget_object_parameter_value, self.ui.alternative_dock_widget, Qt.Orientation.Horizontal
+        )
+        self.splitDockWidget(
+            self.ui.alternative_dock_widget, self.ui.dockWidget_tool_feature_tree, Qt.Orientation.Horizontal
+        )
+        self.splitDockWidget(
+            self.ui.dockWidget_object_tree, self.ui.dockWidget_relationship_tree, Qt.Orientation.Vertical
         )
-        self.splitDockWidget(self.ui.dockWidget_object_tree, self.ui.dockWidget_relationship_tree, Qt.Vertical)
         # right-side
-        self.splitDockWidget(self.ui.dockWidget_alternative_scenario_tree, self.ui.metadata_dock_widget, Qt.Vertical)
-        self.tabify_and_raise(
-            [
-                self.ui.dockWidget_alternative_scenario_tree,
-                self.ui.dockWidget_tool_feature_tree,
-                self.ui.dockWidget_parameter_value_list,
-            ]
+        self.splitDockWidget(self.ui.alternative_dock_widget, self.ui.scenario_dock_widget, Qt.Orientation.Vertical)
+        self.splitDockWidget(
+            self.ui.dockWidget_tool_feature_tree, self.ui.metadata_dock_widget, Qt.Orientation.Vertical
         )
+        self.tabify_and_raise([self.ui.dockWidget_tool_feature_tree, self.ui.dockWidget_parameter_value_list])
         self.tabify_and_raise([self.ui.metadata_dock_widget, self.ui.item_metadata_dock_widget])
         # center
         self.splitDockWidget(
-            self.ui.dockWidget_object_parameter_value, self.ui.dockWidget_relationship_parameter_value, Qt.Vertical
+            self.ui.dockWidget_object_parameter_value,
+            self.ui.dockWidget_relationship_parameter_value,
+            Qt.Orientation.Vertical,
         )
         self.tabify_and_raise(
             [self.ui.dockWidget_object_parameter_value, self.ui.dockWidget_object_parameter_definition]
@@ -1238,22 +1083,26 @@ class SpineDBEditor(TabularViewMixin, GraphViewMixin, ParameterViewMixin, TreeVi
         docks = [
             self.ui.dockWidget_object_tree,
             self.ui.dockWidget_object_parameter_value,
-            self.ui.dockWidget_alternative_scenario_tree,
+            self.ui.alternative_dock_widget,
+            self.ui.dockWidget_tool_feature_tree,
         ]
         width = sum(d.size().width() for d in docks)
-        self.resizeDocks(docks, [0.2 * width, 0.6 * width, 0.2 * width], Qt.Horizontal)
+        self.resizeDocks(docks, [0.2 * width, 0.5 * width, 0.15 * width, 0.15 * width], Qt.Orientation.Horizontal)
         self.end_style_change()
 
     @Slot(QAction)
     def apply_pivot_style(self, _action):
         """Applies the pivot style, inspired in the former tabular view."""
         self.begin_style_change()
-        self.splitDockWidget(self.ui.dockWidget_object_tree, self.ui.dockWidget_pivot_table, Qt.Horizontal)
-        self.splitDockWidget(self.ui.dockWidget_pivot_table, self.ui.dockWidget_frozen_table, Qt.Horizontal)
-        self.splitDockWidget(self.ui.dockWidget_object_tree, self.ui.dockWidget_relationship_tree, Qt.Vertical)
-        self.splitDockWidget(self.ui.dockWidget_frozen_table, self.ui.dockWidget_tool_feature_tree, Qt.Vertical)
+        self.splitDockWidget(self.ui.dockWidget_object_tree, self.ui.dockWidget_pivot_table, Qt.Orientation.Horizontal)
+        self.splitDockWidget(self.ui.dockWidget_pivot_table, self.ui.dockWidget_frozen_table, Qt.Orientation.Horizontal)
         self.splitDockWidget(
-            self.ui.dockWidget_tool_feature_tree, self.ui.dockWidget_alternative_scenario_tree, Qt.Vertical
+            self.ui.dockWidget_object_tree, self.ui.dockWidget_relationship_tree, Qt.Orientation.Vertical
+        )
+        self.splitDockWidget(self.ui.dockWidget_frozen_table, self.ui.alternative_dock_widget, Qt.Orientation.Vertical)
+        self.splitDockWidget(self.ui.alternative_dock_widget, self.ui.scenario_dock_widget, Qt.Orientation.Vertical)
+        self.splitDockWidget(
+            self.ui.scenario_dock_widget, self.ui.dockWidget_tool_feature_tree, Qt.Orientation.Vertical
         )
         self.ui.dockWidget_entity_graph.hide()
         self.ui.dockWidget_object_parameter_value.hide()
@@ -1265,7 +1114,7 @@ class SpineDBEditor(TabularViewMixin, GraphViewMixin, ParameterViewMixin, TreeVi
         self.ui.item_metadata_dock_widget.hide()
         docks = [self.ui.dockWidget_object_tree, self.ui.dockWidget_pivot_table, self.ui.dockWidget_frozen_table]
         width = sum(d.size().width() for d in docks)
-        self.resizeDocks(docks, [0.2 * width, 0.6 * width, 0.2 * width], Qt.Horizontal)
+        self.resizeDocks(docks, [0.2 * width, 0.65 * width, 0.15 * width], Qt.Orientation.Horizontal)
         self.end_style_change()
 
     @Slot(bool)
@@ -1274,30 +1123,35 @@ class SpineDBEditor(TabularViewMixin, GraphViewMixin, ParameterViewMixin, TreeVi
         self.begin_style_change()
         self.ui.dockWidget_pivot_table.hide()
         self.ui.dockWidget_frozen_table.hide()
-        self.splitDockWidget(self.ui.dockWidget_object_tree, self.ui.dockWidget_entity_graph, Qt.Horizontal)
+        self.splitDockWidget(self.ui.dockWidget_object_tree, self.ui.dockWidget_entity_graph, Qt.Orientation.Horizontal)
         self.splitDockWidget(
-            self.ui.dockWidget_entity_graph, self.ui.dockWidget_alternative_scenario_tree, Qt.Horizontal
+            self.ui.dockWidget_entity_graph, self.ui.alternative_dock_widget, Qt.Orientation.Horizontal
         )
         # right-side
-        self.splitDockWidget(self.ui.dockWidget_alternative_scenario_tree, self.ui.metadata_dock_widget, Qt.Vertical)
+        self.splitDockWidget(self.ui.alternative_dock_widget, self.ui.scenario_dock_widget, Qt.Orientation.Vertical)
+        self.splitDockWidget(
+            self.ui.scenario_dock_widget, self.ui.dockWidget_tool_feature_tree, Qt.Orientation.Vertical
+        )
         self.tabify_and_raise(
             [
-                self.ui.dockWidget_alternative_scenario_tree,
                 self.ui.dockWidget_tool_feature_tree,
                 self.ui.dockWidget_parameter_value_list,
+                self.ui.item_metadata_dock_widget,
+                self.ui.metadata_dock_widget,
             ]
         )
-        self.tabify_and_raise([self.ui.metadata_dock_widget, self.ui.item_metadata_dock_widget])
         # left
-        self.splitDockWidget(self.ui.dockWidget_object_tree, self.ui.dockWidget_relationship_tree, Qt.Vertical)
-        self.splitDockWidget(self.ui.dockWidget_entity_graph, self.ui.dockWidget_object_parameter_value, Qt.Vertical)
         self.splitDockWidget(
-            self.ui.dockWidget_object_parameter_value, self.ui.dockWidget_relationship_parameter_value, Qt.Vertical
+            self.ui.dockWidget_object_tree, self.ui.dockWidget_relationship_tree, Qt.Orientation.Vertical
         )
         self.splitDockWidget(
-            self.ui.dockWidget_alternative_scenario_tree, self.ui.dockWidget_tool_feature_tree, Qt.Vertical
+            self.ui.dockWidget_entity_graph, self.ui.dockWidget_object_parameter_value, Qt.Orientation.Vertical
+        )
+        self.splitDockWidget(
+            self.ui.dockWidget_object_parameter_value,
+            self.ui.dockWidget_relationship_parameter_value,
+            Qt.Orientation.Vertical,
         )
-        self.splitDockWidget(self.ui.dockWidget_tool_feature_tree, self.ui.dockWidget_parameter_value_list, Qt.Vertical)
         self.tabify_and_raise(
             [self.ui.dockWidget_object_parameter_value, self.ui.dockWidget_object_parameter_definition]
         )
@@ -1310,59 +1164,17 @@ class SpineDBEditor(TabularViewMixin, GraphViewMixin, ParameterViewMixin, TreeVi
             self.ui.dockWidget_relationship_parameter_value,
         ]
         height = sum(d.size().height() for d in docks)
-        self.resizeDocks(docks, [0.6 * height, 0.2 * height, 0.2 * height], Qt.Vertical)
-        docks = [
-            self.ui.dockWidget_object_tree,
-            self.ui.dockWidget_entity_graph,
-            self.ui.dockWidget_alternative_scenario_tree,
-        ]
+        self.resizeDocks(docks, [0.6 * height, 0.2 * height, 0.2 * height], Qt.Orientation.Vertical)
+        docks = [self.ui.dockWidget_object_tree, self.ui.dockWidget_entity_graph, self.ui.alternative_dock_widget]
         width = sum(d.size().width() for d in docks)
-        self.resizeDocks(docks, [0.2 * width, 0.6 * width, 0.2 * width], Qt.Horizontal)
+        self.resizeDocks(docks, [0.2 * width, 0.65 * width, 0.15 * width], Qt.Orientation.Horizontal)
         self.end_style_change()
         self.ui.graphicsView.reset_zoom()
 
-    def receive_metadata_added(self, db_map_data):
-        super().receive_metadata_added(db_map_data)
-        self._metadata_editor.add_metadata(db_map_data)
-
-    def receive_entity_metadata_added(self, db_map_data):
-        super().receive_entity_metadata_added(db_map_data)
-        self._item_metadata_editor.add_item_metadata(db_map_data)
-
-    def receive_parameter_value_metadata_added(self, db_map_data):
-        super().receive_parameter_value_metadata_added(db_map_data)
-        self._item_metadata_editor.add_item_metadata(db_map_data)
-
-    def receive_metadata_updated(self, db_map_data):
-        super().receive_metadata_updated(db_map_data)
-        self._metadata_editor.update_metadata(db_map_data)
-        self._item_metadata_editor.update_metadata(db_map_data)
-
-    def receive_entity_metadata_updated(self, db_map_data):
-        super().receive_entity_metadata_updated(db_map_data)
-        self._item_metadata_editor.update_item_metadata(db_map_data)
-
-    def receive_parameter_value_metadata_updated(self, db_map_data):
-        super().receive_parameter_value_metadata_updated(db_map_data)
-        self._item_metadata_editor.update_item_metadata(db_map_data)
-
-    def receive_metadata_removed(self, db_map_data):
-        super().receive_metadata_removed(db_map_data)
-        self._metadata_editor.remove_metadata(db_map_data)
-        self._item_metadata_editor.remove_metadata(db_map_data)
-
-    def receive_entity_metadata_removed(self, db_map_data):
-        super().receive_entity_metadata_removed(db_map_data)
-        self._item_metadata_editor.remove_item_metadata(db_map_data)
-
-    def receive_parameter_value_metadata_removed(self, db_map_data):
-        super().receive_parameter_value_metadata_removed(db_map_data)
-        self._item_metadata_editor.remove_item_metadata(db_map_data)
-
     def receive_session_rolled_back(self, db_maps):
         super().receive_session_rolled_back(db_maps)
-        self._metadata_editor.roll_back(db_maps)
-        self._item_metadata_editor.roll_back(db_maps)
+        self._metadata_editor.rollback(db_maps)
+        self._item_metadata_editor.rollback(db_maps)
 
     def tear_down(self):
         if not super().tear_down():
