@@ -18,9 +18,8 @@ Contains TabularViewMixin class.
 
 from itertools import product
 from collections import namedtuple
-from PySide2.QtCore import Qt, Slot, QTimer
-from PySide2.QtWidgets import QActionGroup
-from PySide2.QtGui import QIcon
+from PySide6.QtCore import Qt, Slot, QTimer
+from PySide6.QtGui import QIcon, QActionGroup
 from spinedb_api.helpers import fix_name_ambiguity
 from .custom_menus import TabularViewFilterMenu
 from .tabular_view_header_widget import TabularViewHeaderWidget
@@ -51,6 +50,13 @@ class TabularViewMixin:
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
+        self._pivot_table_models = {
+            self._PARAMETER_VALUE: ParameterValuePivotTableModel(self),
+            self._RELATIONSHIP: RelationshipPivotTableModel(self),
+            self._INDEX_EXPANSION: IndexExpansionPivotTableModel(self),
+            self._SCENARIO_ALTERNATIVE: ScenarioAlternativePivotTableModel(self),
+        }
+        self._pending_reload = False
         self.current_class_type = None
         self.current_class_id = {}  # Mapping from db_map to class_id
         self.current_class_name = None
@@ -90,7 +96,67 @@ class TabularViewMixin:
         self.ui.frozen_table.selectionModel().currentChanged.connect(self.change_frozen_value)
         self.pivot_action_group.triggered.connect(self._handle_pivot_action_triggered)
         self.ui.dockWidget_pivot_table.visibilityChanged.connect(self._handle_pivot_table_visibility_changed)
-        self.ui.dockWidget_frozen_table.visibilityChanged.connect(self._handle_frozen_table_visibility_changed)
+        self.db_mngr.items_updated.connect(self._reload_pivot_table_if_needed)
+
+    def refresh_views(self):
+        for table_view in (self.ui.pivot_table, self.ui.frozen_table):
+            top_left = table_view.indexAt(table_view.rect().topLeft())
+            bottom_right = table_view.indexAt(table_view.rect().bottomRight())
+            if not bottom_right.isValid():
+                model = table_view.model()
+                bottom_right = table_view.model().index(model.rowCount() - 1, model.columnCount() - 1)
+            table_view.model().dataChanged.emit(top_left, bottom_right)
+
+    # FIXME MM: this should be called after modifications
+    @Slot(str)
+    def update_filter_menus(self, action):
+        for identifier, menu in self.filter_menus.items():
+            index_values = dict.fromkeys(self.pivot_table_model.model.index_values.get(identifier, []))
+            index_values.pop(None, None)
+            if action == "add":
+                menu.add_items_to_filter_list(list(index_values.keys()))
+            elif action == "remove":
+                previous = menu._filter._filter_model._data_set
+                menu.remove_items_from_filter_list(list(previous - index_values.keys()))
+        self.reload_frozen_table()
+
+    def _needs_to_update_headers(self, item_type, db_map_data):
+        for db_map in self.db_maps:
+            items = db_map_data.get(db_map)
+            if not items:
+                continue
+            if self.current_input_type in (self._PARAMETER_VALUE, self._INDEX_EXPANSION):
+                if item_type in ("object_class", "relationship_class"):
+                    for item in items:
+                        if item["id"] == self.current_class_id.get(db_map):
+                            return True
+                    if item_type == "object_class":
+                        object_class_id_list = {
+                            db_map_class_id.get(db_map) for db_map_class_id in self.current_object_class_id_list
+                        }
+                        for item in items:
+                            if item["id"] in object_class_id_list:
+                                return True
+            elif self.current_input_type == self._RELATIONSHIP:
+                if item_type == "relationship_class":
+                    for item in items:
+                        if item["id"] == self.current_class_id.get(db_map):
+                            return True
+                elif item_type == "object_class":
+                    object_class_id_list = {
+                        db_map_class_id.get(db_map) for db_map_class_id in self.current_object_class_id_list
+                    }
+                    for item in items:
+                        if item["id"] in object_class_id_list:
+                            return True
+        return False
+
+    @Slot(str, dict)
+    def _reload_pivot_table_if_needed(self, item_type, db_map_data):
+        if not self.pivot_table_model:
+            return
+        if self._needs_to_update_headers(item_type, db_map_data):
+            self.do_reload_pivot_table()
 
     @Slot()
     def _connect_pivot_table_header_signals(self):
@@ -114,8 +180,8 @@ class TabularViewMixin:
         current_object_class_id_list = [{} for _ in self.current_object_class_name_list]
         for db_map, class_id in self.current_class_id.items():
             relationship_class = self.db_mngr.get_item(db_map, "relationship_class", class_id)
-            for k, id_ in enumerate(relationship_class["object_class_id_list"].split(",")):
-                current_object_class_id_list[k][db_map] = int(id_)
+            for k, id_ in enumerate(relationship_class["object_class_id_list"]):
+                current_object_class_id_list[k][db_map] = id_
         return current_object_class_id_list
 
     @property
@@ -124,7 +190,7 @@ class TabularViewMixin:
         if self.current_class_type == "object_class":
             return [self.db_mngr.get_item(db_map, "object_class", class_id)["name"]]
         relationship_class = self.db_mngr.get_item(db_map, "relationship_class", class_id)
-        return fix_name_ambiguity(relationship_class["object_class_name_list"].split(","))
+        return fix_name_ambiguity(relationship_class["object_class_name_list"])
 
     @property
     def current_object_class_ids(self):
@@ -146,6 +212,7 @@ class TabularViewMixin:
         self.current_input_type = action.text()
         # NOTE: Changing the action also triggers a call to `_handle_pivot_table_visibility_changed` with `visible = True`
         # See `SpineDBEditor` class.
+        self.do_reload_pivot_table()
 
     @Slot(bool)
     def _handle_pivot_table_visibility_changed(self, visible):
@@ -153,14 +220,8 @@ class TabularViewMixin:
             for action in self.pivot_actions.values():
                 action.setChecked(False)
             return
-        self.pivot_actions[self.current_input_type].setChecked(True)
-        self.ui.dockWidget_frozen_table.setVisible(True)
-        self.do_reload_pivot_table()
-
-    @Slot(bool)
-    def _handle_frozen_table_visibility_changed(self, visible):
-        if visible:
-            self.ui.dockWidget_pivot_table.show()
+        if self._pending_reload:
+            self.do_reload_pivot_table()
 
     @Slot(dict)
     def _handle_object_tree_selection_changed(self, selected_indexes):
@@ -176,7 +237,7 @@ class TabularViewMixin:
 
     def _handle_entity_tree_current_changed(self, current_index):
         self._update_class_attributes(current_index)
-        if self.current_input_type != self._SCENARIO_ALTERNATIVE and self.ui.dockWidget_pivot_table.isVisible():
+        if self.current_input_type != self._SCENARIO_ALTERNATIVE:
             self.do_reload_pivot_table()
 
     def _update_class_attributes(self, current_index):
@@ -190,7 +251,6 @@ class TabularViewMixin:
         self.current_class_id = class_id
         self.current_class_type = current_class_item.item_type
         self.current_class_name = current_class_item.display_data
-        self.clear_pivot_table()
 
     @staticmethod
     def _get_current_class_item(current_index):
@@ -270,7 +330,7 @@ class TabularViewMixin:
             db_map_relationships = self._get_db_map_entities()
         get_id = self._make_get_id(action)
         return {
-            tuple((db_map, int(id_)) for id_ in rel["object_id_list"].split(',')) + (db_map,): get_id(db_map, rel)
+            tuple((db_map, id_) for id_ in rel["object_id_list"]) + (db_map,): get_id(db_map, rel)
             for db_map, relationships in db_map_relationships.items()
             for rel in relationships
         }
@@ -372,7 +432,7 @@ class TabularViewMixin:
             }
         if self.current_class_type == "relationship_class":
             db_map_entity_ids = {
-                db_map: [tuple((db_map, int(id_)) for id_ in e["object_id_list"].split(',')) for e in entities]
+                db_map: [tuple((db_map, id_) for id_ in e["object_id_list"]) for e in entities]
                 for db_map, entities in db_map_entities.items()
             }
         else:
@@ -415,7 +475,7 @@ class TabularViewMixin:
                 for x in items
             }
         return {
-            tuple((db_map, int(id_)) for id_ in x["object_id_list"].split(','))
+            tuple((db_map, id_) for id_ in x["object_id_list"])
             + ((db_map, x["parameter_id"]), (db_map, x["alternative_id"]), db_map): get_id(db_map, x)
             for db_map, items in db_map_parameter_values.items()
             for x in items
@@ -504,21 +564,22 @@ class TabularViewMixin:
         """Reloads pivot table."""
         if not self._can_build_pivot_table():
             return
-        self.pivot_table_model = {
-            self._PARAMETER_VALUE: ParameterValuePivotTableModel,
-            self._RELATIONSHIP: RelationshipPivotTableModel,
-            self._INDEX_EXPANSION: IndexExpansionPivotTableModel,
-            self._SCENARIO_ALTERNATIVE: ScenarioAlternativePivotTableModel,
-        }[self.current_input_type](self)
+        if not self.ui.dockWidget_pivot_table.isVisible():
+            self._pending_reload = True
+            return
+        self.pivot_actions[self.current_input_type].setChecked(True)
+        self.ui.dockWidget_frozen_table.setVisible(True)
+        self._pending_reload = False
+        self.pivot_table_model = self._pivot_table_models[self.current_input_type]
         self.pivot_table_proxy.setSourceModel(self.pivot_table_model)
+        self.pivot_table_model.modelReset.connect(self.make_pivot_headers)
+        self.pivot_table_model.modelReset.connect(self.reload_frozen_table)
         delegate = self.pivot_table_model.make_delegate(self)
         self.ui.pivot_table.setItemDelegate(delegate)
-        self.pivot_table_model.modelReset.connect(self.make_pivot_headers)
         pivot = self.get_pivot_preferences()
         self.wipe_out_filter_menus()
         self.pivot_table_model.call_reset_model(pivot)
         self.pivot_table_proxy.clear_filter()
-        self.reload_frozen_table()
 
     def _can_build_pivot_table(self):
         if self.current_input_type != self._SCENARIO_ALTERNATIVE and not self.current_class_id:
@@ -532,9 +593,10 @@ class TabularViewMixin:
         if self.pivot_table_model:
             self.pivot_table_model.clear_model()
             self.pivot_table_proxy.clear_filter()
+            self.pivot_table_model.modelReset.disconnect(self.make_pivot_headers)
+            self.pivot_table_model.modelReset.disconnect(self.reload_frozen_table)
             self.pivot_table_model = None
-        if self.frozen_table_model:
-            self.frozen_table_model.clear_model()
+        self.frozen_table_model.clear_model()
 
     def wipe_out_filter_menus(self):
         while self.filter_menus:
@@ -549,11 +611,11 @@ class TabularViewMixin:
         top_indexes, left_indexes = self.pivot_table_model.top_left_indexes()
         for index in left_indexes:
             proxy_index = self.pivot_table_proxy.mapFromSource(index)
-            widget = self.create_header_widget(proxy_index.data(Qt.DisplayRole), "columns")
+            widget = self.create_header_widget(proxy_index.data(Qt.ItemDataRole.DisplayRole), "columns")
             self.ui.pivot_table.setIndexWidget(proxy_index, widget)
         for index in top_indexes:
             proxy_index = self.pivot_table_proxy.mapFromSource(index)
-            widget = self.create_header_widget(proxy_index.data(Qt.DisplayRole), "rows")
+            widget = self.create_header_widget(proxy_index.data(Qt.ItemDataRole.DisplayRole), "rows")
             self.ui.pivot_table.setIndexWidget(proxy_index, widget)
         QTimer.singleShot(0, self._resize_pivot_header_columns)
 
@@ -571,7 +633,7 @@ class TabularViewMixin:
         """
         for column in range(self.frozen_table_model.columnCount()):
             index = self.frozen_table_model.index(0, column)
-            widget = self.create_header_widget(index.data(Qt.DisplayRole), "frozen", with_menu=False)
+            widget = self.create_header_widget(index.data(Qt.ItemDataRole.DisplayRole), "frozen", with_menu=False)
             self.ui.frozen_table.setIndexWidget(index, widget)
             column_width = self.ui.frozen_table.horizontalHeader().sectionSize(column)
             header_width = widget.size().width()
@@ -588,14 +650,26 @@ class TabularViewMixin:
             TabularViewFilterMenu
         """
         if identifier not in self.filter_menus:
-            pivot_top_left_header = self.pivot_table_model.top_left_headers[identifier]
-            data_to_value = pivot_top_left_header.header_data
+            header = self.pivot_table_model.top_left_headers[identifier]
+            if header.header_type == "parameter":
+                item_type = "parameter_definition"
+            elif header.header_type == "index":
+                item_type = "parameter_value"
+            else:
+                item_type = header.header_type
+            if header.header_type == "object":
+                accepts_item = (
+                    self.accepts_entity_item
+                    if self.current_class_type == "object_class"
+                    else lambda item, db_map: self.accepts_ith_member_object_item(header.rank, item, db_map)
+                )
+            elif header.header_type == "parameter":
+                accepts_item = self.accepts_parameter_item
+            else:
+                accepts_item = None
             self.filter_menus[identifier] = menu = TabularViewFilterMenu(
-                self, identifier, data_to_value, show_empty=False
+                self, self.db_mngr, self.db_maps, item_type, accepts_item, identifier, show_empty=False
             )
-            index_values = dict.fromkeys(self.pivot_table_model.model.index_values.get(identifier, []))
-            index_values.pop(None, None)
-            menu.set_filter_list(index_values.keys())
             menu.filterChanged.connect(self.change_filter)
         return self.filter_menus[identifier]
 
@@ -642,8 +716,8 @@ class TabularViewMixin:
             position (str): either "before", "after", or ""
         """
         top_indexes, left_indexes = self.pivot_table_model.top_left_indexes()
-        rows = [index.data(Qt.DisplayRole) for index in top_indexes]
-        columns = [index.data(Qt.DisplayRole) for index in left_indexes]
+        rows = [index.data(Qt.ItemDataRole.DisplayRole) for index in top_indexes]
+        columns = [index.data(Qt.ItemDataRole.DisplayRole) for index in left_indexes]
         frozen = self.frozen_table_model.headers
         dropped_list = {"columns": columns, "rows": rows, "frozen": frozen}[dropped.area]
         catcher_list = {"columns": columns, "rows": rows, "frozen": frozen}[catcher.area]
@@ -731,213 +805,20 @@ class TabularViewMixin:
         """
         return list(dict.fromkeys(zip(*[self.pivot_table_model.model.index_values.get(k, []) for k in frozen])).keys())
 
-    # TODO: Move this to the models?
-    @staticmethod
-    def refresh_table_view(table_view):
-        top_left = table_view.indexAt(table_view.rect().topLeft())
-        bottom_right = table_view.indexAt(table_view.rect().bottomRight())
-        if not bottom_right.isValid():
-            model = table_view.model()
-            bottom_right = table_view.model().index(model.rowCount() - 1, model.columnCount() - 1)
-        table_view.model().dataChanged.emit(top_left, bottom_right)
-
-    @Slot(str)
-    def update_filter_menus(self, action):
-        for identifier, menu in self.filter_menus.items():
-            index_values = dict.fromkeys(self.pivot_table_model.model.index_values.get(identifier, []))
-            index_values.pop(None, None)
-            if action == "add":
-                menu.add_items_to_filter_list(list(index_values.keys()))
-            elif action == "remove":
-                previous = menu._filter._filter_model._data_set
-                menu.remove_items_from_filter_list(list(previous - index_values.keys()))
-        self.reload_frozen_table()
-
-    def receive_objects_added_or_removed(self, db_map_data, action):
-        if not self.pivot_table_model:
-            return
-        if self.pivot_table_model.receive_objects_added_or_removed(db_map_data, action):
-            self.update_filter_menus(action)
-
-    def receive_relationships_added_or_removed(self, db_map_data, action):
-        if not self.pivot_table_model:
-            return
-        if self.pivot_table_model.receive_relationships_added_or_removed(db_map_data, action):
-            self.update_filter_menus(action)
-
-    def receive_parameter_definitions_added_or_removed(self, db_map_data, action):
-        if not self.pivot_table_model:
-            return
-        if self.pivot_table_model.receive_parameter_definitions_added_or_removed(db_map_data, action):
-            self.update_filter_menus(action)
-
-    def receive_alternatives_added_or_removed(self, db_map_data, action):
-        if not self.pivot_table_model:
-            return
-        if self.pivot_table_model.receive_alternatives_added_or_removed(db_map_data, action):
-            self.update_filter_menus(action)
-
-    def receive_parameter_values_added_or_removed(self, db_map_data, action):
-        if not self.pivot_table_model:
-            return
-        if self.pivot_table_model.receive_parameter_values_added_or_removed(db_map_data, action):
-            self.update_filter_menus(action)
-
-    def receive_scenarios_added_or_removed(self, db_map_data, action):
-        if not self.pivot_table_model:
-            return
-        if self.pivot_table_model.receive_scenarios_added_or_removed(db_map_data, action):
-            self.update_filter_menus(action)
-
-    def receive_db_map_data_updated(self, db_map_data, get_class_id):
-        if not self.pivot_table_model:
-            return
-        for db_map, items in db_map_data.items():
-            for item in items:
-                if get_class_id(item) == self.current_class_id.get(db_map):
-                    self.refresh_table_view(self.ui.pivot_table)
-                    self.refresh_table_view(self.ui.frozen_table)
-                    self.make_pivot_headers()
-                    return
-
-    def receive_classes_updated(self, db_map_data):
-        if not self.pivot_table_model:
-            return
-        for db_map, items in db_map_data.items():
-            for item in items:
-                if item["id"] == self.current_class_id.get(db_map):
-                    self.do_reload_pivot_table()
-                    return
-
-    def receive_classes_removed(self, db_map_data):
-        if not self.pivot_table_model:
-            return
-        for db_map, items in db_map_data.items():
-            for item in items:
-                if item["id"] == self.current_class_id.get(db_map):
-                    self.current_class_type = None
-                    self.current_class_id = {}
-                    self.clear_pivot_table()
-                    return
-
-    def receive_alternatives_added(self, db_map_data):
-        """Reacts to alternatives added event."""
-        super().receive_alternatives_added(db_map_data)
-        self.receive_alternatives_added_or_removed(db_map_data, action="add")
-
-    def receive_scenarios_added(self, db_map_data):
-        """Reacts to scenarios added event."""
-        super().receive_scenarios_added(db_map_data)
-        self.receive_scenarios_added_or_removed(db_map_data, action="add")
-
-    def receive_objects_added(self, db_map_data):
-        """Reacts to objects added event."""
-        super().receive_objects_added(db_map_data)
-        self.receive_objects_added_or_removed(db_map_data, action="add")
-
-    def receive_relationships_added(self, db_map_data):
-        """Reacts to relationships added event."""
-        super().receive_relationships_added(db_map_data)
-        self.receive_relationships_added_or_removed(db_map_data, action="add")
-
-    def receive_parameter_definitions_added(self, db_map_data):
-        """Reacts to parameter definitions added event."""
-        super().receive_parameter_definitions_added(db_map_data)
-        self.receive_parameter_definitions_added_or_removed(db_map_data, action="add")
-
-    def receive_parameter_values_added(self, db_map_data):
-        """Reacts to parameter values added event."""
-        super().receive_parameter_values_added(db_map_data)
-        self.receive_parameter_values_added_or_removed(db_map_data, action="add")
-
-    def receive_alternatives_updated(self, db_map_data):
-        """Reacts to alternatives updated event."""
-        super().receive_alternatives_updated(db_map_data)
-        if self.pivot_table_model:
-            self.refresh_table_view(self.ui.pivot_table)
-            self.refresh_table_view(self.ui.frozen_table)
-            self.make_pivot_headers()
-
-    def receive_object_classes_updated(self, db_map_data):
-        """Reacts to object classes updated event."""
-        super().receive_object_classes_updated(db_map_data)
-        self.receive_classes_updated(db_map_data)
-
-    def receive_relationship_classes_updated(self, db_map_data):
-        """Reacts to relationship classes updated event."""
-        super().receive_relationship_classes_updated(db_map_data)
-        self.receive_classes_updated(db_map_data)
-
-    def receive_objects_updated(self, db_map_data):
-        """Reacts to objects updated event."""
-        super().receive_objects_updated(db_map_data)
-        self.receive_db_map_data_updated(db_map_data, get_class_id=lambda x: x["class_id"])
-
-    def receive_relationships_updated(self, db_map_data):
-        """Reacts to relationships updated event."""
-        super().receive_relationships_updated(db_map_data)
-        self.receive_db_map_data_updated(db_map_data, get_class_id=lambda x: x["class_id"])
-
-    def receive_parameter_values_updated(self, db_map_data):
-        """Reacts to parameter values added event."""
-        super().receive_parameter_values_updated(db_map_data)
-        self.receive_db_map_data_updated(
-            db_map_data, get_class_id=lambda x: x.get("object_class_id") or x.get("relationship_class_id")
-        )
-
-    def receive_parameter_definitions_updated(self, db_map_data):
-        """Reacts to parameter definitions updated event."""
-        super().receive_parameter_definitions_updated(db_map_data)
-        self.receive_db_map_data_updated(
-            db_map_data, get_class_id=lambda x: x.get("object_class_id") or x.get("relationship_class_id")
-        )
-
-    def receive_scenarios_updated(self, db_map_data):
-        super().receive_scenarios_updated(db_map_data)
-        if self.pivot_table_model:
-            self.pivot_table_model.receive_scenarios_updated(db_map_data)
-
-    def receive_alternatives_removed(self, db_map_data):
-        """Reacts to alternatives removed event."""
-        super().receive_alternatives_removed(db_map_data)
-        self.receive_alternatives_added_or_removed(db_map_data, action="remove")
-
-    def receive_scenarios_removed(self, db_map_data):
-        """Reacts to scenarios removed event."""
-        super().receive_scenarios_removed(db_map_data)
-        self.receive_scenarios_added_or_removed(db_map_data, action="remove")
-
-    def receive_object_classes_removed(self, db_map_data):
-        """Reacts to object classes removed event."""
-        super().receive_object_classes_removed(db_map_data)
-        self.receive_classes_removed(db_map_data)
-
-    def receive_relationship_classes_removed(self, db_map_data):
-        """Reacts to relationship classes remove event."""
-        super().receive_relationship_classes_removed(db_map_data)
-        self.receive_classes_removed(db_map_data)
-
-    def receive_objects_removed(self, db_map_data):
-        """Reacts to objects removed event."""
-        super().receive_objects_removed(db_map_data)
-        self.receive_objects_added_or_removed(db_map_data, action="remove")
-
-    def receive_relationships_removed(self, db_map_data):
-        """Reacts to relationships removed event."""
-        super().receive_relationships_removed(db_map_data)
-        self.receive_relationships_added_or_removed(db_map_data, action="remove")
-
-    def receive_parameter_definitions_removed(self, db_map_data):
-        """Reacts to parameter definitions removed event."""
-        super().receive_parameter_definitions_removed(db_map_data)
-        self.receive_parameter_definitions_added_or_removed(db_map_data, action="remove")
-
-    def receive_parameter_values_removed(self, db_map_data):
-        """Reacts to parameter values removed event."""
-        super().receive_parameter_values_removed(db_map_data)
-        self.receive_parameter_values_added_or_removed(db_map_data, action="remove")
-
     def receive_session_rolled_back(self, db_maps):
         """Reacts to session rolled back event."""
         super().receive_session_rolled_back(db_maps)
         self.clear_pivot_table()
+
+    def accepts_entity_item(self, item, db_map):
+        return item["class_id"] == self.current_class_id.get(db_map)
+
+    def accepts_parameter_item(self, item, db_map):
+        return item["entity_class_id"] == self.current_class_id.get(db_map)
+
+    def accepts_member_object_item(self, item, db_map):
+        object_class_id_list = {x[db_map] for x in self.current_object_class_id_list}
+        return item["class_id"] in object_class_id_list
+
+    def accepts_ith_member_object_item(self, i, item, db_map):
+        return item["class_id"] == self.current_object_class_id_list[i][db_map]
