@@ -19,7 +19,7 @@ import queue
 import threading
 import json
 from spinetoolbox.server.engine_client import EngineClient, ClientSecurityModel
-from spine_engine.exception import RemoteEngineInitFailed
+from spine_engine.exception import EngineInitFailed, RemoteEngineInitFailed
 from spine_engine.server.util.event_data_converter import EventDataConverter
 
 
@@ -112,6 +112,14 @@ class SpineEngineManagerBase:
         """
         raise NotImplementedError()
 
+    def kill_persistent(self, persistent_key):
+        """Kills a persistent process.
+
+        Args:
+            persistent_key (tuple): persistent identifier
+        """
+        raise NotImplementedError()
+
     def get_persistent_completions(self, persistent_key, text):
         """Returns a list of auto-completion options from given text.
 
@@ -150,7 +158,8 @@ class LocalSpineEngineManager(SpineEngineManagerBase):
         return self._engine.get_event()
 
     def stop_engine(self):
-        self._engine.stop()
+        if self._engine is not None:
+            self._engine.stop()
 
     def answer_prompt(self, item_name, accepted):
         self._engine.answer_prompt(item_name, accepted)
@@ -195,6 +204,12 @@ class LocalSpineEngineManager(SpineEngineManagerBase):
 
         interrupt_persistent(persistent_key)
 
+    def kill_persistent(self, persistent_key):
+        # pylint: disable=import-outside-toplevel
+        from spine_engine.execution_managers.persistent_execution_manager import kill_persistent
+
+        kill_persistent(persistent_key)
+
     def get_persistent_completions(self, persistent_key, text):
         # pylint: disable=import-outside-toplevel
         from spine_engine.execution_managers.persistent_execution_manager import get_persistent_completions
@@ -339,6 +354,10 @@ class RemoteSpineEngineManager(SpineEngineManagerBase):
         """See base class."""
         return self.engine_client.send_interrupt_persistent(persistent_key)
 
+    def kill_persistent(self, persistent_key):
+        """See base class."""
+        return self.engine_client.send_kill_persistent(persistent_key)
+
     def get_persistent_completions(self, persistent_key, text):
         """See base class."""
         return self.engine_client.send_get_persistent_completions(persistent_key, text)
