@@ -15,8 +15,8 @@ Contains SpineEngineWorker.
 :date:   14.10.2020
 """
 import copy
-from PySide2.QtCore import Signal, Slot, QObject, QThread
-from PySide2.QtWidgets import QMessageBox
+from PySide6.QtCore import Signal, Slot, QObject, QThread
+from PySide6.QtWidgets import QMessageBox
 from spine_engine.exception import EngineInitFailed, RemoteEngineInitFailed
 from spine_engine.spine_engine import ItemExecutionFinishState, SpineEngineState
 from .spine_engine_manager import make_engine_manager, LocalSpineEngineManager
@@ -71,12 +71,16 @@ def _handle_prompt_arrived(prompt, engine_mngr):
     item_name = prompt["item_name"]
     # pylint: disable=undefined-variable
     box = QMessageBox(
-        QMessageBox.Question, item_name, text, buttons=QMessageBox.Yes | QMessageBox.No, parent=qApp.activeWindow()
+        QMessageBox.Icon.Question,
+        item_name,
+        text,
+        buttons=QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No,
+        parent=qApp.activeWindow(),
     )
     if info_text:
         box.setInformativeText(info_text)
-    answer = box.exec_()
-    accepted = answer == QMessageBox.Yes
+    answer = box.exec()
+    accepted = answer == QMessageBox.StandardButton.Yes
     engine_mngr.answer_prompt(item_name, accepted)
 
 
@@ -100,7 +104,6 @@ def _mark_all_items_failed(items):
 
 
 class SpineEngineWorker(QObject):
-
     finished = Signal()
     _dag_execution_started = Signal(list)
     _node_execution_started = Signal(object, object)
@@ -285,6 +288,8 @@ class SpineEngineWorker(QObject):
                 "Please go to Settings->Tools and check your setup."
             )
             self._event_message_arrived.emit(item, msg["filter_id"], "msg_error", msg_text)
+        elif msg_type == "persistent_killed":
+            self._logger.persistent_killed(item, msg["filter_id"])
         elif msg_type == "stdin":
             self._logger.add_persistent_stdin(item, msg["filter_id"], msg["data"])
         elif msg_type == "stdout":
@@ -296,6 +301,8 @@ class SpineEngineWorker(QObject):
                 item, msg["filter_id"], "msg", f"*** Starting execution on persistent process <b>{msg['args']}</b> ***"
             )
             self._event_message_arrived.emit(item, msg["filter_id"], "msg_warning", "See Console for messages")
+        else:
+            raise RuntimeError(f"Logic error: unknown persistent execution msg_type '{msg_type}'")
 
     def _handle_kernel_execution_msg(self, msg):
         item = self._project_items[msg["item_name"]] or self._connections.get(msg["item_name"])
