@@ -17,7 +17,7 @@ Classes for custom QDialogs for julia setup.
 """
 
 from enum import IntEnum, auto
-from PySide2.QtWidgets import (
+from PySide6.QtWidgets import (
     QWidget,
     QWizard,
     QWizardPage,
@@ -30,8 +30,8 @@ from PySide2.QtWidgets import (
     QCheckBox,
     QRadioButton,
 )
-from PySide2.QtCore import Slot, Qt
-from PySide2.QtGui import QCursor
+from PySide6.QtCore import Slot, Qt
+from PySide6.QtGui import QCursor
 from ..execution_managers import QProcessExecutionManager
 from ..config import REQUIRED_SPINE_OPT_VERSION
 from .custom_qtextbrowser import MonoSpaceFontTextBrowser
@@ -85,7 +85,7 @@ class IntroPage(QWizardPage):
         self.setTitle("Welcome")
         label = HyperTextLabel(
             "This wizard will help you install or upgrade "
-            "<a title='spine opt' href='https://github.com/Spine-project/SpineOpt.jl#spineoptjl'>SpineOpt</a> "
+            "<a title='spine opt' href='https://github.com/spine-tools/SpineOpt.jl#spineoptjl'>SpineOpt</a> "
             "in a Julia project of your choice."
         )
         layout = QVBoxLayout(self)
@@ -170,10 +170,13 @@ class CheckPreviousInstallPage(QWizardPage):
             f"--project={julia_project}",
             "-e",
             'import Pkg; '
-            'manifest = joinpath(dirname(Base.active_project()), "Manifest.toml");'
+            'manifest = joinpath(dirname(Base.active_project()), "Manifest.toml"); '
             'pkgs = isfile(manifest) ? Pkg.TOML.parsefile(manifest) : Dict(); '
-            'spine_opt = get(pkgs, "SpineOpt", nothing); '
-            'if spine_opt != nothing println(spine_opt[1]["version"]) end',
+            'manifest_format = get(pkgs, "manifest_format", missing); '
+            'if manifest_format === missing '
+            'spine_opt = get(pkgs, "SpineOpt", nothing) '
+            'else spine_opt = get(pkgs["deps"], "SpineOpt", nothing) end; '
+            'if spine_opt != nothing println(spine_opt[1]["version"]) end; ',
         ]
         self._exec_mngr = QProcessExecutionManager(self, julia_exe, args, silent=True)
         self.completeChanged.emit()
@@ -239,7 +242,7 @@ class AddUpSpineOptPage(QWizardProcessPage):
         processing, code, process = {
             "add": (
                 "Installing",
-                'using Pkg; pkg"registry add General https://github.com/Spine-project/SpineJuliaRegistry.git"; '
+                'using Pkg; pkg"registry add General https://github.com/spine-tools/SpineJuliaRegistry.git"; '
                 'pkg"add SpineOpt"',
                 "installation",
             ),
@@ -336,7 +339,7 @@ class TroubleshootProblemsPage(QWizardPage):
         msg1a.append(
             """
             \u22ee<br>
-            Updating git-repo `https://github.com/Spine-project/SpineJuliaRegistry`<br>
+            Updating git-repo `https://github.com/spine-tools/SpineJuliaRegistry`<br>
             Resolving package versions...<br>
             ERROR: expected package `UUIDs [cf7118a7]` to be registered<br>
             \u22ee
@@ -345,7 +348,7 @@ class TroubleshootProblemsPage(QWizardPage):
         msg1b.append(
             """
             \u22ee<br>
-            Updating git-repo `https://github.com/Spine-project/SpineJuliaRegistry`<br>
+            Updating git-repo `https://github.com/spine-tools/SpineJuliaRegistry`<br>
             Resolving package versions...<br>
             ERROR: cannot find name corresponding to UUID f269a46b-ccf7-5d73-abea-4c690281aa53 in a registry<br>
             \u22ee
@@ -516,7 +519,7 @@ class TotalFailurePage(QWizardPage):
     def __init__(self, parent):
         super().__init__(parent)
         self.setTitle("Troubleshooting failed")
-        msg = "<p>Please <a href=https://github.com/Spine-project/SpineOpt.jl/issues>open an issue with SpineOpt</a>."
+        msg = "<p>Please <a href=https://github.com/spine-tools/SpineOpt.jl/issues>open an issue with SpineOpt</a>."
         layout = QVBoxLayout(self)
         layout.addWidget(HyperTextLabel(msg))
 
