@@ -20,9 +20,9 @@ from pygments.styles import get_style_by_name
 from pygments.lexers import get_lexer_by_name
 from pygments.util import ClassNotFound
 from pygments.token import Token
-from PySide2.QtWidgets import QWidget, QPlainTextEdit, QPlainTextDocumentLayout
-from PySide2.QtGui import QColor, QFontMetrics, QFontDatabase, QPainter, QTextDocument
-from PySide2.QtCore import QSize, Slot, QRect, Qt
+from PySide6.QtWidgets import QWidget, QPlainTextEdit, QPlainTextDocumentLayout
+from PySide6.QtGui import QColor, QFontMetrics, QFontDatabase, QPainter
+from PySide6.QtCore import QSize, Slot, QRect, Qt
 from spinetoolbox.helpers import CustomSyntaxHighlighter
 
 
@@ -38,7 +38,7 @@ class CodeTextEdit(QPlainTextEdit):
         self._right_margin = 16
         font = QFontDatabase.systemFont(QFontDatabase.FixedFont)
         self.setFont(font)
-        foreground_color = self._style.styles[Token.Text]
+        foreground_color = self._style.styles[Token]
         self.setStyleSheet(
             f"QPlainTextEdit {{background-color: {self._style.background_color}; color: {foreground_color};}}"
         )
