@@ -17,8 +17,8 @@ Custom editors for model/view programming.
 :date:   2.9.2018
 """
 
-from PySide2.QtCore import Qt, Slot, Signal, QSortFilterProxyModel, QEvent, QCoreApplication, QModelIndex, QPoint, QSize
-from PySide2.QtWidgets import (
+from PySide6.QtCore import Qt, Slot, Signal, QSortFilterProxyModel, QEvent, QCoreApplication, QModelIndex, QPoint, QSize
+from PySide6.QtWidgets import (
     QLineEdit,
     QTableView,
     QStyledItemDelegate,
@@ -32,7 +32,7 @@ from PySide2.QtWidgets import (
     QStyle,
     QLabel,
 )
-from PySide2.QtGui import QStandardItemModel, QStandardItem, QColor, QIcon, QPixmap, QPainter
+from PySide6.QtGui import QPalette, QStandardItemModel, QStandardItem, QColor
 from ..helpers import IconListManager, interpret_icon_id, make_icon_id, try_number_from_string
 
 
@@ -64,18 +64,20 @@ class CustomLineEditor(QLineEdit):
 
 class ParameterValueLineEditor(CustomLineEditor):
     def set_data(self, data):
+        """See base class."""
         if data is not None and not isinstance(data, str):
             self.setAlignment(Qt.AlignRight)
         super().set_data(data)
 
     def data(self):
+        """See base class."""
         return try_number_from_string(super().data())
 
 
 class _CustomLineEditDelegate(QStyledItemDelegate):
     """A delegate for placing a CustomLineEditor on the first row of SearchBarEditor."""
 
-    text_edited = Signal("QString")
+    text_edited = Signal(str)
 
     def setModelData(self, editor, model, index):
         model.setData(index, editor.data())
@@ -109,8 +111,7 @@ class SearchBarEditor(QTableView):
     data_committed = Signal()
 
     def __init__(self, parent, tutor=None):
-        """Initializes instance.
-
+        """
         Args:
             parent (QWidget): parent widget
             tutor (QWidget, optional): another widget used for positioning.
@@ -121,40 +122,51 @@ class SearchBarEditor(QTableView):
         self._original_text = None
         self._orig_pos = None
         self.first_index = QModelIndex()
-        self.model = QStandardItemModel(self)
+        self._model = QStandardItemModel(self)
         self.proxy_model = QSortFilterProxyModel(self)
-        self.proxy_model.setSourceModel(self.model)
+        self.proxy_model.setFilterCaseSensitivity(Qt.CaseSensitivity.CaseInsensitive)
+        self.proxy_model.setSourceModel(self._model)
         self.proxy_model.filterAcceptsRow = self._proxy_model_filter_accepts_row
         self.setModel(self.proxy_model)
         self.verticalHeader().hide()
         self.horizontalHeader().hide()
         self.setShowGrid(False)
-        self.setMouseTracking(True)
         self.setTabKeyNavigation(False)
         delegate = _CustomLineEditDelegate(self)
         delegate.text_edited.connect(self._handle_delegate_text_edited)
         self.setItemDelegateForRow(0, delegate)
+        hover_color = self.palette().color(QPalette.ColorGroup.Active, QPalette.ColorRole.Highlight).lighter(220)
+        self.setStyleSheet(f"QTableView::item:hover {{background: {hover_color.name()};}}")
 
     def set_data(self, current, items):
         """Populates model.
 
         Args:
             current (str): item that is currently selected from given items
-            items (Sequence(str)): items to show in the list
+            items (Sequence of str): items to show in the list
         """
         item_list = [QStandardItem(current)]
         for item in items:
             qitem = QStandardItem(item)
             item_list.append(qitem)
             qitem.setFlags(~Qt.ItemIsEditable)
-        self.model.invisibleRootItem().appendRows(item_list)
-        self.first_index = self.proxy_model.mapFromSource(self.model.index(0, 0))
+        self._model.invisibleRootItem().appendRows(item_list)
+        self.first_index = self.proxy_model.mapFromSource(self._model.index(0, 0))
 
     def set_base_offset(self, offset):
+        """Changes the base offset that is applied to the editor's position.
+
+        Args:
+            offset (QPoint): new offset
+        """
         self._base_offset = offset
 
     def update_geometry(self, option):
-        """Updates geometry."""
+        """Updates geometry.
+
+        Args:
+            option (QStyleOptionViewItem): style information
+        """
         self.resizeColumnsToContents()
         self.verticalHeader().setDefaultSectionSize(option.rect.height())
         self._orig_pos = self.pos() + self._base_offset
@@ -163,12 +175,13 @@ class SearchBarEditor(QTableView):
         self.refit()
 
     def refit(self):
+        """Changes the position and size of the editor to fit the window."""
         self.move(self._orig_pos)
         margins = self.contentsMargins()
         table_height = self.verticalHeader().length() + margins.top() + margins.bottom()
         table_width = self.horizontalHeader().length() + margins.left() + margins.right()
         if table_height > self.parent().size().height():
-            table_width += self.style().pixelMetric(QStyle.PM_ScrollBarExtent)
+            table_width += self.style().pixelMetric(QStyle.PixelMetric.PM_ScrollBarExtent)
         size = QSize(table_width, table_height).boundedTo(self.parent().size())
         self.resize(size)
         # Adjust position if widget is outside parent's limits
@@ -179,18 +192,42 @@ class SearchBarEditor(QTableView):
         self.move(self.pos() - QPoint(x_offset, y_offset))
 
     def data(self):
-        return self.first_index.data(Qt.EditRole)
+        """Returns editor's final data.
+
+        Returns:
+            str: editor data
+        """
+        first_data = self.first_index.data(Qt.ItemDataRole.EditRole)
+        if not first_data:
+            return None
+        model = self.model()
+        rows = model.rowCount()
+        if any(model.index(row, 0).data(Qt.ItemDataRole.EditRole) == first_data for row in range(1, rows)):
+            return first_data
+        return model.index(1, 0).data(Qt.ItemDataRole.EditRole)
 
     @Slot(str)
     def _handle_delegate_text_edited(self, text):
-        """Filters model as the first row is being edited."""
+        """Filters model as the first row is being edited.
+
+        Args:
+            text (str): text the user has entered on the first row
+        """
         self._original_text = text
-        self.proxy_model.setFilterRegExp("^" + text)
+        self.proxy_model.setFilterRegularExpression("^" + text)
         self.proxy_model.setData(self.first_index, text)
         self.refit()
 
     def _proxy_model_filter_accepts_row(self, source_row, source_parent):
-        """Always accept first row."""
+        """Always accept first row while filtering the rest.
+
+        Args:
+            source_row (int): source row index
+            source_parent (QModelIndex): parent index for source row
+
+        Returns:
+            bool: True if row is accepted, False otherwise
+        """
         if source_row == 0:
             return True
         return QSortFilterProxyModel.filterAcceptsRow(self.proxy_model, source_row, source_parent)
@@ -201,7 +238,6 @@ class SearchBarEditor(QTableView):
         """
         super().keyPressEvent(event)
         event.accept()  # Important to avoid unhandled behavior when trying to navigate outside view limits
-        # Initialize original text. TODO: Is there a better place for this?
         if self._original_text is None:
             self.proxy_model.setData(self.first_index, event.text())
             self._handle_delegate_text_edited(event.text())
@@ -225,34 +261,28 @@ class SearchBarEditor(QTableView):
             return
         self.edit(self.first_index)
 
-    def mouseMoveEvent(self, event):
-        """Sets the current index to the one hovered by the mouse."""
-        if not self.currentIndex().isValid():
-            return
-        index = self.indexAt(event.pos())
-        if index.row() == 0:
-            return
-        self.setCurrentIndex(index)
-
     def mousePressEvent(self, event):
         """Commits data."""
-        index = self.indexAt(event.pos())
+        index = self.indexAt(event.position().toPoint())
         if index.row() == 0:
             return
-        self.proxy_model.setData(self.first_index, index.data(Qt.EditRole))
+        self.proxy_model.setData(self.first_index, index.data(Qt.ItemDataRole.EditRole))
         self.data_committed.emit()
 
 
 class CheckListEditor(QTableView):
     """A check list editor."""
 
-    def __init__(self, parent, tutor=None, ranked=False):
-        """Initialize class."""
+    def __init__(self, parent, tutor=None):
+        """
+        Args:
+            parent (QWidget): parent widget
+            tutor (QWidget, optional): a widget that helps in positioning
+        """
         super().__init__(parent)
         self._tutor = tutor
-        self._ranked = ranked
-        self.model = QStandardItemModel(self)
-        self.setModel(self.model)
+        self._model = QStandardItemModel(self)
+        self.setModel(self._model)
         self.verticalHeader().hide()
         self.horizontalHeader().hide()
         self.setShowGrid(False)
@@ -260,18 +290,6 @@ class CheckListEditor(QTableView):
         self._icons = []
         self._selected = []
         self._items = {}
-        self._blank_icon = self._make_icon()
-
-    def _make_icon(self, i=None):
-        if not self._ranked:
-            return None
-        pixmap = QPixmap(16, 16)
-        pixmap.fill(Qt.white)
-        if i is not None:
-            painter = QPainter(pixmap)
-            painter.drawText(0, 0, 16, 16, Qt.AlignCenter, str(i))
-            painter.end()
-        return QIcon(pixmap)
 
     def keyPressEvent(self, event):
         """Toggles checked state if the user presses space."""
@@ -286,40 +304,24 @@ class CheckListEditor(QTableView):
         Args:
             index (QModelIndex): index to toggle
         """
-        item = self.model.itemFromIndex(index).text()
+        item = self._model.itemFromIndex(index).text()
         qitem = self._items[item]
         if item not in self._selected:
             rank = len(self._selected)
-            self._select_item(qitem, rank)
+            qitem.setCheckState(Qt.CheckState.Checked)
             self._selected.append(item)
         else:
             self._selected.remove(item)
-            self._deselect_item(qitem, update_ranks=True)
-
-    def _select_item(self, qitem, rank):
-        if self._ranked:
-            qitem.setData(self._icons[rank], Qt.DecorationRole)
-        else:
-            qitem.setCheckState(Qt.Checked)
-
-    def _deselect_item(self, qitem, update_ranks=False):
-        if self._ranked:
-            qitem.setData(self._blank_icon, Qt.DecorationRole)
-            if update_ranks:
-                for rank, item in enumerate(self._selected):
-                    qitem = self._items[item]
-                    self._select_item(qitem, rank)
-        else:
-            qitem.setCheckState(Qt.Unchecked)
+            qitem.setCheckState(Qt.CheckState.Unchecked)
 
     def mouseMoveEvent(self, event):
         """Sets the current index to the one under mouse."""
-        index = self.indexAt(event.pos())
+        index = self.indexAt(event.position().toPoint())
         self.setCurrentIndex(index)
 
     def mousePressEvent(self, event):
         """Toggles checked state of pressed index."""
-        index = self.indexAt(event.pos())
+        index = self.indexAt(event.position().toPoint())
         self.toggle_selected(index)
 
     def set_data(self, items, checked_items):
@@ -329,18 +331,17 @@ class CheckListEditor(QTableView):
             items (Sequence(str)): All items.
             checked_items (Sequence(str)): Initially checked items.
         """
-        self._icons = [self._make_icon(i + 1) for i in range(len(items))]
         for item in items:
             qitem = QStandardItem(item)
             qitem.setFlags(~Qt.ItemIsEditable)
-            qitem.setData(qApp.palette().window(), Qt.BackgroundRole)  # pylint: disable=undefined-variable
-            self._deselect_item(qitem)
+            qitem.setData(qApp.palette().window(), Qt.ItemDataRole.BackgroundRole)  # pylint: disable=undefined-variable
+            qitem.setCheckState(Qt.CheckState.Unchecked)
             self._items[item] = qitem
-            self.model.appendRow(qitem)
+            self._model.appendRow(qitem)
         self._selected = [item for item in checked_items if item in items]
-        for rank, item in enumerate(self._selected):
+        for item in self._selected:
             qitem = self._items[item]
-            self._select_item(qitem, rank)
+            qitem.setCheckState(Qt.CheckState.Checked)
 
     def data(self):
         """Returns a comma separated list of checked items.
@@ -351,14 +352,18 @@ class CheckListEditor(QTableView):
         return ",".join(self._selected)
 
     def update_geometry(self, option):
-        """Updates geometry."""
+        """Updates geometry.
+
+        Args:
+            option (QStyleOptionViewItem): style information
+        """
         self.resizeColumnsToContents()
         self.verticalHeader().setDefaultSectionSize(option.rect.height())
         margins = self.contentsMargins()
         table_height = self.verticalHeader().length() + margins.top() + margins.bottom()
         table_width = self.horizontalHeader().length() + margins.left() + margins.right()
         if table_height > self.parent().size().height():
-            table_width += self.style().pixelMetric(QStyle.PM_ScrollBarExtent)
+            table_width += self.style().pixelMetric(QStyle.PixelMetric.PM_ScrollBarExtent)
         size = QSize(table_width, table_height).boundedTo(self.parent().size())
         self.resize(size)
         if self._tutor:
@@ -376,7 +381,7 @@ class _IconPainterDelegate(QStyledItemDelegate):
 
     def paint(self, painter, option, index):
         """Paints selected items using the highlight brush."""
-        if option.state & QStyle.State_Selected:
+        if option.state & QStyle.StateFlag.State_Selected:
             painter.fillRect(option.rect, qApp.palette().highlight())  # pylint: disable=undefined-variable
         super().paint(painter, option, index)
 
@@ -385,7 +390,10 @@ class IconColorEditor(QDialog):
     """An editor to let the user select an icon and a color for an object_class."""
 
     def __init__(self, parent):
-        """Init class."""
+        """
+        Args:
+            parent (QWidget): parent widget
+        """
         super().__init__(parent)
         icon_size = QSize(32, 32)
         self.icon_mngr = IconListManager(icon_size)
@@ -405,11 +413,11 @@ class IconColorEditor(QDialog):
         icon_widget_layout.addWidget(self.line_edit)
         icon_widget_layout.addWidget(self.icon_list)
         self.color_dialog = QColorDialog(self)
-        self.color_dialog.setWindowFlags(Qt.Widget)
+        self.color_dialog.setWindowFlags(Qt.WindowType.Widget)
         self.color_dialog.setOption(QColorDialog.NoButtons, True)
         self.color_dialog.setOption(QColorDialog.DontUseNativeDialog, True)
         self.button_box = QDialogButtonBox(self)
-        self.button_box.setStandardButtons(QDialogButtonBox.Cancel | QDialogButtonBox.Ok)
+        self.button_box.setStandardButtons(QDialogButtonBox.StandardButton.Cancel | QDialogButtonBox.StandardButton.Ok)
         top_widget = QWidget(self)
         top_layout = QHBoxLayout(top_widget)
         top_layout.addWidget(self.icon_widget)
@@ -425,11 +433,19 @@ class IconColorEditor(QDialog):
         self.connect_signals()
 
     def _proxy_model_filter_accepts_row(self, source_row, source_parent):
-        """Filters icons according to search terms."""
+        """Filters icons according to search terms.
+
+        Args:
+            source_row (int): source row index
+            source_parent (QModelIndex): parent index for source row
+
+        Returns:
+            bool: True if row is accepted, False otherwise
+        """
         text = self.line_edit.text()
         if not text:
             return QSortFilterProxyModel.filterAcceptsRow(self.proxy_model, source_row, source_parent)
-        searchterms = self.icon_mngr.model.index(source_row, 0, source_parent).data(Qt.UserRole + 1)
+        searchterms = self.icon_mngr.model.index(source_row, 0, source_parent).data(Qt.ItemDataRole.UserRole + 1)
         return any(text in term for term in searchterms)
 
     def connect_signals(self):
@@ -439,16 +455,26 @@ class IconColorEditor(QDialog):
         self.button_box.rejected.connect(self.reject)
 
     def set_data(self, data):
+        """Sets current icon data.
+
+        Args:
+            data (int): database icon data
+        """
         icon_code, color_code = interpret_icon_id(data)
         self.icon_mngr.init_model()
         for i in range(self.proxy_model.rowCount()):
             index = self.proxy_model.index(i, 0)
-            if index.data(Qt.UserRole) == icon_code:
+            if index.data(Qt.ItemDataRole.UserRole) == icon_code:
                 self.icon_list.setCurrentIndex(index)
                 break
         self.color_dialog.setCurrentColor(QColor(color_code))
 
     def data(self):
-        icon_code = self.icon_list.currentIndex().data(Qt.UserRole)
+        """Gets current icon data.
+
+        Returns:
+            int: database icon data
+        """
+        icon_code = self.icon_list.currentIndex().data(Qt.ItemDataRole.UserRole)
         color_code = self.color_dialog.currentColor().rgb()
         return make_icon_id(icon_code, color_code)
