@@ -17,9 +17,10 @@ Classes for custom context menus and pop-up menus.
 """
 
 import os
-from PySide2.QtWidgets import QAction, QMenu
-from PySide2.QtGui import QIcon
-from PySide2.QtCore import Slot, QPersistentModelIndex
+from PySide6.QtWidgets import QMenu, QWidgetAction
+from PySide6.QtGui import QIcon, QAction
+from PySide6.QtCore import Slot, QPersistentModelIndex
+from spinetoolbox.widgets.custom_qwidgets import FilterWidget
 
 
 class CustomContextMenu(QMenu):
@@ -58,7 +59,7 @@ class CustomContextMenu(QMenu):
 
     def get_action(self):
         """Returns the clicked action, a string with a description."""
-        self.exec_(self.position)
+        self.exec(self.position)
         return self.option
 
 
@@ -84,7 +85,7 @@ class CustomPopupMenu(QMenu):
         super().__init__(parent=parent)
         self._parent = parent
 
-    def add_action(self, text, slot, enabled=True, tooltip=None):
+    def add_action(self, text, slot, enabled=True, tooltip=None, icon=None):
         """Adds an action to the popup menu.
 
         Args:
@@ -92,8 +93,12 @@ class CustomPopupMenu(QMenu):
             slot (method): Method to connect to action's triggered signal
             enabled (bool): Is action enabled?
             tooltip (str): Tool tip for the action
+            icon (QIcon): Action icon
         """
-        action = self.addAction(text, slot)
+        if icon is not None:
+            action = self.addAction(icon, text, slot)
+        else:
+            action = self.addAction(text, slot)
         action.setEnabled(enabled)
         if tooltip is not None:
             action.setToolTip(tooltip)
@@ -129,6 +134,17 @@ class RecentProjectsPopupMenu(CustomPopupMenu):
         self._parent = parent
         self.setToolTipsVisible(True)
         self.add_recent_projects()
+        self.addSeparator()
+        self.add_action(
+            "Clear",
+            lambda checked=False: self.call_clear_recents(checked),
+            enabled=self.has_recents(),
+            icon=QIcon(":icons/trash-alt.svg"),
+        )
+
+    def has_recents(self):
+        """Returns True if recent projects available, False otherwise."""
+        return bool(self._parent.qsettings().value("appSettings/recentProjects", defaultValue=None))
 
     def add_recent_projects(self):
         """Reads the previous project names and paths from QSettings. Adds them to the QMenu as QActions."""
@@ -144,6 +160,16 @@ class RecentProjectsPopupMenu(CustomPopupMenu):
                     tooltip=filepath,
                 )
 
+    @Slot(bool)
+    def call_clear_recents(self, checked):
+        """Slot for Clear recents menu item.
+
+        Args:
+            checked (bool): Argument sent by triggered signal
+        """
+
+        self._parent.clear_recent_projects()
+
     @Slot(bool, str)
     def call_open_project(self, checked, p):
         """Slot for catching the user selected action from the recent projects menu.
@@ -177,20 +203,23 @@ class FilterMenuBase(QMenu):
             parent (QWidget): a parent widget
         """
         super().__init__(parent)
-        self._remove_filter = QAction('Remove filters', None)
         self._filter = None
-        self._filter_action = None
+        self._remove_filter = QAction('Remove filters', None)
+        self._filter_action = QWidgetAction(self)
         self.addAction(self._remove_filter)
 
+    def _set_up(self, make_filter_model, *args, **kwargs):
+        self._filter = FilterWidget(self, make_filter_model, *args, **kwargs)
+        self._filter_action.setDefaultWidget(self._filter)
+        self.addAction(self._filter_action)
+        self.connect_signals()
+
     def connect_signals(self):
         self.aboutToShow.connect(self._check_filter)
         self._remove_filter.triggered.connect(self._clear_filter)
         self._filter.okPressed.connect(self._change_filter)
         self._filter.cancelPressed.connect(self.hide)
 
-    def set_filter_list(self, data):
-        self._filter.set_filter_list(data)
-
     def add_items_to_filter_list(self, items):
         self._filter._filter_model.add_items(items)
         self._filter.save_state()
