@@ -23,9 +23,11 @@ import locale
 from contextlib import contextmanager
 from numbers import Number
 import re
-from PySide2.QtWidgets import QTableView, QApplication, QAction
-from PySide2.QtCore import Qt, Slot, QItemSelection, QItemSelectionModel, QPoint
-from PySide2.QtGui import QKeySequence, QIcon
+from operator import methodcaller
+
+from PySide6.QtWidgets import QTableView, QApplication
+from PySide6.QtCore import Qt, Slot, QItemSelection, QItemSelectionModel, QPoint
+from PySide6.QtGui import QKeySequence, QIcon, QAction
 from spinedb_api import (
     DateTime,
     Duration,
@@ -123,7 +125,7 @@ class CopyPasteTableView(QTableView):
                     for j in range(rng.left(), rng.right() + 1):
                         if h_header.isSectionHidden(j):
                             continue
-                        data = self.model().index(i, j).data(Qt.EditRole)
+                        data = self.model().index(i, j).data(Qt.ItemDataRole.EditRole)
                         if data is not None:
                             try:
                                 number = float(data)
@@ -166,7 +168,7 @@ class CopyPasteTableView(QTableView):
         """
 
         def _process_value(value):
-            """Delocalizes value, expect when it's one of our 'complex' value types.
+            """Delocalizes value, except when it's one of our 'complex' value types.
             We need this exception because our complex values are json strings, so they have commas,
             and ``locale.delocalize`` might remove those commas.
 
@@ -290,7 +292,7 @@ class AutoFilterCopyPasteTableView(CopyPasteTableView):
         """
         super().__init__(parent=parent)
         self._show_filter_menu_action = QAction(self)
-        self._show_filter_menu_action.setShortcut(Qt.ALT + Qt.Key_Down)
+        self._show_filter_menu_action.setShortcut(Qt.ALT | Qt.Key_Down)
         self._show_filter_menu_action.setShortcutContext(Qt.WidgetShortcut)
         self._show_filter_menu_action.triggered.connect(self._trigger_filter_menu)
         self.addAction(self._show_filter_menu_action)
@@ -352,7 +354,7 @@ class IndexedParameterValueTableViewBase(CopyPasteTableView):
         data_indexes = row_count * [None]
         data_values = row_count * [None]
         for selected_index in selected_indexes:
-            data = data_model.data(selected_index)
+            data = data_model.data(selected_index, Qt.ItemDataRole.EditRole)
             row = selected_index.row()
             if selected_index.column() == 0:
                 data_indexes[row - row_first] = data
@@ -611,15 +613,31 @@ class ArrayTableView(IndexedParameterValueTableViewBase):
             return False
         model = self.model()
         selected_indexes = [i for i in selection_model.selectedIndexes() if not model.is_expanse_row(i.row())]
-        selected_indexes.sort(key=lambda index: index.row())
-        values = [index.data() for index in selected_indexes]
-        with system_lc_numeric():
-            with io.StringIO() as output:
-                writer = csv.writer(output, delimiter='\t')
-                for value in values:
-                    value = locale.str(value) if isinstance(value, Number) else value
-                    writer.writerow([value])
-                QApplication.clipboard().setText(output.getvalue())
+        selected_indexes.sort(key=methodcaller("row"))
+        first_column = selected_indexes[0].column()
+        if any(index.column() != first_column for index in selected_indexes):
+            values = {}
+            for index in selected_indexes:
+                row = index.row()
+                row_values = values.setdefault(row, {})
+                row_values["x" if index.column() == 0 else "y"] = index.data(Qt.ItemDataRole.EditRole)
+            with system_lc_numeric():
+                with io.StringIO() as output:
+                    writer = csv.writer(output, delimiter='\t')
+                    for row_values in values.values():
+                        x = row_values.get("x", "")
+                        y = row_values.get("y", "")
+                        y = locale.str(y) if isinstance(y, Number) else y
+                        writer.writerow([x, y])
+                    QApplication.clipboard().setText(output.getvalue())
+        else:
+            with system_lc_numeric():
+                with io.StringIO() as output:
+                    writer = csv.writer(output, delimiter='\t')
+                    for index in selected_indexes:
+                        y = index.data(Qt.ItemDataRole.EditRole)
+                        writer.writerow([locale.str(y) if isinstance(y, Number) else y])
+                    QApplication.clipboard().setText(output.getvalue())
         return True
 
     @Slot(bool)
@@ -711,7 +729,7 @@ class MapTableView(CopyPasteTableView):
                     index = model.index(y, x)
                     if not selection.contains(index):
                         continue
-                    data = index.data(Qt.EditRole)
+                    data = index.data(Qt.ItemDataRole.EditRole)
                     try:
                         number = float(data)
                         str_data = locale.str(number)
