@@ -16,12 +16,11 @@ Custom QWidgets for Filtering and Zooming.
 :date:   4.12.2018
 """
 
-from PySide2.QtWidgets import (
+from PySide6.QtWidgets import (
     QWidget,
     QVBoxLayout,
     QHBoxLayout,
     QMenu,
-    QAction,
     QStyle,
     QToolBar,
     QStyleOptionMenuItem,
@@ -34,12 +33,21 @@ from PySide2.QtWidgets import (
     QWizardPage,
     QToolButton,
     QPushButton,
-    QUndoStack,
     QSpinBox,
     QDialog,
 )
-from PySide2.QtCore import Qt, QTimer, Signal, Slot, QSize, QEvent, QRect
-from PySide2.QtGui import QPainter, QFontMetrics, QKeyEvent, QFontDatabase, QFont, QIntValidator, QKeySequence
+from PySide6.QtCore import Qt, QTimer, Signal, Slot, QSize, QEvent, QRect
+from PySide6.QtGui import (
+    QPainter,
+    QFontMetrics,
+    QKeyEvent,
+    QFontDatabase,
+    QFont,
+    QIntValidator,
+    QKeySequence,
+    QAction,
+    QUndoStack,
+)
 from .custom_qtextbrowser import MonoSpaceFontTextBrowser
 from .select_database_items import SelectDatabaseItems
 from ..helpers import format_log_message
@@ -95,17 +103,20 @@ class UndoRedoMixin:
             super().keyPressEvent(e)
 
 
-class FilterWidgetBase(QWidget):
+class FilterWidget(QWidget):
     """Filter widget class."""
 
     okPressed = Signal()
     cancelPressed = Signal()
 
-    def __init__(self, parent):
+    def __init__(self, parent, make_filter_model, *args, **kwargs):
         """Init class.
 
         Args:
-            parent (QWidget)
+            parent (QWidget, optional): parent widget
+            make_filter_model (Callable): callable that constructs the filter model
+            *args: arguments forwarded to ``make_filter_model``
+            **kwargs: keyword arguments forwarded to ``make_filter_model``
         """
         super().__init__(parent)
         # parameters
@@ -113,30 +124,34 @@ class FilterWidgetBase(QWidget):
         self._filter_empty_state = None
         self._search_text = ''
         self.search_delay = 200
-
         # create ui elements
         self._ui_vertical_layout = QVBoxLayout(self)
         self._ui_list = QListView()
         self._ui_edit = QLineEdit()
         self._ui_edit.setPlaceholderText('Search')
         self._ui_edit.setClearButtonEnabled(True)
-        self._ui_buttons = QDialogButtonBox(QDialogButtonBox.Cancel | QDialogButtonBox.Ok)
+        self._ui_buttons = QDialogButtonBox(QDialogButtonBox.StandardButton.Cancel | QDialogButtonBox.StandardButton.Ok)
         self._ui_vertical_layout.addWidget(self._ui_edit)
         self._ui_vertical_layout.addWidget(self._ui_list)
         self._ui_vertical_layout.addWidget(self._ui_buttons)
-
-        # add models
         self._search_timer = QTimer()  # Used to limit search so it doesn't search when typing
         self._search_timer.setSingleShot(True)
+        self._filter_model = make_filter_model(*args, **kwargs)
+        self._filter_model.set_list(self._filter_state)
+        self._ui_list.setModel(self._filter_model)
+        self.connect_signals()
 
-        self._filter_model = None
+    # For tests
+    def set_filter_list(self, items):
+        self._filter_state = items
+        self._filter_model.set_list(self._filter_state)
 
     def connect_signals(self):
         self._ui_list.clicked.connect(self._filter_model._handle_index_clicked)
         self._search_timer.timeout.connect(self._filter_list)
         self._ui_edit.textChanged.connect(self._text_edited)
-        self._ui_buttons.button(QDialogButtonBox.Ok).clicked.connect(self._apply_filter)
-        self._ui_buttons.button(QDialogButtonBox.Cancel).clicked.connect(self._cancel_filter)
+        self._ui_buttons.button(QDialogButtonBox.StandardButton.Ok).clicked.connect(self._apply_filter)
+        self._ui_buttons.button(QDialogButtonBox.StandardButton.Cancel).clicked.connect(self._cancel_filter)
 
     def save_state(self):
         """Saves the state of the FilterCheckboxListModel."""
@@ -157,11 +172,6 @@ class FilterWidgetBase(QWidget):
         """Returns true if any item is filtered in FilterCheckboxListModel false otherwise."""
         return not self._filter_model._all_selected
 
-    def set_filter_list(self, data):
-        """Sets the list of items to filter."""
-        self._filter_state = list(data)
-        self._filter_model.set_list(self._filter_state)
-
     def _apply_filter(self):
         """Apply current filter and save state."""
         self._filter_model.apply_filter()
@@ -271,24 +281,22 @@ class ToolBarWidgetBase(QWidget):
         self.tool_bar = _MenuToolBar(self)
         layout.addStretch()
         layout.addWidget(self.tool_bar)
-        icon_extent = qApp.style().pixelMetric(QStyle.PM_SmallIconSize)  # pylint: disable=undefined-variable
+        icon_extent = qApp.style().pixelMetric(
+            QStyle.PixelMetric.PM_SmallIconSize
+        )  # pylint: disable=undefined-variable
         self.tool_bar.setIconSize(QSize(icon_extent, icon_extent))
-        self.tool_bar.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)
+        self.tool_bar.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextUnderIcon)
 
 
 class ToolBarWidget(ToolBarWidgetBase):
     def __init__(self, text, parent=None):
         super().__init__(text, parent)
-        spacing = qApp.fontMetrics().horizontalAdvance(self._text)  # pylint: disable=undefined-variable
+        spacing = self.fontMetrics().horizontalAdvance(self._text)  # pylint: disable=undefined-variable
         self.layout().insertSpacing(0, spacing)
 
 
 class MenuItemToolBarWidget(ToolBarWidgetBase):
-    """A menu item with a toolbar on the right.
-
-    Attributes:
-        tool_bar (QToolBar)
-    """
+    """A menu item with a toolbar on the right."""
 
     def __init__(self, text, parent=None, compact=False):
         """Class constructor.
@@ -305,14 +313,17 @@ class MenuItemToolBarWidget(ToolBarWidgetBase):
         if compact:
             self.tool_bar.setFixedHeight(self.option.rect.height())
         text_width = self.option.fontMetrics.horizontalAdvance(self._text)
-        icon_width = qApp.style().pixelMetric(QStyle.PM_ToolBarIconSize)  # pylint: disable=undefined-variable
+        icon_width = qApp.style().pixelMetric(
+            QStyle.PixelMetric.PM_ToolBarIconSize
+        )  # pylint: disable=undefined-variable
         spacing = text_width + 3 * icon_width
+        self.option.rect.setWidth(spacing)
         self.layout().insertSpacing(0, spacing)
 
     def paintEvent(self, event):
         """Draws the menu item, then calls the super() method to draw the tool bar."""
         painter = QPainter(self)
-        self.style().drawControl(QStyle.CE_MenuItem, self.option, painter)
+        self.style().drawControl(QStyle.ControlElement.CE_MenuItem, self.option, painter)
         super().paintEvent(event)
 
 
@@ -406,7 +417,7 @@ class _MenuToolBar(QToolBar):
         Must be called everytime an action is added to the tool bar.
 
         Args:
-            QAction
+            action (QAction): Action to set up
         """
         button = self.widgetForAction(action)
         if not button:
@@ -668,7 +679,7 @@ class HorizontalSpinBox(QToolBar):
             return
         if value == self._value:
             return
-        acceptable = self._validator.validate(str(value), 0)[0] == QIntValidator.Acceptable
+        acceptable = self._validator.validate(str(value), 0)[0] == QIntValidator.State.Acceptable
         if strict and not acceptable:
             return
         self._line_edit.setText(str(value))
@@ -693,30 +704,74 @@ class PropertyQSpinBox(UndoRedoMixin, QSpinBox):
     """A spinbox where undo and redo key strokes apply to the project."""
 
 
-class PurgeSettingsDialog(QDialog):
-    """Purge settings dialog."""
+class SelectDatabaseItemsDialog(QDialog):
+    """Dialog that lets selecting database items."""
 
-    def __init__(self, purge_settings, parent=None):
+    _warn_checked_non_data_items = True
+    _ok_button_can_be_disabled = True
+
+    def __init__(self, checked_states, ok_button_text=None, parent=None):
         """
         Args:
-            purge_settings (dict): purge settings
+            checked_states (dict, optional): checked states for each item
+            ok_button_text (str, optional): alternative label for the OK button
             parent (QWidget, optional): parent widget
         """
-        from ..ui.purge_settings_dialog import Ui_Dialog  # pylint: disable=import-outside-toplevel
+        from ..ui.select_database_items_dialog import Ui_Dialog  # pylint: disable=import-outside-toplevel
 
         super().__init__(parent)
         self.setWindowTitle("Database purge settings")
-        self.setWindowFlag(Qt.WindowContextHelpButtonHint, False)
+        self.setWindowFlag(Qt.WindowType.WindowContextHelpButtonHint, False)
         self.setAttribute(Qt.WA_DeleteOnClose)
         self._ui = Ui_Dialog()
         self._ui.setupUi(self)
-        self._item_check_boxes_widget = SelectDatabaseItems(purge_settings, self)
+        if ok_button_text is not None:
+            self._ui.button_box.button(QDialogButtonBox.StandardButton.Ok).setText(ok_button_text)
+        self._item_check_boxes_widget = SelectDatabaseItems(checked_states, self)
         self._ui.root_layout.insertWidget(0, self._item_check_boxes_widget)
+        self._item_check_boxes_widget.checked_state_changed.connect(self._handle_check_box_state_changed)
 
-    def get_purge_settings(self):
-        """Returns current purge settings.
+    def show(self):
+        """Sets the OK button enabled before showing the dialog"""
+        self._handle_check_box_state_changed(False)
+        super().show()
+
+    def get_checked_states(self):
+        """Returns current item checked states.
 
         Returns:
-            dict: mapping from purgeable database item name to purge flag
+            dict: mapping from database item name to checked flag
         """
         return self._item_check_boxes_widget.checked_states()
+
+    @Slot(int)
+    def _handle_check_box_state_changed(self, _checked):
+        if self._ok_button_can_be_disabled:
+            self._ui.button_box.button(QDialogButtonBox.StandardButton.Ok).setEnabled(
+                self._item_check_boxes_widget.any_checked()
+            )
+        if self._warn_checked_non_data_items:
+            if self._item_check_boxes_widget.any_structural_item_checked():
+                self._ui.warning_label.setText("Warning! Structural data items selected.")
+            else:
+                self._ui.warning_label.clear()
+
+
+class PurgeSettingsDialog(SelectDatabaseItemsDialog):
+    _ok_button_can_be_disabled = False
+
+
+class ResizingViewMixin:
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._resizing_timer = QTimer()
+        self._resizing_timer.setSingleShot(True)
+        self._resizing_timer.setInterval(20)
+        self._resizing_timer.timeout.connect(self._do_resize)
+
+    def rowsInserted(self, parent, start, end):
+        super().rowsInserted(parent, start, end)
+        self._resizing_timer.start()
+
+    def _do_resize(self):
+        raise NotImplementedError()
