@@ -15,9 +15,9 @@ from pygments.styles import get_style_by_name
 from pygments.lexers import get_lexer_by_name
 from pygments.util import ClassNotFound
 from pygments.token import Token
-from PySide2.QtCore import Qt, Slot, QTimer, Signal, QRect, QSize
-from PySide2.QtWidgets import QPlainTextEdit, QSizePolicy
-from PySide2.QtGui import (
+from PySide6.QtCore import Qt, Slot, QTimer, Signal, QRect
+from PySide6.QtWidgets import QPlainTextEdit, QSizePolicy
+from PySide6.QtGui import (
     QFontDatabase,
     QTextCharFormat,
     QFont,
@@ -29,8 +29,8 @@ from PySide2.QtGui import (
 )
 from spinetoolbox.helpers import CustomSyntaxHighlighter
 from spinetoolbox.spine_engine_manager import make_engine_manager
-from spinetoolbox.server.engine_client import RemoteEngineInitFailed
 from spinetoolbox.qthread_pool_executor import QtBasedThreadPoolExecutor
+from spine_engine.exception import RemoteEngineInitFailed
 
 
 class _CustomLineEdit(QPlainTextEdit):
@@ -108,14 +108,11 @@ class _CustomLineEdit(QPlainTextEdit):
         if cursor.position() < self.min_pos:
             cursor.setPosition(self.min_pos)
         elif cursor.positionInBlock() < self.new_line_indent:
-            cursor.movePosition(QTextCursor.StartOfBlock)
-            cursor.movePosition(QTextCursor.NextCharacter, n=self.new_line_indent)
+            cursor.movePosition(QTextCursor.MoveOperation.StartOfBlock)
+            cursor.movePosition(QTextCursor.MoveOperation.NextCharacter, n=self.new_line_indent)
         self.setTextCursor(cursor)
         self._updating = False
 
-    def sizeHint(self):
-        return QSize(self._console.width(), 16777215)
-
     def keyPressEvent(self, ev):
         if ev.matches(QKeySequence.Copy):
             ev.ignore()
@@ -125,19 +122,25 @@ class _CustomLineEdit(QPlainTextEdit):
             if cursor.position() == self.min_pos:
                 return
             if cursor.positionInBlock() == self.new_line_indent:
-                cursor.movePosition(QTextCursor.PreviousCharacter, QTextCursor.KeepAnchor, n=self.new_line_indent + 1)
+                cursor.movePosition(
+                    QTextCursor.MoveOperation.PreviousCharacter,
+                    QTextCursor.MoveMode.KeepAnchor,
+                    n=self.new_line_indent + 1,
+                )
                 cursor.removeSelectedText()
                 return
         if ev.key() == Qt.Key_Left:
             cursor = self.textCursor()
             if cursor.positionInBlock() == self.new_line_indent:
-                cursor.movePosition(QTextCursor.PreviousCharacter, n=self.new_line_indent + 1)
+                cursor.movePosition(QTextCursor.MoveOperation.PreviousCharacter, n=self.new_line_indent + 1)
                 self.setTextCursor(cursor)
                 return
         if ev.key() == Qt.Key_Delete:
             cursor = self.textCursor()
             if cursor.atBlockEnd():
-                cursor.movePosition(QTextCursor.NextCharacter, QTextCursor.KeepAnchor, n=self.new_line_indent + 1)
+                cursor.movePosition(
+                    QTextCursor.MoveOperation.NextCharacter, QTextCursor.MoveMode.KeepAnchor, n=self.new_line_indent + 1
+                )
                 cursor.removeSelectedText()
                 return
         if self._console.key_press_event(ev):
@@ -154,6 +157,7 @@ class PersistentConsoleWidget(QPlainTextEdit):
     _history_item_available = Signal(str, str)
     _completions_available = Signal(str, str, list)
     _restarted = Signal()
+    _killed = Signal(bool)
     _flush_needed = Signal()
     _FLUSH_INTERVAL = 200
     _MAX_LINES_PER_SECOND = 2000
@@ -176,6 +180,7 @@ class PersistentConsoleWidget(QPlainTextEdit):
         self.setMaximumBlockCount(self._MAX_LINES_COUNT)
         self._toolbox = toolbox
         self._key = key
+        self._is_dead = False
         self._language = language
         self.owners = {owner}
         self._prompt, self._prompt_format = self._make_prompt()
@@ -228,6 +233,7 @@ class PersistentConsoleWidget(QPlainTextEdit):
         self._history_item_available.connect(self._display_history_item)
         self._completions_available.connect(self._display_completions)
         self._restarted.connect(self._handle_restarted)
+        self._killed.connect(self._do_set_killed)
 
     def closeEvent(self, ev):
         super().closeEvent(ev)
@@ -254,14 +260,14 @@ class PersistentConsoleWidget(QPlainTextEdit):
 
     def mouseMoveEvent(self, ev):
         super().mouseMoveEvent(ev)
-        if self.anchorAt(ev.pos()):
+        if self.anchorAt(ev.position().toPoint()):
             self.viewport().setCursor(Qt.PointingHandCursor)
         else:
             self.viewport().setCursor(Qt.IBeamCursor)
 
     def mousePressEvent(self, ev):
         super().mousePressEvent(ev)
-        self._anchor = self.anchorAt(ev.pos())
+        self._anchor = self.anchorAt(ev.position().toPoint())
 
     def mouseReleaseEvent(self, ev):
         super().mouseReleaseEvent(ev)
@@ -270,7 +276,7 @@ class PersistentConsoleWidget(QPlainTextEdit):
         text_buffer = self._skipped.pop(self._anchor, None)
         if text_buffer is None:
             return
-        cursor = self.cursorForPosition(ev.pos())
+        cursor = self.cursorForPosition(ev.position().toPoint())
         cursor.select(cursor.BlockUnderCursor)
         cursor.removeSelectedText()
         cursor.beginEditBlock()
@@ -305,7 +311,7 @@ class PersistentConsoleWidget(QPlainTextEdit):
             le_selection_start = max(self._line_edit.min_pos, le_selection_start)
             le_selection_end = max(self._line_edit.min_pos, le_selection_end)
             le_cursor.setPosition(le_selection_start)
-            le_cursor.setPosition(le_selection_end, QTextCursor.KeepAnchor)
+            le_cursor.setPosition(le_selection_end, QTextCursor.MoveMode.KeepAnchor)
         self._line_edit.setTextCursor(le_cursor)
 
     @Slot()
@@ -323,26 +329,39 @@ class PersistentConsoleWidget(QPlainTextEdit):
     def _handle_update_request(self, _rect, _dy):
         """Move line edit to input start pos."""
         if not self._updating:
-            self._move_line_edit()
+            self._move_and_resize_line_edit()
 
-    def _move_line_edit(self):
+    def resizeEvent(self, ev):
+        super().resizeEvent(ev)
+        if not self._updating:
+            self._move_and_resize_line_edit()
+
+    def _move_and_resize_line_edit(self):
         if self._prompt_block is None:
             return
         cursor = self.textCursor()
         cursor.setPosition(self._prompt_block.position())
         rect = self.cursorRect(cursor)
         self._line_edit.move(rect.topLeft())
+        width = self.geometry().width()
+        scrollbar = self.verticalScrollBar()
+        if scrollbar.isVisible():
+            width -= scrollbar.width()
+        cursor_width = self._line_edit.cursorWidth()
+        width = (width // cursor_width) * cursor_width
+        self._line_edit.setFixedWidth(width)
 
     @Slot()
     def _update_user_input(self):
         self._updating = True
         cursor = self.textCursor()
         cursor.setPosition(self._input_start_pos)
-        cursor.movePosition(QTextCursor.End, QTextCursor.KeepAnchor)
+        cursor.movePosition(QTextCursor.MoveOperation.End, QTextCursor.MoveMode.KeepAnchor)
         text = self._line_edit.raw_text()
         cursor.insertText(text)
         self._highlight_current_input()
         self._updating = False
+        self._move_and_resize_line_edit()
 
     @Slot()
     def _start_flush_timer(self):
@@ -388,7 +407,7 @@ class PersistentConsoleWidget(QPlainTextEdit):
 
     def _make_prompt_block(self, prompt=""):
         cursor = self.textCursor()
-        cursor.movePosition(QTextCursor.End)
+        cursor.movePosition(QTextCursor.MoveOperation.End)
         cursor.insertBlock()
         self._prompt_block = cursor.block()
         self._insert_prompt(prompt=prompt)
@@ -397,10 +416,10 @@ class PersistentConsoleWidget(QPlainTextEdit):
         cursor = self.textCursor()
         cursor.setPosition(self._prompt_block.position())
         cursor.insertText(prompt, self._prompt_format)
-        cursor.movePosition(QTextCursor.End)
+        cursor.movePosition(QTextCursor.MoveOperation.End)
         self._current_prompt = prompt
         self._line_edit.reset(self._current_prompt)
-        self._move_line_edit()
+        self._move_and_resize_line_edit()
 
     def _insert_stdin_text(self, cursor, text):
         """Inserts highlighted text.
@@ -453,6 +472,26 @@ class PersistentConsoleWidget(QPlainTextEdit):
         else:
             self._insert_stdout_text(cursor, text)
 
+    def set_killed(self, killed):
+        """Emits the ``killed`` signal.
+
+        Args:
+            killed (bool): if True, may the console rest in peace
+        """
+        self._killed.emit(killed)
+
+    @Slot(bool)
+    def _do_set_killed(self, killed):
+        """Sets the console as killed or alive.
+
+        Args:
+            killed (bool): if True, may the console rest in peace
+        """
+        self._is_dead = killed
+        self._line_edit.setVisible(not killed)
+        if killed:
+            self._make_prompt_block("Console killed (can be restarted from the right-click context menu)")
+
     def add_stdin(self, data):
         """Adds new prompt with data. Used when adding stdin from external execution.
 
@@ -483,20 +522,20 @@ class PersistentConsoleWidget(QPlainTextEdit):
 
     def _get_prefix(self):
         le_cursor = self._line_edit.textCursor()
-        le_cursor.setPosition(self._line_edit.min_pos, QTextCursor.KeepAnchor)
+        le_cursor.setPosition(self._line_edit.min_pos, QTextCursor.MoveMode.KeepAnchor)
         return le_cursor.selectedText().rstrip()
 
     def _highlight_current_input(self):
         cursor = self.textCursor()
         cursor.setPosition(self._input_start_pos)
-        cursor.movePosition(QTextCursor.End, QTextCursor.KeepAnchor)
+        cursor.movePosition(QTextCursor.MoveOperation.End, QTextCursor.MoveMode.KeepAnchor)
         text = cursor.selectedText()
         for start, count, text_format in self._highlighter.yield_formats(text):
             start += self._input_start_pos
             cursor.setPosition(start)
-            cursor.setPosition(start + count, QTextCursor.KeepAnchor)
+            cursor.setPosition(start + count, QTextCursor.MoveMode.KeepAnchor)
             cursor.setCharFormat(text_format)
-        cursor.movePosition(QTextCursor.NextCharacter)
+        cursor.movePosition(QTextCursor.MoveOperation.NextCharacter)
         cursor.setCharFormat(QTextCharFormat())
 
     def key_press_event(self, ev):
@@ -679,24 +718,44 @@ class PersistentConsoleWidget(QPlainTextEdit):
 
     @Slot()
     def _handle_restarted(self):
+        self._do_set_killed(False)
         self._make_prompt_block(prompt=self._prompt)
 
     @Slot(bool)
     def _interrupt_persistent(self, _=False):
-        """Interrupts underlying persistent process."""
+        """Sends a task to executor which will interrupt the underlying persistent process."""
         self._executor.submit(self._do_interrupt_persistent)
 
     def _do_interrupt_persistent(self):
+        """Interrupts the underlying persistent process."""
         engine_mngr = self.create_engine_manager()
         if not engine_mngr:
             return
         engine_mngr.interrupt_persistent(self._key)
 
+    @Slot(bool)
+    def _kill_persistent(self, _=False):
+        """Sends a task to executor which will kill the underlying persistent process."""
+        self._do_set_killed(True)
+        self._executor.submit(self._do_kill_persistent)
+
+    def _do_kill_persistent(self):
+        """Kills underlying persistent process."""
+        engine_mngr = self.create_engine_manager()
+        if not engine_mngr:
+            return
+        engine_mngr.kill_persistent(self._key)
+
     def _extend_menu(self, menu):
-        """Adds two more actions: Restart, and Interrupt."""
+        """Appends two more actions: Restart, and Interrupt.
+
+        Args:
+            menu (QMenu): where to append
+        """
         menu.addSeparator()
         menu.addAction("Restart", self._restart_persistent)
-        menu.addAction("Interrupt", self._interrupt_persistent)
+        menu.addAction("Interrupt", self._interrupt_persistent).setEnabled(not self._is_dead)
+        menu.addAction("Kill", self._kill_persistent).setEnabled(not self._is_dead)
 
     def contextMenuEvent(self, ev):
         """Reimplemented to extend menu with custom actions."""
@@ -707,7 +766,7 @@ class PersistentConsoleWidget(QPlainTextEdit):
             else self.createStandardContextMenu()
         )
         self._extend_menu(menu)
-        menu.exec_(ev.globalPos())
+        menu.exec(ev.globalPos())
 
 
 # Translated from
