@@ -19,12 +19,12 @@ A Qt widget showing a toolbar and a matplotlib plotting canvas.
 import itertools
 import io
 import csv
-import numpy as np
+
+import numpy
 from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT as NavigationToolBar
-from PySide2.QtCore import QMetaObject, Qt
-from PySide2.QtWidgets import QVBoxLayout, QWidget, QMenu, QApplication
-from spinedb_api import IndexedValue, Map, TimeSeries
-from .plot_canvas import PlotCanvas
+from PySide6.QtCore import QMetaObject, Qt
+from PySide6.QtWidgets import QVBoxLayout, QWidget, QMenu, QApplication
+from .plot_canvas import PlotCanvas, LegendPosition
 from .custom_qtableview import CopyPasteTableView
 from ..mvcmodels.minimal_table_model import MinimalTableModel
 from ..helpers import busy_effect
@@ -42,14 +42,15 @@ class PlotWidget(QWidget):
     plot_windows = dict()
     """A global list of plot windows."""
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, legend_axes_position=LegendPosition.BOTTOM):
         """
         Args:
-            parent (QWidget): parent widget
+            parent (QWidget, optional): parent widget
+            legend_axes_position (LegendPosition): legend axes position relative to plot axes
         """
         super().__init__(parent)
         self._layout = QVBoxLayout(self)
-        self.canvas = PlotCanvas(self)
+        self.canvas = PlotCanvas(self, legend_axes_position)
         self._toolbar = NavigationToolBar(self.canvas, self)
         self._layout.addWidget(self._toolbar)
         self._layout.addWidget(self.canvas)
@@ -68,7 +69,7 @@ class PlotWidget(QWidget):
         menu = QMenu(self)
         menu.addAction("Show plot data...", self.show_plot_data)
         menu.addAction("Copy plot data", self.copy_plot_data)
-        menu.exec_(event.globalPos())
+        menu.exec(event.globalPos())
 
     def _get_plot_data(self):
         """Gathers plot data into a table.
@@ -76,24 +77,24 @@ class PlotWidget(QWidget):
         Returns:
             list of list: data as table
         """
-        if not self.original_xy_data:
-            return []
-        max_index_count = max(len(xy_data.data_index) for xy_data in self.original_xy_data)
-        header_of_agreeable_width = None
-        rows = []
+        header = ["indexes"]
+        indexes = []
+        data_dicts = []
         for xy_data in self.original_xy_data:
-            if len(xy_data.data_index) == max_index_count:
-                if header_of_agreeable_width is None:
-                    header_of_agreeable_width = xy_data.index_names + [xy_data.x_label, xy_data.y_label]
-                rows += [xy_data.data_index + [x, y] for x, y in zip(xy_data.x, xy_data.y)]
-            else:
-                n = max_index_count - len(xy_data.data_index)
-                rows += [xy_data.data_index + [x, y] + n * [None] for x, y in zip(xy_data.x, xy_data.y)]
-        rows.insert(0, header_of_agreeable_width)
+            label = " | ".join(xy_data.data_index)
+            header.append(label)
+            indexes.append(xy_data.x)
+            data_dict = dict(zip(xy_data.x, xy_data.y))
+            data_dicts.append(data_dict)
+        all_indexes = numpy.unique(numpy.concatenate(indexes))
+        rows = [header]
+        for index in all_indexes:
+            row = [str(index)] + [str(data_dict.get(index, "")) for data_dict in data_dicts]
+            rows.append(row)
         return rows
 
     @busy_effect
-    def copy_plot_data(self, parent=None, document_name=""):
+    def copy_plot_data(self):
         """Copies plot data to clipboard."""
         rows = self._get_plot_data()
         with io.StringIO() as output:
@@ -103,23 +104,22 @@ class PlotWidget(QWidget):
             QApplication.clipboard().setText(output.getvalue())
 
     @busy_effect
-    def show_plot_data(self, parent=None, document_name=""):
+    def show_plot_data(self):
         """Opens a separate window that shows the plot data."""
-        if parent is None:
-            parent = self
         rows = self._get_plot_data()
         widget = _PlotDataWidget(rows, self)
-        widget.setWindowFlag(Qt.Window, True)
+        widget.setWindowFlag(Qt.WindowType.Window, True)
         title = "Plot data"
-        if document_name:
-            title += f"\t-- {document_name} --"
         widget.setWindowTitle(title)
         widget.show()
 
-    def add_legend(self):
-        """Adds a legend to the plot's legend axes."""
-        h, l = self.canvas.axes.get_legend_handles_labels()
-        self.canvas.legend_axes.legend(h, l, loc="upper center")
+    def add_legend(self, handles):
+        """Adds a legend to the plot's legend axes.
+
+        Args:
+            handles (list): legend handles
+        """
+        self.canvas.legend_axes.legend(handles=handles, loc="upper center")
 
     def use_as_window(self, parent_window, document_name):
         """
@@ -130,7 +130,7 @@ class PlotWidget(QWidget):
             document_name (str): a string to add to the window title
         """
         self.setParent(parent_window)
-        self.setWindowFlag(Qt.Window, True)
+        self.setWindowFlag(Qt.WindowType.Window, True)
         self.setAttribute(Qt.WA_DeleteOnClose, True)
         title = "Plot"
         if document_name:
@@ -154,7 +154,7 @@ class _PlotDataView(CopyPasteTableView):
         menu = QMenu(self)
         menu.addAction("Select all", self.selectAll)
         menu.addAction("Copy", self.copy).setEnabled(self.can_copy())
-        menu.exec_(event.globalPos())
+        menu.exec(event.globalPos())
 
 
 class _PlotDataWidget(QWidget):
