@@ -16,9 +16,9 @@ Contains PropertiesWidgetBase.
 :date: 20.01.2022
 """
 
-from PySide2.QtWidgets import QWidget, QAbstractItemView, QLineEdit, QHeaderView
-from PySide2.QtCore import Qt, QRect, QPoint, QEvent
-from PySide2.QtGui import QPainter, QPixmap, QColor
+from PySide6.QtWidgets import QWidget, QAbstractItemView, QLineEdit, QHeaderView
+from PySide6.QtCore import Qt, QRect, QPoint, QEvent
+from PySide6.QtGui import QPainter, QPixmap, QColor
 from ..helpers import fix_lightness_color
 
 
@@ -27,6 +27,7 @@ class PropertiesWidgetBase(QWidget):
 
     def __init__(self, toolbox, base_color=None):
         super().__init__()
+        self._active_item = None
         self._toolbox = toolbox
         self._pixmap = None
         self._bg_color = None
@@ -37,6 +38,18 @@ class PropertiesWidgetBase(QWidget):
         if base_color is not None:
             self.set_color_and_icon(base_color)
 
+    def set_item(self, project_item):
+        """Sets the active project item.
+
+        Args:
+            project_item (ProjectItem): active project item
+        """
+        self._active_item = project_item
+
+    def unset_item(self):
+        """Unsets the active project item."""
+        self._active_item = None
+
     @property
     def fg_color(self):
         return self._fg_color
