@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -21,9 +21,10 @@ from pathlib import Path
 from tempfile import TemporaryDirectory
 import unittest
 from unittest import mock
-from PySide2.QtCore import QVariantAnimation
-from PySide2.QtWidgets import QApplication
+from PySide6.QtCore import QVariantAnimation
+from PySide6.QtWidgets import QApplication
 
+from spine_engine.project_item.project_item_specification import ProjectItemSpecification
 from spine_engine.spine_engine import ItemExecutionFinishState
 from spine_engine.project_item.executable_item_base import ExecutableItemBase
 from spine_engine.utils.helpers import shorten
@@ -41,10 +42,9 @@ from tests.mock_helpers import (
     add_tool,
     add_view,
     add_importer,
-    add_gdx_exporter,
-    add_gimlet,
     add_exporter,
     add_data_transformer,
+    qsettings_value_side_effect,
 )
 
 
@@ -130,27 +130,23 @@ class TestSpineToolboxProject(unittest.TestCase):
 
     def test_add_all_available_items(self):
         """Test that adding multiple items works as expected.
-        Multiple items are added in order DS, DC, DT, Tool, Gimlet, View, Importer, Exporter, GdxExporter."""
+        Multiple items are added in order DS, DC, DT, Tool, View, Importer, Exporter."""
         p = self.toolbox.project()
         # Add items
         ds_name = "DS"
         dc_name = "DC"
         dt_name = "DT"
         tool_name = "Tool"
-        gimlet_name = "Gimlet"
         view_name = "View"
         imp_name = "Importer"
         exporter_name = "Exporter"
-        gdx_exporter_name = "GdxExporter"
         add_ds(p, self.toolbox.item_factories, ds_name)
         add_dc(p, self.toolbox.item_factories, dc_name)
         add_data_transformer(p, self.toolbox.item_factories, dt_name)
         add_tool(p, self.toolbox.item_factories, tool_name)
-        add_gimlet(p, self.toolbox.item_factories, gimlet_name)
         add_view(p, self.toolbox.item_factories, view_name)
         add_importer(p, self.toolbox.item_factories, imp_name)
         add_exporter(p, self.toolbox.item_factories, exporter_name)
-        add_gdx_exporter(p, self.toolbox.item_factories, gdx_exporter_name)
         # Check that the items are found from project item model
         ds = p.get_item(ds_name)
         self.assertEqual(ds_name, ds.name)
@@ -160,19 +156,15 @@ class TestSpineToolboxProject(unittest.TestCase):
         self.assertEqual(dt_name, dt.name)
         tool = p.get_item(tool_name)
         self.assertEqual(tool_name, tool.name)
-        gimlet = p.get_item(gimlet_name)
-        self.assertEqual(gimlet_name, gimlet.name)
         view = p.get_item(view_name)
         self.assertEqual(view_name, view.name)
         importer = p.get_item(imp_name)
         self.assertEqual(imp_name, importer.name)
         exporter = p.get_item(exporter_name)
         self.assertEqual(exporter_name, exporter.name)
-        gdx_exporter = p.get_item(gdx_exporter_name)
-        self.assertEqual(gdx_exporter_name, gdx_exporter.name)
-        # DAG handler should now have six graphs, each with one item
+        # DAG handler should now have seven graphs, each with one item
         n_dags = len(self.toolbox.project().dags())
-        self.assertEqual(9, n_dags)
+        self.assertEqual(7, n_dags)
         # Check that all created items are in graphs
         ds_graph = self.toolbox.project().dag_with_node(ds_name)
         self.assertIsNotNone(ds_graph)
@@ -182,16 +174,12 @@ class TestSpineToolboxProject(unittest.TestCase):
         self.assertIsNotNone(dt_graph)
         tool_graph = self.toolbox.project().dag_with_node(tool_name)
         self.assertIsNotNone(tool_graph)
-        gimlet_graph = self.toolbox.project().dag_with_node(gimlet_name)
-        self.assertIsNotNone(gimlet_graph)
         view_graph = self.toolbox.project().dag_with_node(view_name)
         self.assertIsNotNone(view_graph)
         importer_graph = self.toolbox.project().dag_with_node(imp_name)
         self.assertIsNotNone(importer_graph)
         exporter_graph = self.toolbox.project().dag_with_node(exporter_name)
         self.assertIsNotNone(exporter_graph)
-        gdx_exporter_graph = self.toolbox.project().dag_with_node(gdx_exporter_name)
-        self.assertIsNotNone(gdx_exporter_graph)
 
     def test_remove_item_by_name(self):
         view_name = "View"
@@ -242,21 +230,38 @@ class TestSpineToolboxProject(unittest.TestCase):
     def _execute_project(self):
         waiter = SignalWaiter()
         self.toolbox.project().project_execution_finished.connect(waiter.trigger)
-        self.toolbox.project().execute_project()
+        with mock.patch("spinetoolbox.ui_main.QSettings.value") as mock_qsettings_value, mock.patch(
+            "spinetoolbox.project.make_settings_dict_for_engine"
+        ) as mock_settings_dict:
+            # Make sure that the test uses LocalSpineEngineManager
+            # This mocks the check for engineSettings/remoteEngineEnabled in SpineToolboxProject.execute_dags()
+            mock_qsettings_value.side_effect = qsettings_value_side_effect
+            # This mocks the call to make_settings_dict_for_engine in SpineToolboxProject._execute_dags()
+            mock_settings_dict.return_value = dict()
+            self.toolbox.project().execute_project()
+            mock_qsettings_value.assert_called()
+            mock_settings_dict.assert_called_once()
         waiter.wait()
         self.toolbox.project().project_execution_finished.disconnect(waiter.trigger)
 
     def _execute_selected(self, names):
         waiter = SignalWaiter()
         self.toolbox.project().project_execution_finished.connect(waiter.trigger)
-        self.toolbox.project().execute_selected(names)
+        with mock.patch("spinetoolbox.ui_main.QSettings.value") as mock_qsettings_value, mock.patch(
+            "spinetoolbox.project.make_settings_dict_for_engine"
+        ) as mock_settings_dict:
+            # Make sure that the test uses LocalSpineEngineManager
+            # This mocks the check for engineSettings/remoteEngineEnabled in SpineToolboxProject.execute_dags()
+            mock_settings_dict.return_value = dict()
+            self.toolbox.project().execute_selected(names)
+            mock_settings_dict.assert_called()
         waiter.wait()
         self.toolbox.project().project_execution_finished.disconnect(waiter.trigger)
 
     def test_execute_project_with_single_item(self):
         view = add_view(self.toolbox.project(), self.toolbox.item_factories, "View")
         view_executable = self._make_mock_executable(view)
-        with mock.patch("spine_engine.spine_engine.SpineEngine._make_item") as mock_make_item:
+        with mock.patch("spine_engine.spine_engine.SpineEngine.make_item") as mock_make_item:
             mock_make_item.return_value = view_executable
             self._execute_project()
         self.assertTrue(view_executable.execute_called)
@@ -266,8 +271,8 @@ class TestSpineToolboxProject(unittest.TestCase):
         item1_executable = self._make_mock_executable(item1)
         item2 = add_view(self.toolbox.project(), self.toolbox.item_factories, "View")
         item2_executable = self._make_mock_executable(item2)
-        with mock.patch("spine_engine.spine_engine.SpineEngine._make_item") as mock_make_item:
-            mock_make_item.side_effect = lambda name, *args: {
+        with mock.patch("spine_engine.spine_engine.SpineEngine.make_item") as mock_make_item:
+            mock_make_item.side_effect = lambda name, *args, **kwargs: {
                 item1.name: item1_executable,
                 item2.name: item2_executable,
             }[name]
@@ -280,8 +285,8 @@ class TestSpineToolboxProject(unittest.TestCase):
         item1_executable = self._make_mock_executable(item1)
         item2 = add_view(self.toolbox.project(), self.toolbox.item_factories, "View")
         item2_executable = self._make_mock_executable(item2)
-        with mock.patch("spine_engine.spine_engine.SpineEngine._make_item") as mock_make_item:
-            mock_make_item.side_effect = lambda name, *args: {
+        with mock.patch("spine_engine.spine_engine.SpineEngine.make_item") as mock_make_item:
+            mock_make_item.side_effect = lambda name, *args, **kwargs: {
                 item1.name: item1_executable,
                 item2.name: item2_executable,
             }[name]
@@ -289,6 +294,18 @@ class TestSpineToolboxProject(unittest.TestCase):
         self.assertFalse(item1_executable.execute_called)
         self.assertTrue(item2_executable.execute_called)
 
+    def test_executing_cyclic_dag_fails_graciously(self):
+        item1 = add_dc(self.toolbox.project(), self.toolbox.item_factories, "DC")
+        item2 = add_view(self.toolbox.project(), self.toolbox.item_factories, "View")
+        self.toolbox.project().add_connection(
+            LoggingConnection(item1.name, "right", item2.name, "left", toolbox=self.toolbox)
+        )
+        self.toolbox.project().add_connection(
+            LoggingConnection(item2.name, "bottom", item1.name, "top", toolbox=self.toolbox)
+        )
+        self.toolbox.project().execute_project()
+        self.assertFalse(self.toolbox.project()._execution_in_progress)
+
     def test_change_name(self):
         """Tests renaming a project."""
         new_name = "New Project Name"
@@ -317,8 +334,8 @@ class TestSpineToolboxProject(unittest.TestCase):
         self.toolbox.project().add_connection(
             LoggingConnection(data_connection.name, "bottom", view.name, "top", toolbox=self.toolbox)
         )
-        with mock.patch("spine_engine.spine_engine.SpineEngine._make_item") as mock_make_item:
-            mock_make_item.side_effect = lambda name, *args: {
+        with mock.patch("spine_engine.spine_engine.SpineEngine.make_item") as mock_make_item:
+            mock_make_item.side_effect = lambda name, *args, **kwargs: {
                 data_store.name: data_store_executable,
                 data_connection.name: data_connection_executable,
                 view.name: view_executable,
@@ -482,14 +499,7 @@ class TestSpineToolboxProject(unittest.TestCase):
             project_dict,
             {
                 "items": {"test item": {"type": "Tester", "a": {"c": 2}}},
-                "project": {
-                    "connections": [],
-                    "description": "Project for unit tests.",
-                    "jumps": [],
-                    "name": "UnitTest Project",
-                    "specifications": {},
-                    "version": 7,
-                },
+                "project": {"connections": [], "description": "", "jumps": [], "specifications": {}, "version": 10},
             },
         )
         with Path(project.config_dir, PROJECT_LOCAL_DATA_DIR_NAME, PROJECT_LOCAL_DATA_FILENAME).open() as fp:
@@ -513,6 +523,100 @@ class TestSpineToolboxProject(unittest.TestCase):
         item = self.toolbox.project().get_item("test item")
         self.assertEqual(item.kwargs, {"type": "Tester", "a": {"b": 1, "c": 2, "d": 3}})
 
+    def test_add_and_save_specification(self):
+        project = self.toolbox.project()
+        specification = _MockSpecification(
+            "a specification", "Specification for testing.", "Tester", "Testing category"
+        )
+        project.add_specification(specification)
+        self.assertTrue(specification.is_equivalent(project.get_specification("a specification")))
+        specification_dir = Path(self._temp_dir.name) / ".spinetoolbox" / "specifications" / "Tester"
+        self.assertTrue(specification_dir.exists())
+        specification_file = specification_dir / (specification.short_name + ".json")
+        self.assertEqual(specification_file, Path(specification.definition_file_path))
+        self.assertTrue(specification_file.exists())
+        with open(specification_file) as specification_input:
+            specification_dict = json.load(specification_input)
+        self.assertEqual(specification_dict, specification.to_dict())
+        local_data_dir = Path(self._temp_dir.name) / ".spinetoolbox" / "local"
+        self.assertFalse(local_data_dir.exists())
+
+    def test_add_and_save_specification_with_local_data(self):
+        project = self.toolbox.project()
+        specification = _MockSpecificationWithLocalData(
+            "a specification", "Specification for testing.", "Tester", "Testing category", "my precious data"
+        )
+        project.add_specification(specification)
+        self.assertTrue(specification.is_equivalent(project.get_specification("a specification")))
+        specification_dir = Path(self._temp_dir.name) / ".spinetoolbox" / "specifications" / "Tester"
+        self.assertTrue(specification_dir.exists())
+        specification_file = specification_dir / (specification.short_name + ".json")
+        self.assertEqual(specification_file, Path(specification.definition_file_path))
+        self.assertTrue(specification_file.exists())
+        with open(specification_file) as specification_input:
+            specification_dict = json.load(specification_input)
+        expected = specification.to_dict()
+        expected.pop("data")
+        self.assertEqual(specification_dict, expected)
+        local_data_dir = Path(self._temp_dir.name) / ".spinetoolbox" / "local"
+        self.assertTrue(local_data_dir.exists())
+        local_data_file = local_data_dir / "specification_local_data.json"
+        self.assertTrue(local_data_file.exists())
+        with open(local_data_file) as data_input:
+            local_data = json.load(data_input)
+        self.assertEqual(local_data, {"Tester": {"a specification": {"data": "my precious data"}}})
+
+    def test_renaming_specification_with_local_data_updates_local_data_file(self):
+        project = self.toolbox.project()
+        original_specification = _MockSpecificationWithLocalData(
+            "a specification", "Specification for testing.", "Tester", "Testing category", "my precious data"
+        )
+        project.add_specification(original_specification)
+        local_data_file = Path(self._temp_dir.name) / ".spinetoolbox" / "local" / "specification_local_data.json"
+        self.assertTrue(local_data_file.exists())
+        specification = _MockSpecificationWithLocalData(
+            "another specification", "Specification for testing.", "Tester", "Testing category", "my precious data"
+        )
+        project.replace_specification("a specification", specification)
+        specification_dir = Path(self._temp_dir.name) / ".spinetoolbox" / "specifications" / "Tester"
+        self.assertTrue(specification_dir.exists())
+        specification_file = specification_dir / (specification.short_name + ".json")
+        self.assertEqual(specification_file, Path(specification.definition_file_path))
+        self.assertTrue(specification_file.exists())
+        self.assertTrue(Path(original_specification.definition_file_path).exists())
+        with open(specification_file) as specification_input:
+            specification_dict = json.load(specification_input)
+        expected = specification.to_dict()
+        expected.pop("data")
+        self.assertEqual(specification_dict, expected)
+        with open(local_data_file) as data_input:
+            local_data = json.load(data_input)
+        self.assertEqual(local_data, {"Tester": {"another specification": {"data": "my precious data"}}})
+
+    def test_replace_specification_with_local_data_by_one_without_removes_local_data_from_the_file(self):
+        project = self.toolbox.project()
+        specification_with_local_data = _MockSpecificationWithLocalData(
+            "a specification", "Specification for testing.", "Tester", "Testing category", "my precious data"
+        )
+        project.add_specification(specification_with_local_data)
+        local_data_file = Path(self._temp_dir.name) / ".spinetoolbox" / "local" / "specification_local_data.json"
+        self.assertTrue(local_data_file.exists())
+        specification = _MockSpecification(
+            "another specification", "Specification without local data", "Tester", "Testing category"
+        )
+        project.replace_specification("a specification", specification)
+        specification_dir = Path(self._temp_dir.name) / ".spinetoolbox" / "specifications" / "Tester"
+        self.assertTrue(specification_dir.exists())
+        specification_file = specification_dir / (specification.short_name + ".json")
+        self.assertEqual(specification_file, Path(specification.definition_file_path))
+        self.assertTrue(specification_file.exists())
+        with open(specification_file) as specification_input:
+            specification_dict = json.load(specification_input)
+        self.assertEqual(specification_dict, specification.to_dict())
+        with open(local_data_file) as data_input:
+            local_data = json.load(data_input)
+        self.assertEqual(local_data, {})
+
     def _make_mock_executable(self, item):
         item_name = item.name
         item = self.toolbox.project_item_model.get_item(item_name).project_item
@@ -536,7 +640,10 @@ class _MockExecutableItem(ExecutableItemBase):
     def item_type():
         return "Mock item"
 
-    def execute(self, _forward_resources, _backward_resources):
+    def ready_to_execute(self, _settings):
+        return True
+
+    def execute(self, _forward_resources, _backward_resources, lock):
         self.execute_called = True
         return ItemExecutionFinishState.SUCCESS
 
@@ -578,5 +685,38 @@ class _MockItemFactoryForLocalDataTests(ProjectItemFactory):
         return _MockItemWithLocalData(project, **item_dict)
 
 
+class _MockSpecification(ProjectItemSpecification):
+    def to_dict(self):
+        return {"name": self.name, "description": self.description}
+
+    def is_equivalent(self, other):
+        if not isinstance(other, type(self)):
+            return False
+        return self.name == other.name and self.item_type == other.item_type and self.description == other.description
+
+
+class _MockSpecificationWithLocalData(ProjectItemSpecification):
+    def __init__(self, name, description, item_type, item_category, local_data):
+        super().__init__(name, description, item_type, item_category)
+        self._local_data = local_data
+
+    def to_dict(self):
+        return {"name": self.name, "description": self.description, "data": self._local_data}
+
+    def is_equivalent(self, other):
+        if not isinstance(other, type(self)):
+            return False
+        return (
+            self.name == other.name
+            and self.item_type == other.item_type
+            and self.description == other.description
+            and self._local_data == other._local_data
+        )
+
+    @staticmethod
+    def _definition_local_entries():
+        return [("data",)]
+
+
 if __name__ == '__main__':
     unittest.main()
