@@ -1,5 +1,5 @@
 ######################################################################################################################
-# Copyright (C) 2017-2021 Spine project consortium
+# Copyright (C) 2017-2022 Spine project consortium
 # This file is part of Spine Toolbox.
 # Spine Toolbox is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General
 # Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option)
@@ -17,6 +17,7 @@ Unit tests for ToolboxUI class.
 """
 
 from collections import namedtuple
+from contextlib import contextmanager
 from tempfile import TemporaryDirectory
 import unittest
 from unittest import mock
@@ -25,10 +26,10 @@ import os
 import sys
 
 import spinetoolbox.ui_main
-from PySide2.QtWidgets import QApplication, QMessageBox
-from PySide2.QtCore import Qt, QPoint, QItemSelectionModel, QPointF, QMimeData
-from PySide2.QtTest import QTest
-from PySide2.QtGui import QDropEvent
+from PySide6.QtWidgets import QApplication, QMessageBox
+from PySide6.QtCore import QSettings, Qt, QPoint, QItemSelectionModel, QPointF, QMimeData
+from PySide6.QtTest import QTest
+from PySide6.QtGui import QDropEvent
 from spinetoolbox.project_item_icon import ProjectItemIcon
 from spinetoolbox.project import SpineToolboxProject
 from spinetoolbox.widgets.project_item_drag import ProjectItemDragMixin
@@ -36,7 +37,15 @@ from spinetoolbox.widgets.persistent_console_widget import PersistentConsoleWidg
 from spinetoolbox.link import Link
 from spinetoolbox.mvcmodels.project_tree_item import RootProjectTreeItem
 from spinetoolbox.resources_icons_rc import qInitResources
-from .mock_helpers import clean_up_toolbox, create_toolboxui, create_project, add_ds, add_dc, add_tool
+from .mock_helpers import (
+    clean_up_toolbox,
+    create_toolboxui,
+    create_project,
+    add_ds,
+    add_dc,
+    add_tool,
+    qsettings_value_side_effect,
+)
 
 
 # noinspection PyUnusedLocal,DuplicatedCode
@@ -230,13 +239,13 @@ class TestToolboxUI(unittest.TestCase):
         ):
             self.toolbox.init_project(project_dir)
         self.assertIsNotNone(self.toolbox.project())
-        self.assertEqual(self.toolbox.project().name, "UnitTest Project")
+        self.assertEqual(self.toolbox.project().name, "Project Directory")
 
     def test_new_project(self):
         self._temp_dir = TemporaryDirectory()
-        with mock.patch("spinetoolbox.ui_main.ToolboxUI.update_recent_projects"), mock.patch(
-            "spinetoolbox.widgets.open_project_widget.OpenProjectDialog.remove_directory_from_recents"
-        ), mock.patch("PySide2.QtWidgets.QFileDialog.getExistingDirectory") as mock_dir_getter:
+        with mock.patch("spinetoolbox.ui_main.QSettings.setValue"), mock.patch(
+            "spinetoolbox.ui_main.QSettings.sync"
+        ), mock.patch("PySide6.QtWidgets.QFileDialog.getExistingDirectory") as mock_dir_getter:
             mock_dir_getter.return_value = self._temp_dir.name
             self.toolbox.new_project()
         self.assertIsNotNone(self.toolbox.project())
@@ -244,15 +253,19 @@ class TestToolboxUI(unittest.TestCase):
 
     def test_save_project(self):
         self._temp_dir = TemporaryDirectory()
-        with mock.patch("spinetoolbox.ui_main.ToolboxUI.update_recent_projects"), mock.patch(
-            "spinetoolbox.widgets.open_project_widget.OpenProjectDialog.remove_directory_from_recents"
-        ), mock.patch("PySide2.QtWidgets.QFileDialog.getExistingDirectory") as mock_dir_getter:
+        with mock.patch("spinetoolbox.ui_main.QSettings.setValue"), mock.patch(
+            "spinetoolbox.ui_main.QSettings.sync"
+        ), mock.patch("PySide6.QtWidgets.QFileDialog.getExistingDirectory") as mock_dir_getter:
             mock_dir_getter.return_value = self._temp_dir.name
             self.toolbox.new_project()
         add_dc(self.toolbox.project(), self.toolbox.item_factories, "DC")
         self.toolbox.save_project()
         self.assertTrue(self.toolbox.undo_stack.isClean())
-        self.assertTrue(self.toolbox.close_project())
+        with mock.patch("spinetoolbox.ui_main.QSettings.value") as mock_qsettings_value:
+            # Make sure that the test uses LocalSpineEngineManager
+            mock_qsettings_value.side_effect = qsettings_value_side_effect
+            self.assertTrue(self.toolbox.close_project())
+            mock_qsettings_value.assert_called()
         with mock.patch("spinetoolbox.ui_main.ToolboxUI.save_project"), mock.patch(
             "spinetoolbox.project.create_dir"
         ), mock.patch("spinetoolbox.project_item.project_item.create_dir"), mock.patch(
@@ -269,7 +282,11 @@ class TestToolboxUI(unittest.TestCase):
         with TemporaryDirectory() as project_dir:
             create_project(self.toolbox, project_dir)
             self.assertIsInstance(self.toolbox.project(), SpineToolboxProject)
-            self.assertTrue(self.toolbox.close_project())
+            with mock.patch("spinetoolbox.ui_main.QSettings.value") as mock_qsettings_value:
+                # Make sure that the test uses LocalSpineEngineManager
+                mock_qsettings_value.side_effect = qsettings_value_side_effect
+                self.assertTrue(self.toolbox.close_project())
+                mock_qsettings_value.assert_called()
         self.assertIsNone(self.toolbox.project())
 
     def test_selection_in_project_item_list_1(self):
@@ -576,7 +593,7 @@ class TestToolboxUI(unittest.TestCase):
         pos = QPoint(0, 0)
         event = QDropEvent(pos, Qt.CopyAction, mime_data, Qt.NoButton, Qt.NoModifier)
         with mock.patch(
-            'PySide2.QtWidgets.QGraphicsSceneDragDropEvent.source'
+            'PySide6.QtWidgets.QGraphicsSceneDragDropEvent.source'
         ) as mock_drop_event_source, mock.patch.object(self.toolbox, "project"), mock.patch.object(
             self.toolbox, "show_add_project_item_form"
         ) as mock_show_add_project_item_form:
@@ -595,7 +612,7 @@ class TestToolboxUI(unittest.TestCase):
         pos = gv.mapFromScene(scene_pos)
         event = QDropEvent(pos, Qt.CopyAction, mime_data, Qt.NoButton, Qt.NoModifier)
         with mock.patch(
-            'PySide2.QtWidgets.QGraphicsSceneDragDropEvent.source'
+            'PySide6.QtWidgets.QGraphicsSceneDragDropEvent.source'
         ) as mock_drop_event_source, mock.patch.object(self.toolbox, "project"), mock.patch.object(
             self.toolbox, "show_add_project_item_form"
         ) as mock_show_add_project_item_form:
@@ -627,8 +644,8 @@ class TestToolboxUI(unittest.TestCase):
         # NOW REMOVE DC1
         dc1_ind = self.toolbox.project_item_model.find_item(dc1)
         self.toolbox.ui.treeView_project.selectionModel().select(dc1_ind, QItemSelectionModel.ClearAndSelect)
-        with mock.patch.object(spinetoolbox.ui_main.QMessageBox, "exec_") as mock_message_box_exec:
-            mock_message_box_exec.return_value = QMessageBox.Ok
+        with mock.patch.object(spinetoolbox.ui_main.QMessageBox, "exec") as mock_message_box_exec:
+            mock_message_box_exec.return_value = QMessageBox.StandardButton.Ok
             self.toolbox.ui.actionRemove.trigger()
         self.assertEqual(self.toolbox.project_item_model.n_items(), 0)  # Check the number of project items
         dags = self.toolbox.project().dags()
@@ -671,8 +688,8 @@ class TestToolboxUI(unittest.TestCase):
         # Now, remove the Tool Spec from the model
         index = self.toolbox.specification_model.specification_index("Python Tool Specification")
         self.assertTrue(index.isValid())
-        with mock.patch.object(spinetoolbox.ui_main.QMessageBox, "exec_") as mock_message_box_exec:
-            mock_message_box_exec.return_value = QMessageBox.Ok
+        with mock.patch.object(spinetoolbox.ui_main.QMessageBox, "exec") as mock_message_box_exec:
+            mock_message_box_exec.return_value = QMessageBox.StandardButton.Ok
             self.toolbox.remove_specification(index)
         # Tool spec model must be empty again
         self.assertEqual(0, self.toolbox.specification_model.rowCount())
@@ -694,18 +711,6 @@ class TestToolboxUI(unittest.TestCase):
             mock_qsettings_value.assert_called_once()
             mock_qsettings_value.assert_called_with("appSettings/showExitPrompt", defaultValue="2")
         self.assertEqual(tasks, ["prompt exit"])
-        with mock.patch("spinetoolbox.ui_main.QSettings.value") as mock_qsettings_value:
-            mock_qsettings_value.side_effect = self._tasks_before_exit_scenario_3
-            tasks = self.toolbox._tasks_before_exit()
-            mock_qsettings_value.assert_called_once()
-            mock_qsettings_value.assert_called_with("appSettings/showExitPrompt", defaultValue="2")
-        self.assertEqual(tasks, [])
-        with mock.patch("spinetoolbox.ui_main.QSettings.value") as mock_qsettings_value:
-            mock_qsettings_value.side_effect = self._tasks_before_exit_scenario_4
-            tasks = self.toolbox._tasks_before_exit()
-            mock_qsettings_value.assert_called_once()
-            mock_qsettings_value.assert_called_with("appSettings/showExitPrompt", defaultValue="2")
-        self.assertEqual(tasks, ["prompt exit"])
         with mock.patch("spinetoolbox.ui_main.QSettings.value") as mock_qsettings_value:
             mock_qsettings_value.side_effect = self._tasks_before_exit_scenario_5
             tasks = self.toolbox._tasks_before_exit()
@@ -728,22 +733,12 @@ class TestToolboxUI(unittest.TestCase):
         with mock.patch("spinetoolbox.ui_main.QSettings.value") as mock_qsettings_value:
             mock_qsettings_value.side_effect = self._tasks_before_exit_scenario_1
             tasks = self.toolbox._tasks_before_exit()
-            self.assertEqual(2, mock_qsettings_value.call_count)
-        self.assertEqual(tasks, [])
+            self.assertEqual(1, mock_qsettings_value.call_count)
+        self.assertEqual(tasks, ["prompt save"])
         with mock.patch("spinetoolbox.ui_main.QSettings.value") as mock_qsettings_value:
             mock_qsettings_value.side_effect = self._tasks_before_exit_scenario_2
             tasks = self.toolbox._tasks_before_exit()
-            self.assertEqual(2, mock_qsettings_value.call_count)
-        self.assertEqual(tasks, ["prompt exit"])
-        with mock.patch("spinetoolbox.ui_main.QSettings.value") as mock_qsettings_value:
-            mock_qsettings_value.side_effect = self._tasks_before_exit_scenario_3
-            tasks = self.toolbox._tasks_before_exit()
-            self.assertEqual(2, mock_qsettings_value.call_count)
-        self.assertEqual(tasks, ["prompt save"])
-        with mock.patch("spinetoolbox.ui_main.QSettings.value") as mock_qsettings_value:
-            mock_qsettings_value.side_effect = self._tasks_before_exit_scenario_4
-            tasks = self.toolbox._tasks_before_exit()
-            self.assertEqual(2, mock_qsettings_value.call_count)
+            self.assertEqual(1, mock_qsettings_value.call_count)
         self.assertEqual(tasks, ["prompt save"])
         with mock.patch("spinetoolbox.ui_main.QSettings.value") as mock_qsettings_value:
             mock_qsettings_value.side_effect = self._tasks_before_exit_scenario_5
@@ -933,42 +928,78 @@ class TestToolboxUI(unittest.TestCase):
         if key == "appSettings/showExitPrompt":
             return "0"
         if key == "appSettings/saveAtExit":
-            return "0"
+            return "prompt"
 
     @staticmethod
     def _tasks_before_exit_scenario_2(key, defaultValue="2"):
         if key == "appSettings/showExitPrompt":
             return "2"
         if key == "appSettings/saveAtExit":
-            return "0"
-
-    @staticmethod
-    def _tasks_before_exit_scenario_3(key, defaultValue="2"):
-        if key == "appSettings/showExitPrompt":
-            return "0"
-        if key == "appSettings/saveAtExit":
-            return "1"
-
-    @staticmethod
-    def _tasks_before_exit_scenario_4(key, defaultValue="2"):
-        if key == "appSettings/showExitPrompt":
-            return "2"
-        if key == "appSettings/saveAtExit":
-            return "1"
+            return "prompt"
 
     @staticmethod
     def _tasks_before_exit_scenario_5(key, defaultValue="2"):
         if key == "appSettings/showExitPrompt":
             return "0"
         if key == "appSettings/saveAtExit":
-            return "2"
+            return "automatic"
 
     @staticmethod
     def _tasks_before_exit_scenario_6(key, defaultValue="2"):
         if key == "appSettings/showExitPrompt":
             return "2"
         if key == "appSettings/saveAtExit":
-            return "2"
+            return "automatic"
+
+
+class TestToolboxUIWithTestSettings(unittest.TestCase):
+    @classmethod
+    def setUpClass(cls):
+        if not QApplication.instance():
+            QApplication()
+
+    def test_legacy_settings_keys_get_renamed(self):
+        settings_dict = {"appSettings/useEmbeddedJulia": "julia value", "appSettings/useEmbeddedPython": "python value"}
+        with toolbox_with_settings(settings_dict) as toolbox:
+            settings = toolbox.qsettings()
+            self.assertTrue(settings.contains("appSettings/useJuliaKernel"))
+            self.assertTrue(settings.contains("appSettings/usePythonKernel"))
+            self.assertEqual(settings.value("appSettings/useJuliaKernel"), "julia value")
+            self.assertEqual(settings.value("appSettings/usePythonKernel"), "python value")
+
+    def test_legacy_saveAtExit_value_0_is_updated_to_prompt(self):
+        settings_dict = {"appSettings/saveAtExit": "0"}
+        with toolbox_with_settings(settings_dict) as toolbox:
+            settings = toolbox.qsettings()
+            self.assertEqual(settings.value("appSettings/saveAtExit"), "prompt")
+
+    def test_legacy_saveAtExit_value_1_is_updated_to_prompt(self):
+        settings_dict = {"appSettings/saveAtExit": "1"}
+        with toolbox_with_settings(settings_dict) as toolbox:
+            settings = toolbox.qsettings()
+            self.assertEqual(settings.value("appSettings/saveAtExit"), "prompt")
+
+    def test_legacy_saveAtExit_value_2_is_updated_to_automatic(self):
+        settings_dict = {"appSettings/saveAtExit": "2"}
+        with toolbox_with_settings(settings_dict) as toolbox:
+            settings = toolbox.qsettings()
+            self.assertEqual(settings.value("appSettings/saveAtExit"), "automatic")
+
+
+@contextmanager
+def toolbox_with_settings(settings_dict):
+    settings = QSettings("SpineProject", "Spine Toolbox tests")
+    for key, value in settings_dict.items():
+        settings.setValue(key, value)
+    with mock.patch("spinetoolbox.ui_main.QSettings") as settings_constructor:
+        settings_constructor.return_value = settings
+        toolbox = create_toolboxui()
+    try:
+        yield toolbox
+    finally:
+        settings.clear()
+        settings.deleteLater()
+        clean_up_toolbox(toolbox)
 
 
 if __name__ == '__main__':
